@@ -9,22 +9,25 @@
 #include "jsapi.h"
 #include "nsCOMPtr.h"
 #include "nsPIDOMWindow.h"
-#include "nsIDocument.h"
-#include "nsIPresShell.h"
-#include "nsPresContext.h"
+#include "mozilla/dom/Document.h"
+#include "mozilla/dom/DocumentInlines.h"
 #include "nsIDocShell.h"
 #include "nsIWebNavigation.h"
-#include "nsIURI.h"
-#include "nsIInterfaceRequestorUtils.h"
 #include "nsReadableUtils.h"
 #include "nsContentUtils.h"
-#include "nsISHistory.h"
-#include "nsISHistoryInternal.h"
-#include "mozilla/Preferences.h"
+#include "mozilla/dom/WindowContext.h"
+#include "mozilla/dom/Location.h"
+#include "mozilla/RefPtr.h"
+#include "mozilla/StaticPrefs_dom.h"
+#include "mozilla/BasePrincipal.h"
 
 using namespace mozilla;
 using namespace mozilla::dom;
 
+extern LazyLogModule gSHistoryLog;
+
+#define LOG(format) MOZ_LOG(gSHistoryLog, mozilla::LogLevel::Debug, format)
+
 //
 //  History class implementation
 //
@@ -39,7 +42,7 @@ NS_INTERFACE_MAP_END
 nsHistory::nsHistory(nsPIDOMWindowInner* aInnerWindow)
     : mInnerWindow(do_GetWeakReference(aInnerWindow)) {}
 
-nsHistory::~nsHistory() {}
+nsHistory::~nsHistory() = default;
 
 nsPIDOMWindowInner* nsHistory::GetParentObject() const {
   nsCOMPtr<nsPIDOMWindowInner> win(do_QueryReferent(mInnerWindow));
@@ -48,7 +51,7 @@ nsPIDOMWindowInner* nsHistory::GetParentObject() const {
 
 JSObject* nsHistory::WrapObject(JSContext* aCx,
                                 JS::Handle<JSObject*> aGivenProto) {
-  return HistoryBinding::Wrap(aCx, this, aGivenProto);
+  return History_Binding::Wrap(aCx, this, aGivenProto);
 }
 
 uint32_t nsHistory::GetLength(ErrorResult& aRv) const {
@@ -60,26 +63,16 @@ uint32_t nsHistory::GetLength(ErrorResult& aRv) const {
   }
 
   // Get session History from docshell
-  nsCOMPtr<nsISHistory> sHistory = GetSessionHistory();
+  RefPtr<ChildSHistory> sHistory = GetSessionHistory();
   if (!sHistory) {
-    aRv.Throw(NS_ERROR_FAILURE);
-
-    return 0;
-  }
-
-  int32_t len;
-  nsresult rv = sHistory->GetCount(&len);
-
-  if (NS_FAILED(rv)) {
-    aRv.Throw(rv);
-
-    return 0;
+    return 1;
   }
 
+  int32_t len = sHistory->Count();
   return len >= 0 ? len : 0;
 }
 
-ScrollRestoration nsHistory::GetScrollRestoration(mozilla::ErrorResult& aRv) {
+ScrollRestoration nsHistory::GetScrollRestoration(mozilla::dom::CallerType aCallerType, mozilla::ErrorResult& aRv) {
   nsCOMPtr<nsPIDOMWindowInner> win(do_QueryReferent(mInnerWindow));
   if (!win || !win->HasActiveDocument() || !win->GetDocShell()) {
     aRv.Throw(NS_ERROR_DOM_SECURITY_ERR);
@@ -95,6 +88,7 @@ ScrollRestoration nsHistory::GetScrollRestoration(mozilla::ErrorResult& aRv) {
 }
 
 void nsHistory::SetScrollRestoration(mozilla::dom::ScrollRestoration aMode,
+                                     mozilla::dom::CallerType aCallerType,
                                      mozilla::ErrorResult& aRv) {
   nsCOMPtr<nsPIDOMWindowInner> win(do_QueryReferent(mInnerWindow));
   if (!win || !win->HasActiveDocument() || !win->GetDocShell()) {
@@ -102,6 +96,15 @@ void nsHistory::SetScrollRestoration(mozilla::dom::ScrollRestoration aMode,
     return;
   }
 
+  BrowsingContext* bc = win->GetBrowsingContext();
+  if (bc) {
+    nsresult rv = bc->CheckNavigationRateLimit(aCallerType);
+    if (NS_FAILED(rv)) {
+      aRv.Throw(rv);
+      return;
+    }
+  }
+
   win->GetDocShell()->SetCurrentScrollRestorationIsManual(
       aMode == mozilla::dom::ScrollRestoration::Manual);
 }
@@ -119,88 +122,52 @@ void nsHistory::GetState(JSContext* aCx, JS::MutableHandle<JS::Value> aResult,
     return;
   }
 
-  nsCOMPtr<nsIDocument> doc = do_QueryInterface(win->GetExtantDoc());
+  nsCOMPtr<Document> doc = win->GetExtantDoc();
   if (!doc) {
     aRv.Throw(NS_ERROR_NOT_AVAILABLE);
     return;
   }
 
-  nsCOMPtr<nsIVariant> variant;
-  doc->GetStateObject(getter_AddRefs(variant));
-
-  if (variant) {
-    aRv = variant->GetAsJSVal(aResult);
-
-    if (aRv.Failed()) {
-      return;
-    }
-
-    if (!JS_WrapValue(aCx, aResult)) {
-      aRv.Throw(NS_ERROR_OUT_OF_MEMORY);
-    }
-
-    return;
-  }
-
-  aResult.setNull();
+  aRv = doc->GetStateObject(aResult);
 }
 
-void nsHistory::Go(int32_t aDelta, ErrorResult& aRv) {
+void nsHistory::Go(int32_t aDelta, nsIPrincipal& aSubjectPrincipal,
+                   ErrorResult& aRv) {
+  LOG(("nsHistory::Go(%d)", aDelta));
   nsCOMPtr<nsPIDOMWindowInner> win(do_QueryReferent(mInnerWindow));
   if (!win || !win->HasActiveDocument()) {
-    aRv.Throw(NS_ERROR_DOM_SECURITY_ERR);
-
-    return;
+    return aRv.Throw(NS_ERROR_DOM_SECURITY_ERR);
   }
 
   if (!aDelta) {
-    nsCOMPtr<nsPIDOMWindowOuter> window;
-    if (nsIDocShell* docShell = GetDocShell()) {
-      window = docShell->GetWindow();
-    }
-
-    if (window && window->IsHandlingResizeEvent()) {
-      // history.go(0) (aka location.reload()) was called on a window
-      // that is handling a resize event. Sites do this since Netscape
-      // 4.x needed it, but we don't, and it's a horrible experience
-      // for nothing.  In stead of reloading the page, just clear
-      // style data and reflow the page since some sites may use this
-      // trick to work around gecko reflow bugs, and this should have
-      // the same effect.
-
-      nsCOMPtr<nsIDocument> doc = window->GetExtantDoc();
-
-      nsPresContext* pcx;
-      if (doc && (pcx = doc->GetPresContext())) {
-        pcx->RebuildAllStyleData(NS_STYLE_HINT_REFLOW, eRestyle_Subtree);
-      }
-
-      return;
-    }
+    // https://html.spec.whatwg.org/multipage/history.html#the-history-interface
+    // "When the go(delta) method is invoked, if delta is zero, the user agent
+    // must act as if the location.reload() method was called instead."
+    RefPtr<Location> location = win->Location();
+    return location->Reload(false, aSubjectPrincipal, aRv);
   }
 
-  nsCOMPtr<nsISHistory> session_history = GetSessionHistory();
-  nsCOMPtr<nsIWebNavigation> webnav(do_QueryInterface(session_history));
-  if (!webnav) {
+  RefPtr<ChildSHistory> session_history = GetSessionHistory();
+  if (!session_history) {
     aRv.Throw(NS_ERROR_FAILURE);
-
     return;
   }
 
-  int32_t curIndex = -1;
-  int32_t len = 0;
-  session_history->GetIndex(&curIndex);
-  session_history->GetCount(&len);
+  bool userActivation =
+      win->GetWindowContext()
+          ? win->GetWindowContext()->HasValidTransientUserGestureActivation()
+          : false;
 
-  int32_t index = curIndex + aDelta;
-  if (index > -1 && index < len) webnav->GotoIndex(index);
+  CallerType callerType = aSubjectPrincipal.IsSystemPrincipal()
+                              ? CallerType::System
+                              : CallerType::NonSystem;
 
-  // Ignore the return value from GotoIndex(), since returning errors
-  // from GotoIndex() can lead to exceptions and a possible leak
-  // of history length
+  // AsyncGo throws if we hit the navigation rate limit.
+  session_history->AsyncGo(aDelta, /* aRequireUserInteraction = */ false,
+                           userActivation, callerType, aRv);
 }
 
-void nsHistory::Back(ErrorResult& aRv) {
+void nsHistory::Back(CallerType aCallerType, ErrorResult& aRv) {
   nsCOMPtr<nsPIDOMWindowInner> win(do_QueryReferent(mInnerWindow));
   if (!win || !win->HasActiveDocument()) {
     aRv.Throw(NS_ERROR_DOM_SECURITY_ERR);
@@ -208,18 +175,23 @@ void nsHistory::Back(ErrorResult& aRv) {
     return;
   }
 
-  nsCOMPtr<nsISHistory> sHistory = GetSessionHistory();
-  nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(sHistory));
-  if (!webNav) {
+  RefPtr<ChildSHistory> sHistory = GetSessionHistory();
+  if (!sHistory) {
     aRv.Throw(NS_ERROR_FAILURE);
 
     return;
   }
 
-  webNav->GoBack();
+  bool userActivation =
+      win->GetWindowContext()
+          ? win->GetWindowContext()->HasValidTransientUserGestureActivation()
+          : false;
+
+  sHistory->AsyncGo(-1, /* aRequireUserInteraction = */ false, userActivation,
+                    aCallerType, aRv);
 }
 
-void nsHistory::Forward(ErrorResult& aRv) {
+void nsHistory::Forward(CallerType aCallerType, ErrorResult& aRv) {
   nsCOMPtr<nsPIDOMWindowInner> win(do_QueryReferent(mInnerWindow));
   if (!win || !win->HasActiveDocument()) {
     aRv.Throw(NS_ERROR_DOM_SECURITY_ERR);
@@ -227,32 +199,38 @@ void nsHistory::Forward(ErrorResult& aRv) {
     return;
   }
 
-  nsCOMPtr<nsISHistory> sHistory = GetSessionHistory();
-  nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(sHistory));
-  if (!webNav) {
+  RefPtr<ChildSHistory> sHistory = GetSessionHistory();
+  if (!sHistory) {
     aRv.Throw(NS_ERROR_FAILURE);
 
     return;
   }
 
-  webNav->GoForward();
+  bool userActivation =
+      win->GetWindowContext()
+          ? win->GetWindowContext()->HasValidTransientUserGestureActivation()
+          : false;
+
+  sHistory->AsyncGo(1, /* aRequireUserInteraction = */ false, userActivation,
+                    aCallerType, aRv);
 }
 
 void nsHistory::PushState(JSContext* aCx, JS::Handle<JS::Value> aData,
                           const nsAString& aTitle, const nsAString& aUrl,
-                          ErrorResult& aRv) {
-  PushOrReplaceState(aCx, aData, aTitle, aUrl, aRv, false);
+                          CallerType aCallerType, ErrorResult& aRv) {
+  PushOrReplaceState(aCx, aData, aTitle, aUrl, aCallerType, aRv, false);
 }
 
 void nsHistory::ReplaceState(JSContext* aCx, JS::Handle<JS::Value> aData,
                              const nsAString& aTitle, const nsAString& aUrl,
-                             ErrorResult& aRv) {
-  PushOrReplaceState(aCx, aData, aTitle, aUrl, aRv, true);
+                             CallerType aCallerType, ErrorResult& aRv) {
+  PushOrReplaceState(aCx, aData, aTitle, aUrl, aCallerType, aRv, true);
 }
 
 void nsHistory::PushOrReplaceState(JSContext* aCx, JS::Handle<JS::Value> aData,
                                    const nsAString& aTitle,
-                                   const nsAString& aUrl, ErrorResult& aRv,
+                                   const nsAString& aUrl,
+                                   CallerType aCallerType, ErrorResult& aRv,
                                    bool aReplace) {
   nsCOMPtr<nsPIDOMWindowInner> win(do_QueryReferent(mInnerWindow));
   if (!win) {
@@ -267,6 +245,15 @@ void nsHistory::PushOrReplaceState(JSContext* aCx, JS::Handle<JS::Value> aData,
     return;
   }
 
+  BrowsingContext* bc = win->GetBrowsingContext();
+  if (bc) {
+    nsresult rv = bc->CheckNavigationRateLimit(aCallerType);
+    if (NS_FAILED(rv)) {
+      aRv.Throw(rv);
+      return;
+    }
+  }
+
   // AddState might run scripts, so we need to hold a strong reference to the
   // docShell here to keep it from going away.
   nsCOMPtr<nsIDocShell> docShell = win->GetDocShell();
@@ -283,28 +270,13 @@ void nsHistory::PushOrReplaceState(JSContext* aCx, JS::Handle<JS::Value> aData,
   aRv = docShell->AddState(aData, aTitle, aUrl, aReplace, aCx);
 }
 
-nsIDocShell* nsHistory::GetDocShell() const {
+already_AddRefed<ChildSHistory> nsHistory::GetSessionHistory() const {
   nsCOMPtr<nsPIDOMWindowInner> win = do_QueryReferent(mInnerWindow);
-  if (!win) {
-    return nullptr;
-  }
-  return win->GetDocShell();
-}
+  NS_ENSURE_TRUE(win, nullptr);
 
-already_AddRefed<nsISHistory> nsHistory::GetSessionHistory() const {
-  nsIDocShell* docShell = GetDocShell();
-  NS_ENSURE_TRUE(docShell, nullptr);
+  BrowsingContext* bc = win->GetBrowsingContext();
+  NS_ENSURE_TRUE(bc, nullptr);
 
-  // Get the root DocShell from it
-  nsCOMPtr<nsIDocShellTreeItem> root;
-  docShell->GetSameTypeRootTreeItem(getter_AddRefs(root));
-  nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(root));
-  NS_ENSURE_TRUE(webNav, nullptr);
-
-  nsCOMPtr<nsISHistory> shistory;
-
-  // Get SH from nsIWebNavigation
-  webNav->GetSessionHistory(getter_AddRefs(shistory));
-
-  return shistory.forget();
+  RefPtr<ChildSHistory> childSHistory = bc->Top()->GetChildSessionHistory();
+  return childSHistory.forget();
 }
