@@ -8,11 +8,14 @@
 #define mozilla_dom_ImageBitmap_h
 
 #include "mozilla/Attributes.h"
+#include "mozilla/SurfaceFromElementResult.h"
+#include "mozilla/dom/ImageBitmapBinding.h"
 #include "mozilla/dom/ImageBitmapSource.h"
 #include "mozilla/dom/TypedArray.h"
 #include "mozilla/gfx/Rect.h"
 #include "mozilla/Maybe.h"
 #include "mozilla/UniquePtr.h"
+#include "ImageData.h"
 #include "gfxTypes.h"  // for gfxAlphaType
 #include "nsCycleCollectionParticipant.h"
 
@@ -41,28 +44,26 @@ class OffscreenCanvas;
 
 class ArrayBufferViewOrArrayBuffer;
 class CanvasRenderingContext2D;
-struct ChannelPixelLayout;
 class CreateImageBitmapFromBlob;
 class CreateImageBitmapFromBlobTask;
 class CreateImageBitmapFromBlobWorkerTask;
+class ImageBitmapShutdownObserver;
 class File;
 class HTMLCanvasElement;
 class HTMLImageElement;
 class HTMLVideoElement;
-enum class ImageBitmapFormat : uint8_t;
 class ImageData;
 class ImageUtils;
-template <typename T>
-class MapDataIntoBufferSource;
 class Promise;
 class PostMessageEvent;  // For StructuredClone between windows.
-class ImageBitmapShutdownObserver;
+class SVGImageElement;
+class VideoFrame;
+class SendShutdownToWorkerThread;
 
 struct ImageBitmapCloneData final {
   RefPtr<gfx::DataSourceSurface> mSurface;
   gfx::IntRect mPictureRect;
   gfxAlphaType mAlphaType;
-  bool mIsCroppingAreaOutSideOfSourceImage;
   bool mWriteOnly;
 };
 
@@ -81,7 +82,7 @@ struct ImageBitmapCloneData final {
 class ImageBitmap final : public nsISupports, public nsWrapperCache {
  public:
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
-  NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS(ImageBitmap)
+  NS_DECL_CYCLE_COLLECTION_WRAPPERCACHE_CLASS(ImageBitmap)
 
   nsCOMPtr<nsIGlobalObject> GetParentObject() const { return mParent; }
 
@@ -94,6 +95,8 @@ class ImageBitmap final : public nsISupports, public nsWrapperCache {
 
   void Close();
 
+  SurfaceFromElementResult SurfaceFrom(uint32_t aSurfaceFlags);
+
   /*
    * The PrepareForDrawTarget() might return null if the mPictureRect does not
    * intersect with the size of mData.
@@ -107,8 +110,12 @@ class ImageBitmap final : public nsISupports, public nsWrapperCache {
    */
   already_AddRefed<layers::Image> TransferAsImage();
 
+  // This method returns null if the image has been already closed.
   UniquePtr<ImageBitmapCloneData> ToCloneData() const;
 
+  static already_AddRefed<ImageBitmap> CreateFromSourceSurface(
+      nsIGlobalObject* aGlobal, gfx::SourceSurface* aSource, ErrorResult& aRv);
+
   static already_AddRefed<ImageBitmap> CreateFromCloneData(
       nsIGlobalObject* aGlobal, ImageBitmapCloneData* aData);
 
@@ -119,52 +126,31 @@ class ImageBitmap final : public nsISupports, public nsWrapperCache {
   static already_AddRefed<Promise> Create(nsIGlobalObject* aGlobal,
                                           const ImageBitmapSource& aSrc,
                                           const Maybe<gfx::IntRect>& aCropRect,
+                                          const ImageBitmapOptions& aOptions,
                                           ErrorResult& aRv);
 
-  static already_AddRefed<Promise> Create(
-      nsIGlobalObject* aGlobal, const ImageBitmapSource& aBuffer,
-      int32_t aOffset, int32_t aLength, mozilla::dom::ImageBitmapFormat aFormat,
-      const Sequence<mozilla::dom::ChannelPixelLayout>& aLayout,
-      ErrorResult& aRv);
-
   static JSObject* ReadStructuredClone(
       JSContext* aCx, JSStructuredCloneReader* aReader,
       nsIGlobalObject* aParent,
       const nsTArray<RefPtr<gfx::DataSourceSurface>>& aClonedSurfaces,
       uint32_t aIndex);
 
-  static bool WriteStructuredClone(
+  static void WriteStructuredClone(
       JSStructuredCloneWriter* aWriter,
       nsTArray<RefPtr<gfx::DataSourceSurface>>& aClonedSurfaces,
-      ImageBitmap* aImageBitmap);
+      ImageBitmap* aImageBitmap, ErrorResult& aRv);
 
   friend CreateImageBitmapFromBlob;
   friend CreateImageBitmapFromBlobTask;
   friend CreateImageBitmapFromBlobWorkerTask;
-
-  template <typename T>
-  friend class MapDataIntoBufferSource;
-
-  // Mozilla Extensions
-  ImageBitmapFormat FindOptimalFormat(
-      const Optional<Sequence<ImageBitmapFormat>>& aPossibleFormats,
-      ErrorResult& aRv);
-
-  int32_t MappedDataLength(ImageBitmapFormat aFormat, ErrorResult& aRv);
-
-  already_AddRefed<Promise> MapDataInto(
-      JSContext* aCx, ImageBitmapFormat aFormat,
-      const ArrayBufferViewOrArrayBuffer& aBuffer, int32_t aOffset,
-      ErrorResult& aRv);
+  friend ImageBitmapShutdownObserver;
 
   size_t GetAllocatedSize() const;
 
   void OnShutdown();
 
-  bool IsWriteOnly() const
-  {
-    return mWriteOnly;
-  }
+  bool IsWriteOnly() const { return mWriteOnly; }
+  bool IsClosed() const { return !mData; };
 
  protected:
   /*
@@ -186,41 +172,63 @@ class ImageBitmap final : public nsISupports, public nsWrapperCache {
    * the aIsPremultipliedAlpha to be false in the
    * CreateInternal(from ImageData) method.
    */
-  ImageBitmap(nsIGlobalObject* aGlobal, layers::Image* aData,
-              bool aWriteOnly,
+  ImageBitmap(nsIGlobalObject* aGlobal, layers::Image* aData, bool aWriteOnly,
               gfxAlphaType aAlphaType = gfxAlphaType::Premult);
 
   virtual ~ImageBitmap();
 
   void SetPictureRect(const gfx::IntRect& aRect, ErrorResult& aRv);
 
-  void SetIsCroppingAreaOutSideOfSourceImage(
-      const gfx::IntSize& aSourceSize,
-      const Maybe<gfx::IntRect>& aCroppingRect);
+  static already_AddRefed<ImageBitmap> CreateImageBitmapInternal(
+      nsIGlobalObject* aGlobal, gfx::SourceSurface* aSurface,
+      const Maybe<gfx::IntRect>& aCropRect, const ImageBitmapOptions& aOptions,
+      const bool aWriteOnly, const bool aAllocatedImageData,
+      const bool aMustCopy, const gfxAlphaType aAlphaType, ErrorResult& aRv);
 
   static already_AddRefed<ImageBitmap> CreateInternal(
       nsIGlobalObject* aGlobal, HTMLImageElement& aImageEl,
-      const Maybe<gfx::IntRect>& aCropRect, ErrorResult& aRv);
+      const Maybe<gfx::IntRect>& aCropRect, const ImageBitmapOptions& aOptions,
+      ErrorResult& aRv);
+
+  static already_AddRefed<ImageBitmap> CreateInternal(
+      nsIGlobalObject* aGlobal, SVGImageElement& aImageEl,
+      const Maybe<gfx::IntRect>& aCropRect, const ImageBitmapOptions& aOptions,
+      ErrorResult& aRv);
 
   static already_AddRefed<ImageBitmap> CreateInternal(
       nsIGlobalObject* aGlobal, HTMLVideoElement& aVideoEl,
-      const Maybe<gfx::IntRect>& aCropRect, ErrorResult& aRv);
+      const Maybe<gfx::IntRect>& aCropRect, const ImageBitmapOptions& aOptions,
+      ErrorResult& aRv);
 
   static already_AddRefed<ImageBitmap> CreateInternal(
       nsIGlobalObject* aGlobal, HTMLCanvasElement& aCanvasEl,
-      const Maybe<gfx::IntRect>& aCropRect, ErrorResult& aRv);
+      const Maybe<gfx::IntRect>& aCropRect, const ImageBitmapOptions& aOptions,
+      ErrorResult& aRv);
+
+  static already_AddRefed<ImageBitmap> CreateInternal(
+      nsIGlobalObject* aGlobal, OffscreenCanvas& aOffscreenCanvas,
+      const Maybe<gfx::IntRect>& aCropRect, const ImageBitmapOptions& aOptions,
+      ErrorResult& aRv);
 
   static already_AddRefed<ImageBitmap> CreateInternal(
       nsIGlobalObject* aGlobal, ImageData& aImageData,
-      const Maybe<gfx::IntRect>& aCropRect, ErrorResult& aRv);
+      const Maybe<gfx::IntRect>& aCropRect, const ImageBitmapOptions& aOptions,
+      ErrorResult& aRv);
 
   static already_AddRefed<ImageBitmap> CreateInternal(
       nsIGlobalObject* aGlobal, CanvasRenderingContext2D& aCanvasCtx,
-      const Maybe<gfx::IntRect>& aCropRect, ErrorResult& aRv);
+      const Maybe<gfx::IntRect>& aCropRect, const ImageBitmapOptions& aOptions,
+      ErrorResult& aRv);
 
   static already_AddRefed<ImageBitmap> CreateInternal(
       nsIGlobalObject* aGlobal, ImageBitmap& aImageBitmap,
-      const Maybe<gfx::IntRect>& aCropRect, ErrorResult& aRv);
+      const Maybe<gfx::IntRect>& aCropRect, const ImageBitmapOptions& aOptions,
+      ErrorResult& aRv);
+
+  static already_AddRefed<ImageBitmap> CreateInternal(
+      nsIGlobalObject* aGlobal, VideoFrame& aVideoFrame,
+      const Maybe<gfx::IntRect>& aCropRect, const ImageBitmapOptions& aOptions,
+      ErrorResult& aRv);
 
   nsCOMPtr<nsIGlobalObject> mParent;
 
@@ -241,13 +249,6 @@ class ImageBitmap final : public nsISupports, public nsWrapperCache {
   RefPtr<layers::Image> mData;
   RefPtr<gfx::SourceSurface> mSurface;
 
-  /*
-   * This is used in the ImageBitmap-Extensions implementation.
-   * ImageUtils is a wrapper to layers::Image, which add some common methods for
-   * accessing the layers::Image's data.
-   */
-  UniquePtr<ImageUtils> mDataWrapper;
-
   /*
    * The mPictureRect is the size of the source image in default, however, if
    * users specify the cropping area while creating an ImageBitmap, then this
@@ -261,18 +262,9 @@ class ImageBitmap final : public nsISupports, public nsWrapperCache {
    */
   gfx::IntRect mPictureRect;
 
-  const gfxAlphaType mAlphaType;
+  gfxAlphaType mAlphaType;
 
-  RefPtr<ImageBitmapShutdownObserver> mShutdownObserver;
-
-  /*
-   * Set mIsCroppingAreaOutSideOfSourceImage if image bitmap was cropped to the
-   * source rectangle so that it contains any transparent black pixels (cropping
-   * area is outside of the source image).
-   * This is used in mapDataInto() to check if we should reject promise with
-   * IndexSizeError.
-   */
-  bool mIsCroppingAreaOutSideOfSourceImage;
+  RefPtr<SendShutdownToWorkerThread> mShutdownRunnable;
 
   /*
    * Whether this object allocated allocated and owns the image data.
