@@ -8,7 +8,8 @@
 #define mozilla_dom_HTMLFormSubmission_h
 
 #include "mozilla/Attributes.h"
-#include "mozilla/dom/Element.h"
+#include "mozilla/dom/UserActivation.h"
+#include "mozilla/dom/HTMLDialogElement.h"
 #include "nsCOMPtr.h"
 #include "mozilla/Encoding.h"
 #include "nsString.h"
@@ -18,11 +19,12 @@ class nsIInputStream;
 class nsGenericHTMLElement;
 class nsIMultiplexInputStream;
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 class Blob;
+class DialogFormSubmission;
 class Directory;
+class Element;
 class HTMLFormElement;
 
 /**
@@ -36,14 +38,16 @@ class HTMLFormSubmission {
    * METHOD)
    *
    * @param aForm the form to get a submission object based on
-   * @param aOriginatingElement the originating element (can be null)
+   * @param aSubmitter the submitter element (can be null)
+   * @param aEncoding the submiter element's encoding
    * @param aFormSubmission the form submission object (out param)
    */
   static nsresult GetFromForm(HTMLFormElement* aForm,
-                              nsGenericHTMLElement* aOriginatingElement,
+                              nsGenericHTMLElement* aSubmitter,
+                              NotNull<const Encoding*>& aEncoding,
                               HTMLFormSubmission** aFormSubmission);
 
-  virtual ~HTMLFormSubmission() { MOZ_COUNT_DTOR(HTMLFormSubmission); }
+  MOZ_COUNTED_DTOR_VIRTUAL(HTMLFormSubmission)
 
   /**
    * Submit a name/value pair
@@ -59,11 +63,10 @@ class HTMLFormSubmission {
    *
    * @param aName the name of the parameter
    * @param aBlob the blob to submit. The file's name will be used if the Blob
-   * is actually a File, otherwise 'blob' string is used instead if the aBlob is
-   * not null.
+   * is actually a File, otherwise 'blob' string is used instead. Must not be
+   * null.
    */
-  virtual nsresult AddNameBlobOrNullPair(const nsAString& aName,
-                                         Blob* aBlob) = 0;
+  virtual nsresult AddNameBlobPair(const nsAString& aName, Blob* aBlob) = 0;
 
   /**
    * Submit a name/directory pair
@@ -80,12 +83,10 @@ class HTMLFormSubmission {
    *
    * @param aURI the URI being submitted to [IN]
    * @param aPostDataStream a data stream for POST data [OUT]
-   * @param aPostDataStreamLength a data stream for POST data length [OUT]
    * @param aOutURI the resulting URI. May be the same as aURI [OUT]
    */
   virtual nsresult GetEncodedSubmission(nsIURI* aURI,
                                         nsIInputStream** aPostDataStream,
-                                        int64_t* aPostDataStreamLength,
                                         nsCOMPtr<nsIURI>& aOutURI) = 0;
 
   /**
@@ -93,8 +94,6 @@ class HTMLFormSubmission {
    */
   void GetCharset(nsACString& aCharset) { mEncoding->Name(aCharset); }
 
-  Element* GetOriginatingElement() const { return mOriginatingElement.get(); }
-
   /**
    * Get the action URI that will be used for submission.
    */
@@ -105,22 +104,21 @@ class HTMLFormSubmission {
    */
   void GetTarget(nsAString& aTarget) { aTarget = mTarget; }
 
+  /**
+   * Return true if this form submission was user-initiated.
+   */
+  bool IsInitiatedFromUserInput() const { return mInitiatedFromUserInput; }
+
+  virtual DialogFormSubmission* GetAsDialogSubmission() { return nullptr; }
+
  protected:
   /**
    * Can only be constructed by subclasses.
    *
    * @param aEncoding the character encoding of the form
-   * @param aOriginatingElement the originating element (can be null)
    */
   HTMLFormSubmission(nsIURI* aActionURL, const nsAString& aTarget,
-                     mozilla::NotNull<const mozilla::Encoding*> aEncoding,
-                     Element* aOriginatingElement)
-      : mActionURL(aActionURL),
-        mTarget(aTarget),
-        mEncoding(aEncoding),
-        mOriginatingElement(aOriginatingElement) {
-    MOZ_COUNT_CTOR(HTMLFormSubmission);
-  }
+                     mozilla::NotNull<const mozilla::Encoding*> aEncoding);
 
   // The action url.
   nsCOMPtr<nsIURI> mActionURL;
@@ -131,29 +129,83 @@ class HTMLFormSubmission {
   // The character encoding of this form submission
   mozilla::NotNull<const mozilla::Encoding*> mEncoding;
 
-  // Originating element.
-  RefPtr<Element> mOriginatingElement;
+  // Keep track of whether this form submission was user-initiated or not
+  bool mInitiatedFromUserInput;
 };
 
 class EncodingFormSubmission : public HTMLFormSubmission {
  public:
   EncodingFormSubmission(nsIURI* aActionURL, const nsAString& aTarget,
                          mozilla::NotNull<const mozilla::Encoding*> aEncoding,
-                         Element* aOriginatingElement);
+                         Element* aSubmitter);
 
   virtual ~EncodingFormSubmission();
 
+  // Indicates the type of newline normalization and escaping to perform in
+  // `EncodeVal`, in addition to encoding the string into bytes.
+  enum EncodeType {
+    // Normalizes newlines to CRLF and then escapes for use in
+    // `Content-Disposition`. (Useful for `multipart/form-data` entry names.)
+    eNameEncode,
+    // Escapes for use in `Content-Disposition`. (Useful for
+    // `multipart/form-data` filenames.)
+    eFilenameEncode,
+    // Normalizes newlines to CRLF.
+    eValueEncode,
+  };
+
   /**
-   * Encode a Unicode string to bytes using the encoder (or just copy the input
-   * if there is no encoder).
+   * Encode a Unicode string to bytes, additionally performing escapes or
+   * normalizations.
    * @param aStr the string to encode
-   * @param aResult the encoded string [OUT]
-   * @param aHeaderEncode If true, turns all linebreaks into spaces and escapes
-   *                      all quotes
+   * @param aOut the encoded string [OUT]
+   * @param aEncodeType The type of escapes or normalizations to perform on the
+   *                    encoded string.
    * @throws an error if UnicodeToNewBytes fails
    */
-  nsresult EncodeVal(const nsAString& aStr, nsCString& aResult,
-                     bool aHeaderEncode);
+  nsresult EncodeVal(const nsAString& aStr, nsCString& aOut,
+                     EncodeType aEncodeType);
+};
+
+class DialogFormSubmission final : public HTMLFormSubmission {
+ public:
+  DialogFormSubmission(nsAString& aResult, NotNull<const Encoding*> aEncoding,
+                       HTMLDialogElement* aDialogElement)
+      : HTMLFormSubmission(nullptr, u""_ns, aEncoding),
+        mDialogElement(aDialogElement),
+        mReturnValue(aResult) {}
+  nsresult AddNameValuePair(const nsAString& aName,
+                            const nsAString& aValue) override {
+    MOZ_CRASH("This method should not be called");
+    return NS_OK;
+  }
+
+  nsresult AddNameBlobPair(const nsAString& aName, Blob* aBlob) override {
+    MOZ_CRASH("This method should not be called");
+    return NS_OK;
+  }
+
+  nsresult AddNameDirectoryPair(const nsAString& aName,
+                                Directory* aDirectory) override {
+    MOZ_CRASH("This method should not be called");
+    return NS_OK;
+  }
+
+  nsresult GetEncodedSubmission(nsIURI* aURI, nsIInputStream** aPostDataStream,
+                                nsCOMPtr<nsIURI>& aOutURI) override {
+    MOZ_CRASH("This method should not be called");
+    return NS_OK;
+  }
+
+  DialogFormSubmission* GetAsDialogSubmission() override { return this; }
+
+  HTMLDialogElement* DialogElement() { return mDialogElement; }
+
+  nsString& ReturnValue() { return mReturnValue; }
+
+ private:
+  const RefPtr<HTMLDialogElement> mDialogElement;
+  nsString mReturnValue;
 };
 
 /**
@@ -167,26 +219,24 @@ class FSMultipartFormData : public EncodingFormSubmission {
    */
   FSMultipartFormData(nsIURI* aActionURL, const nsAString& aTarget,
                       mozilla::NotNull<const mozilla::Encoding*> aEncoding,
-                      Element* aOriginatingElement);
+                      Element* aSubmitter);
   ~FSMultipartFormData();
 
   virtual nsresult AddNameValuePair(const nsAString& aName,
                                     const nsAString& aValue) override;
 
-  virtual nsresult AddNameBlobOrNullPair(const nsAString& aName,
-                                         Blob* aBlob) override;
+  virtual nsresult AddNameBlobPair(const nsAString& aName,
+                                   Blob* aBlob) override;
 
   virtual nsresult AddNameDirectoryPair(const nsAString& aName,
                                         Directory* aDirectory) override;
 
   virtual nsresult GetEncodedSubmission(nsIURI* aURI,
                                         nsIInputStream** aPostDataStream,
-                                        int64_t* aPostDataStreamLength,
                                         nsCOMPtr<nsIURI>& aOutURI) override;
 
   void GetContentType(nsACString& aContentType) {
-    aContentType =
-        NS_LITERAL_CSTRING("multipart/form-data; boundary=") + mBoundary;
+    aContentType = "multipart/form-data; boundary="_ns + mBoundary;
   }
 
   nsIInputStream* GetSubmissionBody(uint64_t* aContentLength);
@@ -236,7 +286,6 @@ class FSMultipartFormData : public EncodingFormSubmission {
   uint64_t mTotalLength;
 };
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
 
 #endif /* mozilla_dom_HTMLFormSubmission_h */
