@@ -5,21 +5,20 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "mozilla/dom/HTMLOptionElement.h"
+
+#include "HTMLOptGroupElement.h"
 #include "mozilla/dom/HTMLOptionElementBinding.h"
 #include "mozilla/dom/HTMLSelectElement.h"
 #include "nsGkAtoms.h"
 #include "nsStyleConsts.h"
 #include "nsIFormControl.h"
-#include "nsIForm.h"
-#include "nsIDOMNode.h"
 #include "nsISelectControlFrame.h"
 
 // Notify/query select frame for selected state
 #include "nsIFormControlFrame.h"
-#include "nsIDocument.h"
+#include "mozilla/dom/Document.h"
 #include "nsNodeInfoManager.h"
 #include "nsCOMPtr.h"
-#include "mozilla/EventStates.h"
 #include "nsContentCreatorFunctions.h"
 #include "mozAutoDocUpdate.h"
 #include "nsTextNode.h"
@@ -30,20 +29,16 @@
 
 NS_IMPL_NS_NEW_HTML_ELEMENT(Option)
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 HTMLOptionElement::HTMLOptionElement(
-    already_AddRefed<mozilla::dom::NodeInfo>& aNodeInfo)
-    : nsGenericHTMLElement(aNodeInfo),
-      mSelectedChanged(false),
-      mIsSelected(false),
-      mIsInSetDefaultSelected(false) {
+    already_AddRefed<mozilla::dom::NodeInfo>&& aNodeInfo)
+    : nsGenericHTMLElement(std::move(aNodeInfo)) {
   // We start off enabled
-  AddStatesSilently(NS_EVENT_STATE_ENABLED);
+  AddStatesSilently(ElementState::ENABLED);
 }
 
-HTMLOptionElement::~HTMLOptionElement() {}
+HTMLOptionElement::~HTMLOptionElement() = default;
 
 NS_IMPL_ELEMENT_CLONE(HTMLOptionElement)
 
@@ -54,13 +49,7 @@ mozilla::dom::HTMLFormElement* HTMLOptionElement::GetForm() {
 
 void HTMLOptionElement::SetSelectedInternal(bool aValue, bool aNotify) {
   mSelectedChanged = true;
-  mIsSelected = aValue;
-
-  // When mIsInSetDefaultSelected is true, the state change will be handled by
-  // SetAttr/UnsetAttr.
-  if (!mIsInSetDefaultSelected) {
-    UpdateState(aNotify);
-  }
+  SetStates(ElementState::CHECKED, aValue, aNotify);
 }
 
 void HTMLOptionElement::OptGroupDisabledChanged(bool aNotify) {
@@ -68,24 +57,24 @@ void HTMLOptionElement::OptGroupDisabledChanged(bool aNotify) {
 }
 
 void HTMLOptionElement::UpdateDisabledState(bool aNotify) {
-  bool isDisabled = HasAttr(kNameSpaceID_None, nsGkAtoms::disabled);
+  bool isDisabled = HasAttr(nsGkAtoms::disabled);
 
   if (!isDisabled) {
     nsIContent* parent = GetParent();
-    if (auto optGroupElement = HTMLOptGroupElement::FromContentOrNull(parent)) {
+    if (auto optGroupElement = HTMLOptGroupElement::FromNodeOrNull(parent)) {
       isDisabled = optGroupElement->IsDisabled();
     }
   }
 
-  EventStates disabledStates;
+  ElementState disabledStates;
   if (isDisabled) {
-    disabledStates |= NS_EVENT_STATE_DISABLED;
+    disabledStates |= ElementState::DISABLED;
   } else {
-    disabledStates |= NS_EVENT_STATE_ENABLED;
+    disabledStates |= ElementState::ENABLED;
   }
 
-  EventStates oldDisabledStates = State() & DISABLED_STATES;
-  EventStates changedStates = disabledStates ^ oldDisabledStates;
+  ElementState oldDisabledStates = State() & ElementState::DISABLED_STATES;
+  ElementState changedStates = disabledStates ^ oldDisabledStates;
 
   if (!changedStates.IsEmpty()) {
     ToggleStates(changedStates, aNotify);
@@ -98,9 +87,11 @@ void HTMLOptionElement::SetSelected(bool aValue) {
   HTMLSelectElement* selectInt = GetSelect();
   if (selectInt) {
     int32_t index = Index();
-    uint32_t mask = HTMLSelectElement::SET_DISABLED | HTMLSelectElement::NOTIFY;
+    HTMLSelectElement::OptionFlags mask{
+        HTMLSelectElement::OptionFlag::SetDisabled,
+        HTMLSelectElement::OptionFlag::Notify};
     if (aValue) {
-      mask |= HTMLSelectElement::IS_SELECTED;
+      mask += HTMLSelectElement::OptionFlag::IsSelected;
     }
 
     // This should end up calling SetSelectedInternal
@@ -134,22 +125,21 @@ nsChangeHint HTMLOptionElement::GetAttributeChangeHint(const nsAtom* aAttribute,
   nsChangeHint retval =
       nsGenericHTMLElement::GetAttributeChangeHint(aAttribute, aModType);
 
-  if (aAttribute == nsGkAtoms::label || aAttribute == nsGkAtoms::text) {
+  if (aAttribute == nsGkAtoms::label) {
+    retval |= nsChangeHint_ReconstructFrame;
+  } else if (aAttribute == nsGkAtoms::text) {
     retval |= NS_STYLE_HINT_REFLOW;
   }
   return retval;
 }
 
-nsresult HTMLOptionElement::BeforeSetAttr(int32_t aNamespaceID, nsAtom* aName,
-                                          const nsAttrValueOrString* aValue,
-                                          bool aNotify) {
-  nsresult rv =
-      nsGenericHTMLElement::BeforeSetAttr(aNamespaceID, aName, aValue, aNotify);
-  NS_ENSURE_SUCCESS(rv, rv);
+void HTMLOptionElement::BeforeSetAttr(int32_t aNamespaceID, nsAtom* aName,
+                                      const nsAttrValue* aValue, bool aNotify) {
+  nsGenericHTMLElement::BeforeSetAttr(aNamespaceID, aName, aValue, aNotify);
 
   if (aNamespaceID != kNameSpaceID_None || aName != nsGkAtoms::selected ||
       mSelectedChanged) {
-    return NS_OK;
+    return;
   }
 
   // We just changed out selected state (since we look at the "selected"
@@ -157,10 +147,10 @@ nsresult HTMLOptionElement::BeforeSetAttr(int32_t aNamespaceID, nsAtom* aName,
   // it.
   HTMLSelectElement* selectInt = GetSelect();
   if (!selectInt) {
-    // If option is a child of select, SetOptionsSelectedByIndex will set
-    // mIsSelected if needed.
-    mIsSelected = aValue;
-    return NS_OK;
+    // If option is a child of select, SetOptionsSelectedByIndex will set the
+    // selected state if needed.
+    SetStates(ElementState::CHECKED, !!aValue, aNotify);
+    return;
   }
 
   NS_ASSERTION(!mSelectedChanged, "Shouldn't be here");
@@ -169,13 +159,14 @@ nsresult HTMLOptionElement::BeforeSetAttr(int32_t aNamespaceID, nsAtom* aName,
   mIsInSetDefaultSelected = true;
 
   int32_t index = Index();
-  uint32_t mask = HTMLSelectElement::SET_DISABLED;
+  HTMLSelectElement::OptionFlags mask =
+      HTMLSelectElement::OptionFlag::SetDisabled;
   if (aValue) {
-    mask |= HTMLSelectElement::IS_SELECTED;
+    mask += HTMLSelectElement::OptionFlag::IsSelected;
   }
 
   if (aNotify) {
-    mask |= HTMLSelectElement::NOTIFY;
+    mask += HTMLSelectElement::OptionFlag::Notify;
   }
 
   // This can end up calling SetSelectedInternal if our selected state needs to
@@ -187,32 +178,33 @@ nsresult HTMLOptionElement::BeforeSetAttr(int32_t aNamespaceID, nsAtom* aName,
   // Now reset our members; when we finish the attr set we'll end up with the
   // rigt selected state.
   mIsInSetDefaultSelected = inSetDefaultSelected;
-  // mIsSelected might have been changed by SetOptionsSelectedByIndex.  Possibly
-  // more than once; make sure our mSelectedChanged state is set back correctly.
+  // the selected state might have been changed by SetOptionsSelectedByIndex,
+  // possibly more than once; make sure our mSelectedChanged state is set back
+  // correctly.
   mSelectedChanged = false;
-
-  return NS_OK;
 }
 
-nsresult HTMLOptionElement::AfterSetAttr(int32_t aNameSpaceID, nsAtom* aName,
-                                         const nsAttrValue* aValue,
-                                         const nsAttrValue* aOldValue,
-                                         nsIPrincipal* aSubjectPrincipal,
-                                         bool aNotify) {
+void HTMLOptionElement::AfterSetAttr(int32_t aNameSpaceID, nsAtom* aName,
+                                     const nsAttrValue* aValue,
+                                     const nsAttrValue* aOldValue,
+                                     nsIPrincipal* aSubjectPrincipal,
+                                     bool aNotify) {
   if (aNameSpaceID == kNameSpaceID_None) {
     if (aName == nsGkAtoms::disabled) {
       UpdateDisabledState(aNotify);
     }
 
     if (aName == nsGkAtoms::value && Selected()) {
-      // Since this option is selected, changing value
-      // may have changed missing validity state of the
-      // Select element
-      HTMLSelectElement* select = GetSelect();
-      if (select) {
+      // Since this option is selected, changing value may have changed missing
+      // validity state of the select element
+      if (HTMLSelectElement* select = GetSelect()) {
         select->UpdateValueMissingValidityState();
       }
     }
+
+    if (aName == nsGkAtoms::selected) {
+      SetStates(ElementState::DEFAULT, !!aValue, aNotify);
+    }
   }
 
   return nsGenericHTMLElement::AfterSetAttr(
@@ -224,9 +216,8 @@ void HTMLOptionElement::GetText(nsAString& aText) {
 
   nsIContent* child = nsINode::GetFirstChild();
   while (child) {
-    if (child->NodeType() == TEXT_NODE ||
-        child->NodeType() == CDATA_SECTION_NODE) {
-      child->AppendTextTo(text);
+    if (Text* textChild = child->GetAsText()) {
+      textChild->AppendTextTo(text);
     }
     if (child->IsHTMLElement(nsGkAtoms::script) ||
         child->IsSVGElement(nsGkAtoms::script)) {
@@ -242,15 +233,12 @@ void HTMLOptionElement::GetText(nsAString& aText) {
 }
 
 void HTMLOptionElement::SetText(const nsAString& aText, ErrorResult& aRv) {
-  aRv = nsContentUtils::SetNodeTextContent(this, aText, true);
+  aRv = nsContentUtils::SetNodeTextContent(this, aText, false);
 }
 
-nsresult HTMLOptionElement::BindToTree(nsIDocument* aDocument,
-                                       nsIContent* aParent,
-                                       nsIContent* aBindingParent,
-                                       bool aCompileEventHandlers) {
-  nsresult rv = nsGenericHTMLElement::BindToTree(
-      aDocument, aParent, aBindingParent, aCompileEventHandlers);
+nsresult HTMLOptionElement::BindToTree(BindContext& aContext,
+                                       nsINode& aParent) {
+  nsresult rv = nsGenericHTMLElement::BindToTree(aContext, aParent);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Our new parent might change :disabled/:enabled state.
@@ -259,25 +247,13 @@ nsresult HTMLOptionElement::BindToTree(nsIDocument* aDocument,
   return NS_OK;
 }
 
-void HTMLOptionElement::UnbindFromTree(bool aDeep, bool aNullParent) {
-  nsGenericHTMLElement::UnbindFromTree(aDeep, aNullParent);
+void HTMLOptionElement::UnbindFromTree(UnbindContext& aContext) {
+  nsGenericHTMLElement::UnbindFromTree(aContext);
 
   // Our previous parent could have been involved in :disabled/:enabled state.
   UpdateDisabledState(false);
 }
 
-EventStates HTMLOptionElement::IntrinsicState() const {
-  EventStates state = nsGenericHTMLElement::IntrinsicState();
-  if (Selected()) {
-    state |= NS_EVENT_STATE_CHECKED;
-  }
-  if (DefaultSelected()) {
-    state |= NS_EVENT_STATE_DEFAULT;
-  }
-
-  return state;
-}
-
 // Get the select content element that contains this option
 HTMLSelectElement* HTMLOptionElement::GetSelect() {
   nsIContent* parent = GetParent();
@@ -285,7 +261,7 @@ HTMLSelectElement* HTMLOptionElement::GetSelect() {
     return nullptr;
   }
 
-  HTMLSelectElement* select = HTMLSelectElement::FromContent(parent);
+  HTMLSelectElement* select = HTMLSelectElement::FromNode(parent);
   if (select) {
     return select;
   }
@@ -294,7 +270,7 @@ HTMLSelectElement* HTMLOptionElement::GetSelect() {
     return nullptr;
   }
 
-  return HTMLSelectElement::FromContentOrNull(parent->GetParent());
+  return HTMLSelectElement::FromNodeOrNull(parent->GetParent());
 }
 
 already_AddRefed<HTMLOptionElement> HTMLOptionElement::Option(
@@ -302,26 +278,27 @@ already_AddRefed<HTMLOptionElement> HTMLOptionElement::Option(
     const Optional<nsAString>& aValue, bool aDefaultSelected, bool aSelected,
     ErrorResult& aError) {
   nsCOMPtr<nsPIDOMWindowInner> win = do_QueryInterface(aGlobal.GetAsSupports());
-  nsIDocument* doc;
+  Document* doc;
   if (!win || !(doc = win->GetExtantDoc())) {
     aError.Throw(NS_ERROR_FAILURE);
     return nullptr;
   }
 
-  already_AddRefed<mozilla::dom::NodeInfo> nodeInfo =
-      doc->NodeInfoManager()->GetNodeInfo(nsGkAtoms::option, nullptr,
-                                          kNameSpaceID_XHTML, ELEMENT_NODE);
+  RefPtr<mozilla::dom::NodeInfo> nodeInfo = doc->NodeInfoManager()->GetNodeInfo(
+      nsGkAtoms::option, nullptr, kNameSpaceID_XHTML, ELEMENT_NODE);
 
-  RefPtr<HTMLOptionElement> option = new HTMLOptionElement(nodeInfo);
+  auto* nim = nodeInfo->NodeInfoManager();
+  RefPtr<HTMLOptionElement> option =
+      new (nim) HTMLOptionElement(nodeInfo.forget());
 
   if (!aText.IsEmpty()) {
     // Create a new text node and append it to the option
-    RefPtr<nsTextNode> textContent =
-        new nsTextNode(option->NodeInfo()->NodeInfoManager());
+    RefPtr<nsTextNode> textContent = new (option->NodeInfo()->NodeInfoManager())
+        nsTextNode(option->NodeInfo()->NodeInfoManager());
 
     textContent->SetText(aText, false);
 
-    aError = option->AppendChildTo(textContent, false);
+    option->AppendChildTo(textContent, false, aError);
     if (aError.Failed()) {
       return nullptr;
     }
@@ -340,8 +317,8 @@ already_AddRefed<HTMLOptionElement> HTMLOptionElement::Option(
   if (aDefaultSelected) {
     // We're calling SetAttr directly because we want to pass
     // aNotify == false.
-    aError = option->SetAttr(kNameSpaceID_None, nsGkAtoms::selected,
-                             EmptyString(), false);
+    aError =
+        option->SetAttr(kNameSpaceID_None, nsGkAtoms::selected, u""_ns, false);
     if (aError.Failed()) {
       return nullptr;
     }
@@ -353,9 +330,8 @@ already_AddRefed<HTMLOptionElement> HTMLOptionElement::Option(
   return option.forget();
 }
 
-nsresult HTMLOptionElement::CopyInnerTo(Element* aDest,
-                                        bool aPreallocateChildren) {
-  nsresult rv = nsGenericHTMLElement::CopyInnerTo(aDest, aPreallocateChildren);
+nsresult HTMLOptionElement::CopyInnerTo(Element* aDest) {
+  nsresult rv = nsGenericHTMLElement::CopyInnerTo(aDest);
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (aDest->OwnerDoc()->IsStaticDocument()) {
@@ -366,8 +342,7 @@ nsresult HTMLOptionElement::CopyInnerTo(Element* aDest,
 
 JSObject* HTMLOptionElement::WrapNode(JSContext* aCx,
                                       JS::Handle<JSObject*> aGivenProto) {
-  return HTMLOptionElementBinding::Wrap(aCx, this, aGivenProto);
+  return HTMLOptionElement_Binding::Wrap(aCx, this, aGivenProto);
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
