@@ -21,55 +21,37 @@
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsIURI.h"
-#include "nsIContentViewer.h"
 #include "mozilla/dom/NodeInfo.h"
 #include "mozilla/dom/ScriptLoader.h"
-#include "nsIAppShell.h"
 #include "nsCRT.h"
 #include "prtime.h"
 #include "mozilla/Logging.h"
-#include "nsNodeUtils.h"
 #include "nsIContent.h"
 #include "mozilla/dom/CustomElementRegistry.h"
 #include "mozilla/dom/Element.h"
+#include "mozilla/dom/MutationObservers.h"
 #include "mozilla/Preferences.h"
 
 #include "nsGenericHTMLElement.h"
 
-#include "nsIDOMDocument.h"
-#include "nsIDOMDocumentType.h"
 #include "nsIScriptElement.h"
 
-#include "nsIComponentManager.h"
-#include "nsIServiceManager.h"
-
+#include "nsDocElementCreatedNotificationRunner.h"
 #include "nsGkAtoms.h"
 #include "nsContentUtils.h"
 #include "nsIChannel.h"
-#include "nsIHttpChannel.h"
-#include "nsIDocShell.h"
-#include "nsIDocument.h"
+#include "mozilla/dom/Document.h"
 #include "nsStubDocumentObserver.h"
-#include "nsIHTMLDocument.h"
-#include "nsICookieService.h"
+#include "nsHTMLDocument.h"
 #include "nsTArray.h"
-#include "nsIScriptSecurityManager.h"
-#include "nsIPrincipal.h"
 #include "nsTextFragment.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsNameSpaceManager.h"
 
-#include "nsIStyleSheetLinkingElement.h"
-#include "nsITimer.h"
 #include "nsError.h"
 #include "nsContentPolicyUtils.h"
+#include "nsIDocShell.h"
 #include "nsIScriptContext.h"
-#include "nsStyleLinkElement.h"
-
-#include "nsWeakReference.h"  // nsHTMLElementFactory supports weak references
-#include "nsIPrompt.h"
-#include "nsLayoutCID.h"
-#include "nsIDocShellTreeItem.h"
 
 #include "nsEscape.h"
 #include "nsNodeInfoManager.h"
@@ -85,7 +67,7 @@ using namespace mozilla::dom;
 nsGenericHTMLElement* NS_NewHTMLNOTUSEDElement(
     already_AddRefed<mozilla::dom::NodeInfo>&& aNodeInfo,
     FromParser aFromParser) {
-  NS_NOTREACHED("The element ctor should never be called");
+  MOZ_ASSERT_UNREACHABLE("The element ctor should never be called");
   return nullptr;
 }
 
@@ -112,7 +94,7 @@ class HTMLContentSink : public nsContentSink, public nsIHTMLContentSink {
 
   HTMLContentSink();
 
-  nsresult Init(nsIDocument* aDoc, nsIURI* aURI, nsISupports* aContainer,
+  nsresult Init(Document* aDoc, nsIURI* aURI, nsISupports* aContainer,
                 nsIChannel* aChannel);
 
   // nsISupports
@@ -124,12 +106,13 @@ class HTMLContentSink : public nsContentSink, public nsIHTMLContentSink {
   NS_IMETHOD WillBuildModel(nsDTDMode aDTDMode) override;
   NS_IMETHOD DidBuildModel(bool aTerminated) override;
   NS_IMETHOD WillInterrupt(void) override;
-  NS_IMETHOD WillResume(void) override;
+  void WillResume() override;
   NS_IMETHOD SetParser(nsParserBase* aParser) override;
   virtual void FlushPendingNotifications(FlushType aType) override;
   virtual void SetDocumentCharset(NotNull<const Encoding*> aEncoding) override;
   virtual nsISupports* GetTarget() override;
   virtual bool IsScriptExecuting() override;
+  virtual bool WaitForPendingSheets() override;
   virtual void ContinueInterruptedParsingAsync() override;
 
   // nsIHTMLContentSink
@@ -139,7 +122,7 @@ class HTMLContentSink : public nsContentSink, public nsIHTMLContentSink {
  protected:
   virtual ~HTMLContentSink();
 
-  nsCOMPtr<nsIHTMLDocument> mHTMLDocument;
+  RefPtr<nsHTMLDocument> mHTMLDocument;
 
   // The maximum length of a text run
   int32_t mMaxTextRun;
@@ -193,7 +176,7 @@ class SinkContext {
   nsresult GrowStack();
   nsresult FlushTags();
 
-  bool IsCurrentContainer(nsHTMLTag mType);
+  bool IsCurrentContainer(nsHTMLTag aTag) const;
 
   void DidAddContent(nsIContent* aContent);
   void UpdateChildCounts();
@@ -248,7 +231,7 @@ already_AddRefed<nsGenericHTMLElement> CreateHTMLElement(
   NS_ASSERTION(cb != NS_NewHTMLNOTUSEDElement,
                "Don't know how to construct tag element!");
 
-  RefPtr<nsGenericHTMLElement> result = cb(Move(aNodeInfo), aFromParser);
+  RefPtr<nsGenericHTMLElement> result = cb(std::move(aNodeInfo), aFromParser);
 
   return result.forget();
 }
@@ -294,12 +277,8 @@ nsresult SinkContext::Begin(nsHTMLTag aNodeType, nsGenericHTMLElement* aRoot,
   return NS_OK;
 }
 
-bool SinkContext::IsCurrentContainer(nsHTMLTag aTag) {
-  if (aTag == mStack[mStackPos - 1].mType) {
-    return true;
-  }
-
-  return false;
+bool SinkContext::IsCurrentContainer(nsHTMLTag aTag) const {
+  return aTag == mStack[mStackPos - 1].mType;
 }
 
 void SinkContext::DidAddContent(nsIContent* aContent) {
@@ -375,9 +354,10 @@ nsIContent* SinkContext::Node::Add(nsIContent* child) {
                  "Inserting multiple children without flushing.");
     nsCOMPtr<nsIContent> nodeToInsertBefore =
         mContent->GetChildAt_Deprecated(mInsertionPoint++);
-    mContent->InsertChildBefore(child, nodeToInsertBefore, false);
+    mContent->InsertChildBefore(child, nodeToInsertBefore, false,
+                                IgnoreErrors());
   } else {
-    mContent->AppendChildTo(child, false);
+    mContent->AppendChildTo(child, false, IgnoreErrors());
   }
   return child;
 }
@@ -459,15 +439,13 @@ nsresult SinkContext::GrowStack() {
  */
 nsresult SinkContext::FlushTags() {
   mSink->mDeferredFlushTags = false;
-  bool oldBeganUpdate = mSink->mBeganUpdate;
   uint32_t oldUpdates = mSink->mUpdatesInNotification;
 
   ++(mSink->mInNotification);
   mSink->mUpdatesInNotification = 0;
   {
     // Scope so we call EndUpdate before we decrease mInNotification
-    mozAutoDocUpdate updateBatch(mSink->mDocument, UPDATE_CONTENT_MODEL, true);
-    mSink->mBeganUpdate = true;
+    mozAutoDocUpdate updateBatch(mSink->mDocument, true);
 
     // Start from the base of the stack (growing downward) and do
     // a notification from the node that is closest to the root of
@@ -517,7 +495,6 @@ nsresult SinkContext::FlushTags() {
   }
 
   mSink->mUpdatesInNotification = oldUpdates;
-  mSink->mBeganUpdate = oldBeganUpdate;
 
   return NS_OK;
 }
@@ -542,7 +519,7 @@ void SinkContext::UpdateChildCounts() {
   mNotifyLevel = mStackPos - 1;
 }
 
-nsresult NS_NewHTMLContentSink(nsIHTMLContentSink** aResult, nsIDocument* aDoc,
+nsresult NS_NewHTMLContentSink(nsIHTMLContentSink** aResult, Document* aDoc,
                                nsIURI* aURI, nsISupports* aContainer,
                                nsIChannel* aChannel) {
   NS_ENSURE_ARG_POINTER(aResult);
@@ -571,15 +548,13 @@ HTMLContentSink::~HTMLContentSink() {
     mNotificationTimer->Cancel();
   }
 
-  int32_t numContexts = mContextStack.Length();
-
-  if (mCurrentContext == mHeadContext && numContexts > 0) {
+  if (mCurrentContext == mHeadContext && !mContextStack.IsEmpty()) {
     // Pop off the second html context if it's not done earlier
-    mContextStack.RemoveElementAt(--numContexts);
+    mContextStack.RemoveLastElement();
   }
 
-  int32_t i;
-  for (i = 0; i < numContexts; i++) {
+  for (int32_t i = 0, numContexts = mContextStack.Length(); i < numContexts;
+       i++) {
     SinkContext* sc = mContextStack.ElementAt(i);
     if (sc) {
       sc->End();
@@ -606,7 +581,7 @@ NS_IMPL_CYCLE_COLLECTION_INHERITED(HTMLContentSink, nsContentSink,
 NS_IMPL_ISUPPORTS_CYCLE_COLLECTION_INHERITED(HTMLContentSink, nsContentSink,
                                              nsIContentSink, nsIHTMLContentSink)
 
-nsresult HTMLContentSink::Init(nsIDocument* aDoc, nsIURI* aURI,
+nsresult HTMLContentSink::Init(Document* aDoc, nsIURI* aURI,
                                nsISupports* aContainer, nsIChannel* aChannel) {
   NS_ENSURE_TRUE(aContainer, NS_ERROR_NULL_POINTER);
 
@@ -617,7 +592,7 @@ nsresult HTMLContentSink::Init(nsIDocument* aDoc, nsIURI* aURI,
 
   aDoc->AddObserver(this);
   mIsDocumentObserver = true;
-  mHTMLDocument = do_QueryInterface(aDoc);
+  mHTMLDocument = aDoc->AsHTMLDocument();
 
   NS_ASSERTION(mDocShell, "oops no docshell!");
 
@@ -637,8 +612,11 @@ nsresult HTMLContentSink::Init(nsIDocument* aDoc, nsIURI* aURI,
 
   NS_ASSERTION(mDocument->GetChildCount() == 0,
                "Document should have no kids here!");
-  rv = mDocument->AppendChildTo(mRoot, false);
-  NS_ENSURE_SUCCESS(rv, rv);
+  ErrorResult error;
+  mDocument->AppendChildTo(mRoot, false, error);
+  if (error.Failed()) {
+    return error.StealNSResult();
+  }
 
   // Make head part
   nodeInfo = mNodeInfoManager->GetNodeInfo(
@@ -649,7 +627,7 @@ nsresult HTMLContentSink::Init(nsIDocument* aDoc, nsIURI* aURI,
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
-  mRoot->AppendChildTo(mHead, false);
+  mRoot->AppendChildTo(mHead, false, IgnoreErrors());
 
   mCurrentContext = new SinkContext(this);
   mCurrentContext->Begin(eHTMLTag_html, mRoot, 0, -1);
@@ -665,20 +643,9 @@ NS_IMETHODIMP
 HTMLContentSink::WillBuildModel(nsDTDMode aDTDMode) {
   WillBuildModelImpl();
 
-  if (mHTMLDocument) {
-    nsCompatibility mode = eCompatibility_NavQuirks;
-    switch (aDTDMode) {
-      case eDTDMode_full_standards:
-        mode = eCompatibility_FullStandards;
-        break;
-      case eDTDMode_almost_standards:
-        mode = eCompatibility_AlmostStandards;
-        break;
-      default:
-        break;
-    }
-    mHTMLDocument->SetCompatibilityMode(mode);
-  }
+  mDocument->SetCompatibilityMode(aDTDMode == eDTDMode_full_standards
+                                      ? eCompatibility_FullStandards
+                                      : eCompatibility_NavQuirks);
 
   // Notify document that the load is beginning
   mDocument->BeginLoad();
@@ -730,7 +697,7 @@ HTMLContentSink::DidBuildModel(bool aTerminated) {
 
 NS_IMETHODIMP
 HTMLContentSink::SetParser(nsParserBase* aParser) {
-  NS_PRECONDITION(aParser, "Should have a parser here!");
+  MOZ_ASSERT(aParser, "Should have a parser here!");
   mParser = aParser;
   return NS_OK;
 }
@@ -738,11 +705,8 @@ HTMLContentSink::SetParser(nsParserBase* aParser) {
 nsresult HTMLContentSink::CloseHTML() {
   if (mHeadContext) {
     if (mCurrentContext == mHeadContext) {
-      uint32_t numContexts = mContextStack.Length();
-
       // Pop off the second html context if it's not done earlier
-      mCurrentContext = mContextStack.ElementAt(--numContexts);
-      mContextStack.RemoveElementAt(numContexts);
+      mCurrentContext = mContextStack.PopLastElement();
     }
 
     mHeadContext->End();
@@ -826,7 +790,6 @@ HTMLContentSink::OpenContainer(ElementType aElementType) {
         if (!mNotifiedRootInsertion) {
           NotifyRootInsertion();
         }
-        ProcessOfflineManifest(mRoot);
       }
       break;
   }
@@ -853,8 +816,7 @@ HTMLContentSink::CloseContainer(const ElementType aTag) {
 NS_IMETHODIMP
 HTMLContentSink::WillInterrupt() { return WillInterruptImpl(); }
 
-NS_IMETHODIMP
-HTMLContentSink::WillResume() { return WillResumeImpl(); }
+void HTMLContentSink::WillResume() { WillResumeImpl(); }
 
 void HTMLContentSink::CloseHeadContext() {
   if (mCurrentContext) {
@@ -864,26 +826,21 @@ void HTMLContentSink::CloseHeadContext() {
   }
 
   if (!mContextStack.IsEmpty()) {
-    uint32_t n = mContextStack.Length() - 1;
-    mCurrentContext = mContextStack.ElementAt(n);
-    mContextStack.RemoveElementAt(n);
+    mCurrentContext = mContextStack.PopLastElement();
   }
 }
 
 void HTMLContentSink::NotifyInsert(nsIContent* aContent,
                                    nsIContent* aChildContent) {
-  if (aContent && aContent->GetUncomposedDoc() != mDocument) {
-    // aContent is not actually in our document anymore.... Just bail out of
-    // here; notifying on our document for this insert would be wrong.
-    return;
-  }
-
   mInNotification++;
 
   {
     // Scope so we call EndUpdate before we decrease mInNotification
-    MOZ_AUTO_DOC_UPDATE(mDocument, UPDATE_CONTENT_MODEL, !mBeganUpdate);
-    nsNodeUtils::ContentInserted(NODE_FROM(aContent, mDocument), aChildContent);
+    // Note that aContent->OwnerDoc() may be different to mDocument already.
+    MOZ_AUTO_DOC_UPDATE(aContent ? aContent->OwnerDoc() : mDocument.get(),
+                        true);
+    MutationObservers::NotifyContentInserted(NODE_FROM(aContent, mDocument),
+                                             aChildContent);
     mLastNotificationTime = PR_Now();
   }
 
@@ -891,7 +848,7 @@ void HTMLContentSink::NotifyInsert(nsIContent* aContent,
 }
 
 void HTMLContentSink::NotifyRootInsertion() {
-  NS_PRECONDITION(!mNotifiedRootInsertion, "Double-notifying on root?");
+  MOZ_ASSERT(!mNotifiedRootInsertion, "Double-notifying on root?");
   NS_ASSERTION(!mLayoutStarted,
                "How did we start layout without notifying on root?");
   // Now make sure to notify that we have now inserted our root.  If
@@ -908,6 +865,9 @@ void HTMLContentSink::NotifyRootInsertion() {
   // contexts, since we just inserted the root and notified on
   // our whole tree
   UpdateChildCounts();
+
+  nsContentUtils::AddScriptRunner(
+      new nsDocElementCreatedNotificationRunner(mDocument));
 }
 
 void HTMLContentSink::UpdateChildCounts() {
@@ -953,7 +913,7 @@ void HTMLContentSink::SetDocumentCharset(NotNull<const Encoding*> aEncoding) {
   MOZ_ASSERT_UNREACHABLE("<meta charset> case doesn't occur with about:blank");
 }
 
-nsISupports* HTMLContentSink::GetTarget() { return mDocument; }
+nsISupports* HTMLContentSink::GetTarget() { return ToSupports(mDocument); }
 
 bool HTMLContentSink::IsScriptExecuting() { return IsScriptExecutingImpl(); }
 
@@ -963,11 +923,13 @@ void HTMLContentSink::ContinueInterruptedParsingIfEnabled() {
   }
 }
 
+bool HTMLContentSink::WaitForPendingSheets() {
+  return nsContentSink::WaitForPendingSheets();
+}
+
 void HTMLContentSink::ContinueInterruptedParsingAsync() {
   nsCOMPtr<nsIRunnable> ev = NewRunnableMethod(
       "HTMLContentSink::ContinueInterruptedParsingIfEnabled", this,
       &HTMLContentSink::ContinueInterruptedParsingIfEnabled);
-
-  nsCOMPtr<nsIDocument> doc = do_QueryInterface(mHTMLDocument);
-  doc->Dispatch(mozilla::TaskCategory::Other, ev.forget());
+  mHTMLDocument->Dispatch(ev.forget());
 }
