@@ -77,6 +77,13 @@ namespace ipc {
 class StructuredCloneData;
 }  // namespace ipc
 
+#define DOM_BROWSERPARENT_IID                        \
+  {                                                  \
+    0x58b47b52, 0x77dc, 0x44cf, {                    \
+      0x8b, 0xe5, 0x8e, 0x78, 0x24, 0xd9, 0xae, 0xc5 \
+    }                                                \
+  }
+
 /**
  * BrowserParent implements the parent actor part of the PBrowser protocol. See
  * PBrowser for more information.
@@ -98,6 +105,7 @@ class BrowserParent final : public PBrowserParent,
   // Helper class for ContentParent::RecvCreateWindow.
   struct AutoUseNewTab;
 
+  NS_DECLARE_STATIC_IID_ACCESSOR(DOM_BROWSERPARENT_IID)
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
   NS_DECL_NSIAUTHPROMPTPROVIDER
   // nsIDOMEventListener interfaces
@@ -131,7 +139,7 @@ class BrowserParent final : public PBrowserParent,
 
   const TabId GetTabId() const { return mTabId; }
 
-  ContentParent* Manager() const { return mManager; }
+  ContentParent* Manager() const;
 
   CanonicalBrowsingContext* GetBrowsingContext() { return mBrowsingContext; }
 
@@ -303,7 +311,10 @@ class BrowserParent final : public PBrowserParent,
       const bool aBlocked, const nsACString& aTrackingOrigin,
       nsTArray<nsCString>&& aTrackingFullHashes,
       const Maybe<mozilla::ContentBlockingNotifier::
-                      StorageAccessPermissionGrantedReason>& aReason);
+                      StorageAccessPermissionGrantedReason>& aReason,
+      const Maybe<mozilla::ContentBlockingNotifier::CanvasFingerprinter>&
+          aCanvasFingerprinter,
+      const Maybe<bool>& aCanvasFingerprinterKnownText);
 
   mozilla::ipc::IPCResult RecvNavigationFinished();
 
@@ -353,10 +364,11 @@ class BrowserParent final : public PBrowserParent,
       const widget::IMENotification& aEventMessage);
 
   mozilla::ipc::IPCResult RecvOnEventNeedingAckHandled(
-      const EventMessage& aMessage);
+      const EventMessage& aMessage, const uint32_t& aCompositionId);
 
   mozilla::ipc::IPCResult RecvRequestIMEToCommitComposition(
-      const bool& aCancel, bool* aIsCommitted, nsString* aCommittedString);
+      const bool& aCancel, const uint32_t& aCompositionId, bool* aIsCommitted,
+      nsString* aCommittedString);
 
   mozilla::ipc::IPCResult RecvGetInputContext(widget::IMEState* aIMEState);
 
@@ -379,12 +391,9 @@ class BrowserParent final : public PBrowserParent,
       nsTArray<nsCString>&& aDisabledCommands);
 
   mozilla::ipc::IPCResult RecvSetCursor(
-      const nsCursor& aValue, const bool& aHasCustomCursor,
-      Maybe<BigBuffer>&& aCursorData, const uint32_t& aWidth,
-      const uint32_t& aHeight, const float& aResolutionX,
-      const float& aResolutionY, const uint32_t& aStride,
-      const gfx::SurfaceFormat& aFormat, const uint32_t& aHotspotX,
-      const uint32_t& aHotspotY, const bool& aForce);
+      const nsCursor& aValue, Maybe<IPCImage>&& aCustomCursor,
+      const float& aResolutionX, const float& aResolutionY,
+      const uint32_t& aHotspotX, const uint32_t& aHotspotY, const bool& aForce);
 
   mozilla::ipc::IPCResult RecvSetLinkStatus(const nsString& aStatus);
 
@@ -419,9 +428,9 @@ class BrowserParent final : public PBrowserParent,
       const nsString& aTitle, const nsString& aInitialColor,
       const nsTArray<nsString>& aDefaultColors);
 
-  PVsyncParent* AllocPVsyncParent();
+  already_AddRefed<PVsyncParent> AllocPVsyncParent();
 
-  bool DeallocPVsyncParent(PVsyncParent* aActor);
+  mozilla::ipc::IPCResult RecvPVsyncConstructor(PVsyncParent* aActor) override;
 
 #ifdef ACCESSIBILITY
   PDocAccessibleParent* AllocPDocAccessibleParent(
@@ -580,16 +589,19 @@ class BrowserParent final : public PBrowserParent,
    * If you need to check if it's actually posted to the remote process,
    * you can refer aEvent.HasBeenPostedToRemoteProcess().
    */
-  bool SendCompositionEvent(mozilla::WidgetCompositionEvent& aEvent);
+  bool SendCompositionEvent(mozilla::WidgetCompositionEvent& aEvent,
+                            uint32_t aCompositionId);
 
   bool SendSelectionEvent(mozilla::WidgetSelectionEvent& aEvent);
 
   MOZ_CAN_RUN_SCRIPT_BOUNDARY bool SendHandleTap(
       TapType aType, const LayoutDevicePoint& aPoint, Modifiers aModifiers,
-      const ScrollableLayerGuid& aGuid, uint64_t aInputBlockId);
+      const ScrollableLayerGuid& aGuid, uint64_t aInputBlockId,
+      const Maybe<DoubleTapToZoomMetrics>& aDoubleTapToZoomMetrics);
 
   already_AddRefed<PFilePickerParent> AllocPFilePickerParent(
-      const nsString& aTitle, const nsIFilePicker::Mode& aMode);
+      const nsString& aTitle, const nsIFilePicker::Mode& aMode,
+      const MaybeDiscarded<BrowsingContext>& aBrowsingContext);
 
   bool GetGlobalJSObject(JSContext* cx, JSObject** globalp);
 
@@ -601,10 +613,7 @@ class BrowserParent final : public PBrowserParent,
 
   bool SendInsertText(const nsString& aStringToInsert);
 
-  bool SendPasteTransferable(IPCTransferableData&& aTransferableData,
-                             const bool& aIsPrivateData,
-                             nsIPrincipal* aRequestingPrincipal,
-                             const nsContentPolicyType& aContentPolicyType);
+  bool SendPasteTransferable(IPCTransferable&& aTransferable);
 
   // Helper for transforming a point
   LayoutDeviceIntPoint TransformPoint(
@@ -616,6 +625,7 @@ class BrowserParent final : public PBrowserParent,
 
   // Transform a coordinate from the parent process coordinate space to the
   // child process coordinate space.
+  LayoutDeviceIntPoint TransformParentToChild(const WidgetMouseEvent& aEvent);
   LayoutDeviceIntPoint TransformParentToChild(
       const LayoutDeviceIntPoint& aPoint);
   LayoutDevicePoint TransformParentToChild(const LayoutDevicePoint& aPoint);
@@ -658,7 +668,7 @@ class BrowserParent final : public PBrowserParent,
   bool SendLoadRemoteScript(const nsAString& aURL,
                             const bool& aRunInGlobalScope);
 
-  void LayerTreeUpdate(const LayersObserverEpoch& aEpoch, bool aActive);
+  void LayerTreeUpdate(bool aActive);
 
   mozilla::ipc::IPCResult RecvInvokeDragSession(
       nsTArray<IPCTransferableData>&& aTransferables, const uint32_t& aAction,
@@ -669,6 +679,9 @@ class BrowserParent final : public PBrowserParent,
       const MaybeDiscarded<WindowContext>& aSourceWindowContext,
       const MaybeDiscarded<WindowContext>& aSourceTopWindowContext);
 
+  mozilla::ipc::IPCResult RecvUpdateDropEffect(const uint32_t& aDragAction,
+                                               const uint32_t& aDropEffect);
+
   void AddInitialDnDDataTo(IPCTransferableData* aTransferableData,
                            nsIPrincipal** aPrincipal);
 
@@ -706,6 +719,8 @@ class BrowserParent final : public PBrowserParent,
   // Called when the BrowserParent is being destroyed or entering bfcache.
   void Deactivated();
 
+  void MaybeInvokeDragSession(EventMessage aMessage);
+
  protected:
   friend BrowserBridgeParent;
   friend BrowserHost;
@@ -721,9 +736,6 @@ class BrowserParent final : public PBrowserParent,
 
   mozilla::ipc::IPCResult RecvRemoteIsReadyToHandleInputEvents();
 
-  mozilla::ipc::IPCResult RecvPaintWhileInterruptingJSNoOp(
-      const LayersObserverEpoch& aEpoch);
-
   mozilla::ipc::IPCResult RecvSetDimensions(mozilla::DimensionRequest aRequest,
                                             const double& aScale);
 
@@ -753,11 +765,12 @@ class BrowserParent final : public PBrowserParent,
 
   mozilla::ipc::IPCResult RecvShowDynamicToolbar();
 
+  void GetIPCTransferableData(nsIDragSession* aSession,
+                              nsTArray<IPCTransferableData>& aIPCTransferables);
+
  private:
   void SuppressDisplayport(bool aEnabled);
 
-  void DestroyInternal();
-
   void SetRenderLayersInternal(bool aEnabled);
 
   already_AddRefed<nsFrameLoader> GetFrameLoader(
@@ -855,10 +868,8 @@ class BrowserParent final : public PBrowserParent,
  private:
   TabId mTabId;
 
-  RefPtr<ContentParent> mManager;
   // The root browsing context loaded in this BrowserParent.
   RefPtr<CanonicalBrowsingContext> mBrowsingContext;
-  nsCOMPtr<nsILoadContext> mLoadContext;
   RefPtr<Element> mFrameElement;
   nsCOMPtr<nsIBrowserDOMWindow> mBrowserDOMWindow;
   // We keep a strong reference to the frameloader after we've sent the
@@ -880,7 +891,6 @@ class BrowserParent final : public PBrowserParent,
   ContentCacheInParent mContentCache;
 
   layout::RemoteLayerTreeOwner mRemoteLayerTreeOwner;
-  LayersObserverEpoch mLayerTreeEpoch;
 
   Maybe<LayoutDeviceToLayoutDeviceMatrix4x4> mChildToParentConversionMatrix;
   Maybe<ScreenRect> mRemoteDocumentRect;
@@ -943,8 +953,6 @@ class BrowserParent final : public PBrowserParent,
 
   nsTArray<nsString> mVerifyDropLinks;
 
-  RefPtr<VsyncParent> mVsyncParent;
-
 #ifdef DEBUG
   int32_t mActiveSupressDisplayportCount = 0;
 #endif
@@ -996,6 +1004,8 @@ class BrowserParent final : public PBrowserParent,
   bool mShowingTooltip : 1;
 };
 
+NS_DEFINE_STATIC_IID_ACCESSOR(BrowserParent, DOM_BROWSERPARENT_IID)
+
 struct MOZ_STACK_CLASS BrowserParent::AutoUseNewTab final {
  public:
   explicit AutoUseNewTab(BrowserParent* aNewTab) : mNewTab(aNewTab) {
