@@ -7,27 +7,25 @@
 #include "mozilla/dom/MathMLElement.h"
 
 #include "base/compiler_specific.h"
+#include "mozilla/FocusModel.h"
 #include "mozilla/dom/BindContext.h"
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/EventListenerManager.h"
-#include "mozilla/FontPropertyTypes.h"
 #include "mozilla/StaticPrefs_mathml.h"
 #include "mozilla/TextUtils.h"
 #include "nsGkAtoms.h"
 #include "nsIContentInlines.h"
 #include "nsITableCellLayout.h"  // for MAX_COLSPAN / MAX_ROWSPAN
 #include "nsCSSValue.h"
-#include "nsMappedAttributes.h"
 #include "nsStyleConsts.h"
 #include "mozilla/dom/Document.h"
 #include "nsPresContext.h"
-#include "mozAutoDocUpdate.h"
 #include "nsIScriptError.h"
 #include "nsContentUtils.h"
 #include "nsIURI.h"
 
 #include "mozilla/EventDispatcher.h"
-#include "mozilla/MappedDeclarations.h"
+#include "mozilla/MappedDeclarationsBuilder.h"
 #include "mozilla/dom/MathMLElementBinding.h"
 
 using namespace mozilla;
@@ -47,36 +45,28 @@ static nsresult ReportLengthParseError(const nsString& aValue,
 }
 
 static nsresult ReportParseErrorNoTag(const nsString& aValue, nsAtom* aAtom,
-                                      Document* aDocument) {
+                                      Document& aDocument) {
   AutoTArray<nsString, 2> argv = {aValue, nsDependentAtomString(aAtom)};
   return nsContentUtils::ReportToConsole(
-      nsIScriptError::errorFlag, "MathML"_ns, aDocument,
+      nsIScriptError::errorFlag, "MathML"_ns, &aDocument,
       nsContentUtils::eMATHML_PROPERTIES, "AttributeParsingErrorNoTag", argv);
 }
 
 MathMLElement::MathMLElement(
     already_AddRefed<mozilla::dom::NodeInfo>& aNodeInfo)
     : MathMLElementBase(std::move(aNodeInfo)),
-      ALLOW_THIS_IN_INITIALIZER_LIST(Link(this)),
-      mIncrementScriptLevel(false) {}
+      ALLOW_THIS_IN_INITIALIZER_LIST(Link(this)) {}
 
 MathMLElement::MathMLElement(
     already_AddRefed<mozilla::dom::NodeInfo>&& aNodeInfo)
     : MathMLElementBase(std::move(aNodeInfo)),
-      ALLOW_THIS_IN_INITIALIZER_LIST(Link(this)),
-      mIncrementScriptLevel(false) {}
+      ALLOW_THIS_IN_INITIALIZER_LIST(Link(this)) {}
 
 nsresult MathMLElement::BindToTree(BindContext& aContext, nsINode& aParent) {
-  Link::ResetLinkState(false, Link::ElementHasHref());
-
   nsresult rv = MathMLElementBase::BindToTree(aContext, aParent);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  // FIXME(emilio): Probably should be composed, this uses all the other link
-  // infrastructure.
-  if (Document* doc = aContext.GetUncomposedDoc()) {
-    doc->RegisterPendingLinkUpdate(this);
-  }
+  Link::BindToTree(aContext);
 
   // Set the bit in the document for telemetry.
   if (Document* doc = aContext.GetComposedDoc()) {
@@ -86,12 +76,11 @@ nsresult MathMLElement::BindToTree(BindContext& aContext, nsINode& aParent) {
   return rv;
 }
 
-void MathMLElement::UnbindFromTree(bool aNullParent) {
-  // Without removing the link state we risk a dangling pointer
-  // in the mStyledLinks hashtable
-  Link::ResetLinkState(false, Link::ElementHasHref());
-
-  MathMLElementBase::UnbindFromTree(aNullParent);
+void MathMLElement::UnbindFromTree(UnbindContext& aContext) {
+  MathMLElementBase::UnbindFromTree(aContext);
+  // Without removing the link state we risk a dangling pointer in the
+  // mStyledLinks hashtable
+  Link::UnbindFromTree();
 }
 
 bool MathMLElement::ParseAttribute(int32_t aNamespaceID, nsAtom* aAttribute,
@@ -127,10 +116,13 @@ bool MathMLElement::ParseAttribute(int32_t aNamespaceID, nsAtom* aAttribute,
 
 // https://mathml-refresh.github.io/mathml-core/#global-attributes
 static Element::MappedAttributeEntry sGlobalAttributes[] = {
-    {nsGkAtoms::dir},           {nsGkAtoms::mathbackground_},
-    {nsGkAtoms::mathcolor_},    {nsGkAtoms::mathsize_},
-    {nsGkAtoms::mathvariant_},  {nsGkAtoms::scriptlevel_},
-    {nsGkAtoms::displaystyle_}, {nullptr}};
+    {nsGkAtoms::dir},
+    {nsGkAtoms::mathbackground_},
+    {nsGkAtoms::mathcolor_},
+    {nsGkAtoms::mathsize_},
+    {nsGkAtoms::scriptlevel_},
+    {nsGkAtoms::displaystyle_},
+    {nullptr}};
 
 bool MathMLElement::IsAttributeMapped(const nsAtom* aAttribute) const {
   MOZ_ASSERT(IsMathMLElement());
@@ -138,19 +130,22 @@ bool MathMLElement::IsAttributeMapped(const nsAtom* aAttribute) const {
   static const MappedAttributeEntry* const globalMap[] = {sGlobalAttributes};
 
   return FindAttributeDependence(aAttribute, globalMap) ||
-         (!StaticPrefs::mathml_scriptminsize_attribute_disabled() &&
-          aAttribute == nsGkAtoms::scriptminsize_) ||
-         (!StaticPrefs::mathml_scriptsizemultiplier_attribute_disabled() &&
-          aAttribute == nsGkAtoms::scriptsizemultiplier_) ||
+         ((!StaticPrefs::mathml_legacy_mathvariant_attribute_disabled() ||
+           mNodeInfo->Equals(nsGkAtoms::mi_)) &&
+          aAttribute == nsGkAtoms::mathvariant_) ||
          (mNodeInfo->Equals(nsGkAtoms::mtable_) &&
           aAttribute == nsGkAtoms::width);
 }
 
 nsMapRuleToAttributesFunc MathMLElement::GetAttributeMappingFunction() const {
-  // It doesn't really matter what our tag is here, because only attributes
-  // that satisfy IsAttributeMapped will be stored in the mapped attributes
-  // list and available to the mapping function
-  return &MapMathMLAttributesInto;
+  if (mNodeInfo->Equals(nsGkAtoms::mtable_)) {
+    return &MapMTableAttributesInto;
+  }
+  if (StaticPrefs::mathml_legacy_mathvariant_attribute_disabled() &&
+      mNodeInfo->Equals(nsGkAtoms::mi_)) {
+    return &MapMiAttributesInto;
+  }
+  return &MapGlobalMathMLAttributesInto;
 }
 
 /* static */
@@ -194,8 +189,11 @@ bool MathMLElement::ParseNamedSpaceValue(const nsString& aString,
     }
   }
   if (0 != i) {
+    AutoTArray<nsString, 1> params;
+    params.AppendElement(aString);
     aDocument.WarnOnceAbout(
-        dom::DeprecatedOperations::eMathML_DeprecatedMathSpaceValue);
+        dom::DeprecatedOperations::eMathML_DeprecatedMathSpaceValue2, false,
+        params);
     aCSSValue.SetFloatValue(float(i) / float(18), eCSSUnit_EM);
     return true;
   }
@@ -356,193 +354,8 @@ bool MathMLElement::ParseNumericValue(const nsString& aString,
   return true;
 }
 
-void MathMLElement::MapMathMLAttributesInto(
-    const nsMappedAttributes* aAttributes, MappedDeclarations& aDecls) {
-  // scriptsizemultiplier
-  //
-  // "Specifies the multiplier to be used to adjust font size due to changes
-  // in scriptlevel.
-  //
-  // values: number
-  // default: 0.71
-  //
-  const nsAttrValue* value =
-      aAttributes->GetAttr(nsGkAtoms::scriptsizemultiplier_);
-  if (value && value->Type() == nsAttrValue::eString &&
-      !aDecls.PropertyIsSet(eCSSProperty__moz_script_size_multiplier)) {
-    aDecls.Document()->WarnOnceAbout(
-        dom::DeprecatedOperations::
-            eMathML_DeprecatedScriptsizemultiplierAttribute);
-    auto str = value->GetStringValue();
-    str.CompressWhitespace();
-    // MathML numbers can't have leading '+'
-    if (str.Length() > 0 && str.CharAt(0) != '+') {
-      nsresult errorCode;
-      float floatValue = str.ToFloat(&errorCode);
-      // Negative scriptsizemultipliers are not parsed
-      if (NS_SUCCEEDED(errorCode) && floatValue >= 0.0f) {
-        aDecls.SetNumberValue(eCSSProperty__moz_script_size_multiplier,
-                              floatValue);
-      } else {
-        ReportParseErrorNoTag(str, nsGkAtoms::scriptsizemultiplier_,
-                              aDecls.Document());
-      }
-    }
-  }
-
-  // scriptminsize
-  //
-  // "Specifies the minimum font size allowed due to changes in scriptlevel.
-  // Note that this does not limit the font size due to changes to mathsize."
-  //
-  // values: length
-  // default: 8pt
-  //
-  // We don't allow negative values.
-  // Unitless and percent values give a multiple of the default value.
-  //
-  value = aAttributes->GetAttr(nsGkAtoms::scriptminsize_);
-  if (value && value->Type() == nsAttrValue::eString &&
-      !aDecls.PropertyIsSet(eCSSProperty__moz_script_min_size)) {
-    aDecls.Document()->WarnOnceAbout(
-        dom::DeprecatedOperations::eMathML_DeprecatedScriptminsizeAttribute);
-    nsCSSValue scriptMinSize;
-    ParseNumericValue(value->GetStringValue(), scriptMinSize, 0,
-                      aDecls.Document());
-
-    if (scriptMinSize.GetUnit() == eCSSUnit_Percent) {
-      scriptMinSize.SetFloatValue(8.0 * scriptMinSize.GetPercentValue(),
-                                  eCSSUnit_Point);
-    }
-    if (scriptMinSize.GetUnit() != eCSSUnit_Null) {
-      aDecls.SetLengthValue(eCSSProperty__moz_script_min_size, scriptMinSize);
-    }
-  }
-
-  // scriptlevel
-  //
-  // "Changes the scriptlevel in effect for the children. When the value is
-  // given without a sign, it sets scriptlevel to the specified value; when a
-  // sign is given, it increments ("+") or decrements ("-") the current
-  // value. (Note that large decrements can result in negative values of
-  // scriptlevel, but these values are considered legal.)"
-  //
-  // values: ( "+" | "-" )? unsigned-integer
-  // default: inherited
-  //
-  value = aAttributes->GetAttr(nsGkAtoms::scriptlevel_);
-  if (value && value->Type() == nsAttrValue::eString &&
-      !aDecls.PropertyIsSet(eCSSProperty_math_depth)) {
-    auto str = value->GetStringValue();
-    str.CompressWhitespace();
-    if (str.Length() > 0) {
-      nsresult errorCode;
-      int32_t intValue = str.ToInteger(&errorCode);
-      if (NS_SUCCEEDED(errorCode)) {
-        char16_t ch = str.CharAt(0);
-        bool isRelativeScriptLevel = (ch == '+' || ch == '-');
-        aDecls.SetMathDepthValue(intValue, isRelativeScriptLevel);
-      } else {
-        ReportParseErrorNoTag(str, nsGkAtoms::scriptlevel_, aDecls.Document());
-      }
-    }
-  }
-
-  // mathsize
-  // https://w3c.github.io/mathml-core/#dfn-mathsize
-  value = aAttributes->GetAttr(nsGkAtoms::mathsize_);
-  if (value && value->Type() == nsAttrValue::eString &&
-      !aDecls.PropertyIsSet(eCSSProperty_font_size)) {
-    auto str = value->GetStringValue();
-    nsCSSValue fontSize;
-    ParseNumericValue(str, fontSize, 0, nullptr);
-    if (fontSize.GetUnit() == eCSSUnit_Percent) {
-      aDecls.SetPercentValue(eCSSProperty_font_size,
-                             fontSize.GetPercentValue());
-    } else if (fontSize.GetUnit() != eCSSUnit_Null) {
-      aDecls.SetLengthValue(eCSSProperty_font_size, fontSize);
-    }
-  }
-
-  // mathvariant
-  //
-  // "Specifies the logical class of the token. Note that this class is more
-  // than styling, it typically conveys semantic intent;"
-  //
-  // values: "normal" | "bold" | "italic" | "bold-italic" | "double-struck" |
-  // "bold-fraktur" | "script" | "bold-script" | "fraktur" | "sans-serif" |
-  // "bold-sans-serif" | "sans-serif-italic" | "sans-serif-bold-italic" |
-  // "monospace" | "initial" | "tailed" | "looped" | "stretched"
-  // default: normal (except on <mi>)
-  //
-  value = aAttributes->GetAttr(nsGkAtoms::mathvariant_);
-  if (value && value->Type() == nsAttrValue::eString &&
-      !aDecls.PropertyIsSet(eCSSProperty__moz_math_variant)) {
-    auto str = value->GetStringValue();
-    str.CompressWhitespace();
-    static const char sizes[19][23] = {"normal",
-                                       "bold",
-                                       "italic",
-                                       "bold-italic",
-                                       "script",
-                                       "bold-script",
-                                       "fraktur",
-                                       "double-struck",
-                                       "bold-fraktur",
-                                       "sans-serif",
-                                       "bold-sans-serif",
-                                       "sans-serif-italic",
-                                       "sans-serif-bold-italic",
-                                       "monospace",
-                                       "initial",
-                                       "tailed",
-                                       "looped",
-                                       "stretched"};
-    static const StyleMathVariant values[MOZ_ARRAY_LENGTH(sizes)] = {
-        StyleMathVariant::Normal,
-        StyleMathVariant::Bold,
-        StyleMathVariant::Italic,
-        StyleMathVariant::BoldItalic,
-        StyleMathVariant::Script,
-        StyleMathVariant::BoldScript,
-        StyleMathVariant::Fraktur,
-        StyleMathVariant::DoubleStruck,
-        StyleMathVariant::BoldFraktur,
-        StyleMathVariant::SansSerif,
-        StyleMathVariant::BoldSansSerif,
-        StyleMathVariant::SansSerifItalic,
-        StyleMathVariant::SansSerifBoldItalic,
-        StyleMathVariant::Monospace,
-        StyleMathVariant::Initial,
-        StyleMathVariant::Tailed,
-        StyleMathVariant::Looped,
-        StyleMathVariant::Stretched};
-    for (uint32_t i = 0; i < ArrayLength(sizes); ++i) {
-      if (str.LowerCaseEqualsASCII(sizes[i])) {
-        aDecls.SetKeywordValue(eCSSProperty__moz_math_variant, values[i]);
-        break;
-      }
-    }
-  }
-
-  // mathbackground
-  // https://w3c.github.io/mathml-core/#dfn-mathbackground
-  value = aAttributes->GetAttr(nsGkAtoms::mathbackground_);
-  if (value) {
-    nscolor color;
-    if (value->GetColorValue(color)) {
-      aDecls.SetColorValueIfUnset(eCSSProperty_background_color, color);
-    }
-  }
-
-  // mathcolor
-  // https://w3c.github.io/mathml-core/#dfn-mathcolor
-  value = aAttributes->GetAttr(nsGkAtoms::mathcolor_);
-  nscolor color;
-  if (value && value->GetColorValue(color)) {
-    aDecls.SetColorValueIfUnset(eCSSProperty_color, color);
-  }
-
+void MathMLElement::MapMTableAttributesInto(
+    MappedDeclarationsBuilder& aBuilder) {
   // width
   //
   // "Specifies the desired width of the entire table and is intended for
@@ -555,32 +368,201 @@ void MathMLElement::MapMathMLAttributesInto(
   // values: "auto" | length
   // default: auto
   //
-  if (!aDecls.PropertyIsSet(eCSSProperty_width)) {
-    const nsAttrValue* value = aAttributes->GetAttr(nsGkAtoms::width);
+  if (!aBuilder.PropertyIsSet(eCSSProperty_width)) {
+    const nsAttrValue* value = aBuilder.GetAttr(nsGkAtoms::width);
     nsCSSValue width;
     // This does not handle auto and unitless values
     if (value && value->Type() == nsAttrValue::eString) {
-      ParseNumericValue(value->GetStringValue(), width, 0, aDecls.Document());
+      ParseNumericValue(value->GetStringValue(), width, 0,
+                        &aBuilder.Document());
       if (width.GetUnit() == eCSSUnit_Percent) {
-        aDecls.SetPercentValue(eCSSProperty_width, width.GetPercentValue());
+        aBuilder.SetPercentValue(eCSSProperty_width, width.GetPercentValue());
       } else if (width.GetUnit() != eCSSUnit_Null) {
-        aDecls.SetLengthValue(eCSSProperty_width, width);
+        aBuilder.SetLengthValue(eCSSProperty_width, width);
+      }
+    }
+  }
+  MapGlobalMathMLAttributesInto(aBuilder);
+}
+
+void MathMLElement::MapMiAttributesInto(MappedDeclarationsBuilder& aBuilder) {
+  // mathvariant
+  // https://w3c.github.io/mathml-core/#dfn-mathvariant
+  if (!aBuilder.PropertyIsSet(eCSSProperty_text_transform)) {
+    const nsAttrValue* value = aBuilder.GetAttr(nsGkAtoms::mathvariant_);
+    if (value && value->Type() == nsAttrValue::eString) {
+      auto str = value->GetStringValue();
+      str.CompressWhitespace();
+      if (value->GetStringValue().LowerCaseEqualsASCII("normal")) {
+        aBuilder.SetKeywordValue(eCSSProperty_text_transform,
+                                 StyleTextTransform::NONE._0);
+      }
+    }
+  }
+  MapGlobalMathMLAttributesInto(aBuilder);
+}
+
+void MathMLElement::MapGlobalMathMLAttributesInto(
+    MappedDeclarationsBuilder& aBuilder) {
+  // scriptlevel
+  // https://w3c.github.io/mathml-core/#dfn-scriptlevel
+  const nsAttrValue* value = aBuilder.GetAttr(nsGkAtoms::scriptlevel_);
+  if (value && value->Type() == nsAttrValue::eString &&
+      !aBuilder.PropertyIsSet(eCSSProperty_math_depth)) {
+    auto str = value->GetStringValue();
+    // FIXME: Should we remove whitespace trimming?
+    // See https://github.com/w3c/mathml/issues/122
+    str.CompressWhitespace();
+    if (str.Length() > 0) {
+      nsresult errorCode;
+      int32_t intValue = str.ToInteger(&errorCode);
+      bool reportParseError = true;
+      if (NS_SUCCEEDED(errorCode)) {
+        char16_t ch = str.CharAt(0);
+        bool isRelativeScriptLevel = (ch == '+' || ch == '-');
+        // ToInteger is not very strict, check this is really <unsigned>.
+        reportParseError = false;
+        for (uint32_t i = isRelativeScriptLevel ? 1 : 0; i < str.Length();
+             i++) {
+          if (!IsAsciiDigit(str.CharAt(i))) {
+            reportParseError = true;
+            break;
+          }
+        }
+        if (!reportParseError) {
+          aBuilder.SetMathDepthValue(intValue, isRelativeScriptLevel);
+        }
+      }
+      if (reportParseError) {
+        ReportParseErrorNoTag(str, nsGkAtoms::scriptlevel_,
+                              aBuilder.Document());
       }
     }
   }
 
+  // mathsize
+  // https://w3c.github.io/mathml-core/#dfn-mathsize
+  value = aBuilder.GetAttr(nsGkAtoms::mathsize_);
+  if (value && value->Type() == nsAttrValue::eString &&
+      !aBuilder.PropertyIsSet(eCSSProperty_font_size)) {
+    auto str = value->GetStringValue();
+    nsCSSValue fontSize;
+    ParseNumericValue(str, fontSize, 0, nullptr);
+    if (fontSize.GetUnit() == eCSSUnit_Percent) {
+      aBuilder.SetPercentValue(eCSSProperty_font_size,
+                               fontSize.GetPercentValue());
+    } else if (fontSize.GetUnit() != eCSSUnit_Null) {
+      aBuilder.SetLengthValue(eCSSProperty_font_size, fontSize);
+    }
+  }
+
+  if (!StaticPrefs::mathml_legacy_mathvariant_attribute_disabled()) {
+    // mathvariant
+    //
+    // "Specifies the logical class of the token. Note that this class is more
+    // than styling, it typically conveys semantic intent;"
+    //
+    // values: "normal" | "bold" | "italic" | "bold-italic" | "double-struck" |
+    // "bold-fraktur" | "script" | "bold-script" | "fraktur" | "sans-serif" |
+    // "bold-sans-serif" | "sans-serif-italic" | "sans-serif-bold-italic" |
+    // "monospace" | "initial" | "tailed" | "looped" | "stretched"
+    // default: normal (except on <mi>)
+    //
+    value = aBuilder.GetAttr(nsGkAtoms::mathvariant_);
+    if (value && value->Type() == nsAttrValue::eString &&
+        !aBuilder.PropertyIsSet(eCSSProperty__moz_math_variant)) {
+      auto str = value->GetStringValue();
+      str.CompressWhitespace();
+      static const char sizes[19][23] = {"normal",
+                                         "bold",
+                                         "italic",
+                                         "bold-italic",
+                                         "script",
+                                         "bold-script",
+                                         "fraktur",
+                                         "double-struck",
+                                         "bold-fraktur",
+                                         "sans-serif",
+                                         "bold-sans-serif",
+                                         "sans-serif-italic",
+                                         "sans-serif-bold-italic",
+                                         "monospace",
+                                         "initial",
+                                         "tailed",
+                                         "looped",
+                                         "stretched"};
+      static const StyleMathVariant values[MOZ_ARRAY_LENGTH(sizes)] = {
+          StyleMathVariant::Normal,
+          StyleMathVariant::Bold,
+          StyleMathVariant::Italic,
+          StyleMathVariant::BoldItalic,
+          StyleMathVariant::Script,
+          StyleMathVariant::BoldScript,
+          StyleMathVariant::Fraktur,
+          StyleMathVariant::DoubleStruck,
+          StyleMathVariant::BoldFraktur,
+          StyleMathVariant::SansSerif,
+          StyleMathVariant::BoldSansSerif,
+          StyleMathVariant::SansSerifItalic,
+          StyleMathVariant::SansSerifBoldItalic,
+          StyleMathVariant::Monospace,
+          StyleMathVariant::Initial,
+          StyleMathVariant::Tailed,
+          StyleMathVariant::Looped,
+          StyleMathVariant::Stretched};
+      for (uint32_t i = 0; i < ArrayLength(sizes); ++i) {
+        if (str.LowerCaseEqualsASCII(sizes[i])) {
+          if (values[i] != StyleMathVariant::Normal) {
+            // Warn about deprecated mathvariant attribute values. Strictly
+            // speaking, we should also warn about mathvariant="normal" if the
+            // element is not an <mi>. However this would require exposing the
+            // tag name via aBuilder. Moreover, this use case is actually to
+            // revert the effect of a non-normal mathvariant value on an
+            // ancestor element, which should consequently have already
+            // triggered a warning.
+            AutoTArray<nsString, 1> params;
+            params.AppendElement(str);
+            aBuilder.Document().WarnOnceAbout(
+                dom::DeprecatedOperations::eMathML_DeprecatedMathVariant, false,
+                params);
+          }
+          aBuilder.SetKeywordValue(eCSSProperty__moz_math_variant, values[i]);
+          break;
+        }
+      }
+    }
+  }
+
+  // mathbackground
+  // https://w3c.github.io/mathml-core/#dfn-mathbackground
+  value = aBuilder.GetAttr(nsGkAtoms::mathbackground_);
+  if (value) {
+    nscolor color;
+    if (value->GetColorValue(color)) {
+      aBuilder.SetColorValueIfUnset(eCSSProperty_background_color, color);
+    }
+  }
+
+  // mathcolor
+  // https://w3c.github.io/mathml-core/#dfn-mathcolor
+  value = aBuilder.GetAttr(nsGkAtoms::mathcolor_);
+  nscolor color;
+  if (value && value->GetColorValue(color)) {
+    aBuilder.SetColorValueIfUnset(eCSSProperty_color, color);
+  }
+
   // dir
   // https://w3c.github.io/mathml-core/#dfn-dir
-  value = aAttributes->GetAttr(nsGkAtoms::dir);
+  value = aBuilder.GetAttr(nsGkAtoms::dir);
   if (value && value->Type() == nsAttrValue::eString &&
-      !aDecls.PropertyIsSet(eCSSProperty_direction)) {
+      !aBuilder.PropertyIsSet(eCSSProperty_direction)) {
     auto str = value->GetStringValue();
     static const char dirs[][4] = {"ltr", "rtl"};
     static const StyleDirection dirValues[MOZ_ARRAY_LENGTH(dirs)] = {
         StyleDirection::Ltr, StyleDirection::Rtl};
     for (uint32_t i = 0; i < ArrayLength(dirs); ++i) {
       if (str.LowerCaseEqualsASCII(dirs[i])) {
-        aDecls.SetKeywordValue(eCSSProperty_direction, dirValues[i]);
+        aBuilder.SetKeywordValue(eCSSProperty_direction, dirValues[i]);
         break;
       }
     }
@@ -588,16 +570,16 @@ void MathMLElement::MapMathMLAttributesInto(
 
   // displaystyle
   // https://mathml-refresh.github.io/mathml-core/#dfn-displaystyle
-  value = aAttributes->GetAttr(nsGkAtoms::displaystyle_);
+  value = aBuilder.GetAttr(nsGkAtoms::displaystyle_);
   if (value && value->Type() == nsAttrValue::eString &&
-      !aDecls.PropertyIsSet(eCSSProperty_math_style)) {
+      !aBuilder.PropertyIsSet(eCSSProperty_math_style)) {
     auto str = value->GetStringValue();
     static const char displaystyles[][6] = {"false", "true"};
     static const StyleMathStyle mathStyle[MOZ_ARRAY_LENGTH(displaystyles)] = {
         StyleMathStyle::Compact, StyleMathStyle::Normal};
     for (uint32_t i = 0; i < ArrayLength(displaystyles); ++i) {
       if (str.LowerCaseEqualsASCII(displaystyles[i])) {
-        aDecls.SetKeywordValue(eCSSProperty_math_style, mathStyle[i]);
+        aBuilder.SetKeywordValue(eCSSProperty_math_style, mathStyle[i]);
         break;
       }
     }
@@ -616,62 +598,49 @@ nsresult MathMLElement::PostHandleEvent(EventChainPostVisitor& aVisitor) {
 
 NS_IMPL_ELEMENT_CLONE(MathMLElement)
 
-ElementState MathMLElement::IntrinsicState() const {
-  return Link::LinkState() | MathMLElementBase::IntrinsicState() |
-         (mIncrementScriptLevel ? ElementState::INCREMENT_SCRIPT_LEVEL
-                                : ElementState());
-}
-
 void MathMLElement::SetIncrementScriptLevel(bool aIncrementScriptLevel,
                                             bool aNotify) {
-  if (aIncrementScriptLevel == mIncrementScriptLevel) return;
-  mIncrementScriptLevel = aIncrementScriptLevel;
-
   NS_ASSERTION(aNotify, "We always notify!");
-
-  UpdateState(true);
+  if (aIncrementScriptLevel) {
+    AddStates(ElementState::INCREMENT_SCRIPT_LEVEL);
+  } else {
+    RemoveStates(ElementState::INCREMENT_SCRIPT_LEVEL);
+  }
 }
 
 int32_t MathMLElement::TabIndexDefault() { return IsLink() ? 0 : -1; }
 
 // XXX Bug 1586011: Share logic with other element classes.
-bool MathMLElement::IsFocusableInternal(int32_t* aTabIndex, bool aWithMouse) {
+Focusable MathMLElement::IsFocusableWithoutStyle(IsFocusableFlags) {
   if (!IsInComposedDoc() || IsInDesignMode()) {
     // In designMode documents we only allow focusing the document.
-    if (aTabIndex) {
-      *aTabIndex = -1;
-    }
-    return false;
+    return {};
   }
 
   int32_t tabIndex = TabIndex();
-  if (aTabIndex) {
-    *aTabIndex = tabIndex;
-  }
-
   if (!IsLink()) {
     // If a tabindex is specified at all we're focusable
-    return GetTabIndexAttrValue().isSome();
+    if (GetTabIndexAttrValue().isSome()) {
+      return {true, tabIndex};
+    }
+    return {};
   }
 
   if (!OwnerDoc()->LinkHandlingEnabled()) {
-    return false;
+    return {};
   }
 
   // Links that are in an editable region should never be focusable, even if
   // they are in a contenteditable="false" region.
   if (nsContentUtils::IsNodeInEditableRegion(this)) {
-    if (aTabIndex) {
-      *aTabIndex = -1;
-    }
-    return false;
+    return {};
   }
 
-  if (aTabIndex && (sTabFocusModel & eTabFocus_linksMask) == 0) {
-    *aTabIndex = -1;
+  if (!FocusModel::IsTabFocusable(TabFocusableType::Links)) {
+    tabIndex = -1;
   }
 
-  return true;
+  return {true, tabIndex};
 }
 
 already_AddRefed<nsIURI> MathMLElement::GetHrefURI() const {
