@@ -6,9 +6,8 @@
 
 #include "DecoderTraits.h"
 #include "MediaContainerType.h"
-#include "nsMimeTypes.h"
+#include "mozilla/glean/GleanMetrics.h"
 #include "mozilla/Preferences.h"
-#include "mozilla/Telemetry.h"
 
 #include "OggDecoder.h"
 #include "OggDemuxer.h"
@@ -17,12 +16,10 @@
 #include "WebMDemuxer.h"
 
 #ifdef MOZ_ANDROID_HLS_SUPPORT
-#include "HLSDecoder.h"
+#  include "HLSDecoder.h"
 #endif
-#ifdef MOZ_FMP4
 #include "MP4Decoder.h"
 #include "MP4Demuxer.h"
-#endif
 #include "MediaFormatReader.h"
 
 #include "MP3Decoder.h"
@@ -37,13 +34,10 @@
 #include "FlacDecoder.h"
 #include "FlacDemuxer.h"
 
-#include "nsPluginHost.h"
-#include "MediaPrefs.h"
-
 namespace mozilla {
 
-/* static */ bool DecoderTraits::IsHttpLiveStreamingType(
-    const MediaContainerType& aType) {
+/* static */
+bool DecoderTraits::IsHttpLiveStreamingType(const MediaContainerType& aType) {
   const auto& mimeType = aType.Type();
   return  // For m3u8.
           // https://tools.ietf.org/html/draft-pantos-http-live-streaming-19#section-10
@@ -54,23 +48,14 @@ namespace mozilla {
       mimeType == MEDIAMIMETYPE("audio/x-mpegurl");
 }
 
-/* static */ bool DecoderTraits::IsMatroskaType(
-    const MediaContainerType& aType) {
+/* static */
+bool DecoderTraits::IsMatroskaType(const MediaContainerType& aType) {
   const auto& mimeType = aType.Type();
   // https://matroska.org/technical/specs/notes.html
   return mimeType == MEDIAMIMETYPE("audio/x-matroska") ||
          mimeType == MEDIAMIMETYPE("video/x-matroska");
 }
 
-/* static */ bool DecoderTraits::IsMP4SupportedType(
-    const MediaContainerType& aType, DecoderDoctorDiagnostics* aDiagnostics) {
-#ifdef MOZ_FMP4
-  return MP4Decoder::IsSupportedType(aType, aDiagnostics);
-#else
-  return false;
-#endif
-}
-
 static CanPlayStatus CanHandleCodecsType(
     const MediaContainerType& aType, DecoderDoctorDiagnostics* aDiagnostics) {
   // We should have been given a codecs string, though it may be empty.
@@ -91,8 +76,8 @@ static CanPlayStatus CanHandleCodecsType(
     if (WaveDecoder::IsSupportedType(aType)) {
       return CANPLAY_YES;
     }
-    // We can only reach this position if a particular codec was requested,
-    // ogg is supported and working: the codec must be invalid.
+    // We can only reach this position if a particular codec was requested, wave
+    // is supported and working: the codec must be invalid or not supported.
     return CANPLAY_NO;
   }
   if (WebMDecoder::IsSupportedType(mimeType)) {
@@ -103,7 +88,6 @@ static CanPlayStatus CanHandleCodecsType(
     // webm is supported and working: the codec must be invalid.
     return CANPLAY_NO;
   }
-#ifdef MOZ_FMP4
   if (MP4Decoder::IsSupportedType(mimeType,
                                   /* DecoderDoctorDiagnostics* */ nullptr)) {
     if (MP4Decoder::IsSupportedType(aType, aDiagnostics)) {
@@ -113,15 +97,29 @@ static CanPlayStatus CanHandleCodecsType(
     // fmp4 is supported and working: the codec must be invalid.
     return CANPLAY_NO;
   }
-#endif
-  if (MP3Decoder::IsSupportedType(aType)) {
-    return CANPLAY_YES;
+  if (MP3Decoder::IsSupportedType(mimeType)) {
+    if (MP3Decoder::IsSupportedType(aType)) {
+      return CANPLAY_YES;
+    }
+    // We can only reach this position if a particular codec was requested,
+    // mp3 is supported and working: the codec must be invalid.
+    return CANPLAY_NO;
   }
-  if (ADTSDecoder::IsSupportedType(aType)) {
-    return CANPLAY_YES;
+  if (ADTSDecoder::IsSupportedType(mimeType)) {
+    if (ADTSDecoder::IsSupportedType(aType)) {
+      return CANPLAY_YES;
+    }
+    // We can only reach this position if a particular codec was requested,
+    // adts is supported and working: the codec must be invalid.
+    return CANPLAY_NO;
   }
-  if (FlacDecoder::IsSupportedType(aType)) {
-    return CANPLAY_YES;
+  if (FlacDecoder::IsSupportedType(mimeType)) {
+    if (FlacDecoder::IsSupportedType(aType)) {
+      return CANPLAY_YES;
+    }
+    // We can only reach this position if a particular codec was requested,
+    // flac is supported and working: the codec must be invalid.
+    return CANPLAY_NO;
   }
 
   return CANPLAY_MAYBE;
@@ -129,17 +127,17 @@ static CanPlayStatus CanHandleCodecsType(
 
 static CanPlayStatus CanHandleMediaType(
     const MediaContainerType& aType, DecoderDoctorDiagnostics* aDiagnostics) {
-  MOZ_ASSERT(NS_IsMainThread());
-
+  if (DecoderTraits::IsHttpLiveStreamingType(aType)) {
+    glean::hls::canplay_requested.Add();
+  }
 #ifdef MOZ_ANDROID_HLS_SUPPORT
   if (HLSDecoder::IsSupportedType(aType)) {
+    glean::hls::canplay_supported.Add();
     return CANPLAY_MAYBE;
   }
 #endif
 
-  if (DecoderTraits::IsHttpLiveStreamingType(aType)) {
-    Telemetry::Accumulate(Telemetry::MEDIA_HLS_CANPLAY_REQUESTED, true);
-  } else if (DecoderTraits::IsMatroskaType(aType)) {
+  if (DecoderTraits::IsMatroskaType(aType)) {
     Telemetry::Accumulate(Telemetry::MEDIA_MKV_CANPLAY_REQUESTED, true);
   }
 
@@ -159,11 +157,9 @@ static CanPlayStatus CanHandleMediaType(
   if (WaveDecoder::IsSupportedType(mimeType)) {
     return CANPLAY_MAYBE;
   }
-#ifdef MOZ_FMP4
   if (MP4Decoder::IsSupportedType(mimeType, aDiagnostics)) {
     return CANPLAY_MAYBE;
   }
-#endif
   if (WebMDecoder::IsSupportedType(mimeType)) {
     return CANPLAY_MAYBE;
   }
@@ -203,75 +199,56 @@ bool DecoderTraits::ShouldHandleMediaType(
     return false;
   }
 
-  // If an external plugin which can handle quicktime video is available
-  // (and not disabled), prefer it over native playback as there several
-  // codecs found in the wild that we do not handle.
-  if (containerType->Type() == MEDIAMIMETYPE("video/quicktime")) {
-    RefPtr<nsPluginHost> pluginHost = nsPluginHost::GetInst();
-    if (pluginHost &&
-        pluginHost->HavePluginForType(containerType->Type().AsString())) {
-      return false;
-    }
+  return CanHandleMediaType(*containerType, aDiagnostics) != CANPLAY_NO;
+}
+
+/* static */
+already_AddRefed<MediaDataDemuxer> DecoderTraits::CreateDemuxer(
+    const MediaContainerType& aType, MediaResource* aResource) {
+  MOZ_ASSERT(NS_IsMainThread());
+  RefPtr<MediaDataDemuxer> demuxer;
+
+  if (MP4Decoder::IsSupportedType(aType,
+                                  /* DecoderDoctorDiagnostics* */ nullptr)) {
+    demuxer = new MP4Demuxer(aResource);
+  } else if (MP3Decoder::IsSupportedType(aType)) {
+    demuxer = new MP3Demuxer(aResource);
+  } else if (ADTSDecoder::IsSupportedType(aType)) {
+    demuxer = new ADTSDemuxer(aResource);
+  } else if (WaveDecoder::IsSupportedType(aType)) {
+    demuxer = new WAVDemuxer(aResource);
+  } else if (FlacDecoder::IsSupportedType(aType)) {
+    demuxer = new FlacDemuxer(aResource);
+  } else if (OggDecoder::IsSupportedType(aType)) {
+    demuxer = new OggDemuxer(aResource);
+  } else if (WebMDecoder::IsSupportedType(aType)) {
+    demuxer = new WebMDemuxer(aResource);
   }
 
-  return CanHandleMediaType(*containerType, aDiagnostics) != CANPLAY_NO;
+  return demuxer.forget();
 }
 
 /* static */
 MediaFormatReader* DecoderTraits::CreateReader(const MediaContainerType& aType,
                                                MediaFormatReaderInit& aInit) {
   MOZ_ASSERT(NS_IsMainThread());
-  MediaFormatReader* decoderReader = nullptr;
-  MediaResource* resource = aInit.mResource;
 
-#ifdef MOZ_FMP4
-  if (MP4Decoder::IsSupportedType(aType,
-                                  /* DecoderDoctorDiagnostics* */ nullptr)) {
-    decoderReader = new MediaFormatReader(aInit, new MP4Demuxer(resource));
-  } else
-#endif
-      if (MP3Decoder::IsSupportedType(aType)) {
-    decoderReader = new MediaFormatReader(aInit, new MP3Demuxer(resource));
-  } else if (ADTSDecoder::IsSupportedType(aType)) {
-    decoderReader = new MediaFormatReader(aInit, new ADTSDemuxer(resource));
-  } else if (WaveDecoder::IsSupportedType(aType)) {
-    decoderReader = new MediaFormatReader(aInit, new WAVDemuxer(resource));
-  } else if (FlacDecoder::IsSupportedType(aType)) {
-    decoderReader = new MediaFormatReader(aInit, new FlacDemuxer(resource));
-  } else if (OggDecoder::IsSupportedType(aType)) {
-    RefPtr<OggDemuxer> demuxer = new OggDemuxer(resource);
-    decoderReader = new MediaFormatReader(aInit, demuxer);
-    demuxer->SetChainingEvents(&decoderReader->TimedMetadataProducer(),
-                               &decoderReader->MediaNotSeekableProducer());
-  } else if (WebMDecoder::IsSupportedType(aType)) {
-    decoderReader = new MediaFormatReader(aInit, new WebMDemuxer(resource));
+  RefPtr<MediaDataDemuxer> demuxer = CreateDemuxer(aType, aInit.mResource);
+  if (!demuxer) {
+    return nullptr;
+  }
+
+  MediaFormatReader* decoderReader = new MediaFormatReader(aInit, demuxer);
+
+  if (OggDecoder::IsSupportedType(aType)) {
+    static_cast<OggDemuxer*>(demuxer.get())
+        ->SetChainingEvents(&decoderReader->TimedMetadataProducer(),
+                            &decoderReader->MediaNotSeekableProducer());
   }
 
   return decoderReader;
 }
 
-/* static */
-bool DecoderTraits::IsSupportedType(const MediaContainerType& aType) {
-  typedef bool (*IsSupportedFunction)(const MediaContainerType& aType);
-  static const IsSupportedFunction funcs[] = {
-      &ADTSDecoder::IsSupportedType,
-      &FlacDecoder::IsSupportedType,
-      &MP3Decoder::IsSupportedType,
-#ifdef MOZ_FMP4
-      &MP4Decoder::IsSupportedTypeWithoutDiagnostics,
-#endif
-      &OggDecoder::IsSupportedType,
-      &WaveDecoder::IsSupportedType,
-      &WebMDecoder::IsSupportedType,
-  };
-  for (IsSupportedFunction func : funcs) {
-    if (func(aType)) {
-      return true;
-    }
-  }
-  return false;
-}
-
 /* static */
 bool DecoderTraits::IsSupportedInVideoDocument(const nsACString& aType) {
   // Forbid playing media in video documents if the user has opted
@@ -289,10 +266,8 @@ bool DecoderTraits::IsSupportedInVideoDocument(const nsACString& aType) {
 
   return OggDecoder::IsSupportedType(*type) ||
          WebMDecoder::IsSupportedType(*type) ||
-#ifdef MOZ_FMP4
          MP4Decoder::IsSupportedType(*type,
                                      /* DecoderDoctorDiagnostics* */ nullptr) ||
-#endif
          MP3Decoder::IsSupportedType(*type) ||
          ADTSDecoder::IsSupportedType(*type) ||
          FlacDecoder::IsSupportedType(*type) ||
@@ -303,9 +278,33 @@ bool DecoderTraits::IsSupportedInVideoDocument(const nsACString& aType) {
 }
 
 /* static */
-bool DecoderTraits::CrossOriginRedirectsProhibited(
+nsTArray<UniquePtr<TrackInfo>> DecoderTraits::GetTracksInfo(
     const MediaContainerType& aType) {
-  return WaveDecoder::IsSupportedType(aType);
+  // Container type with just the MIME type/subtype, no codecs.
+  const MediaContainerType mimeType(aType.Type());
+
+  if (OggDecoder::IsSupportedType(mimeType)) {
+    return OggDecoder::GetTracksInfo(aType);
+  }
+  if (WaveDecoder::IsSupportedType(mimeType)) {
+    return WaveDecoder::GetTracksInfo(aType);
+  }
+  if (MP4Decoder::IsSupportedType(mimeType, nullptr)) {
+    return MP4Decoder::GetTracksInfo(aType);
+  }
+  if (WebMDecoder::IsSupportedType(mimeType)) {
+    return WebMDecoder::GetTracksInfo(aType);
+  }
+  if (MP3Decoder::IsSupportedType(mimeType)) {
+    return MP3Decoder::GetTracksInfo(aType);
+  }
+  if (ADTSDecoder::IsSupportedType(mimeType)) {
+    return ADTSDecoder::GetTracksInfo(aType);
+  }
+  if (FlacDecoder::IsSupportedType(mimeType)) {
+    return FlacDecoder::GetTracksInfo(aType);
+  }
+  return nsTArray<UniquePtr<TrackInfo>>();
 }
 
 }  // namespace mozilla
