@@ -8,7 +8,6 @@
 
 #include "mozilla/Maybe.h"
 #include "BitReader.h"
-#include "nsAutoPtr.h"
 #include "prenv.h"
 #include "FlacFrameParser.h"
 #include "VideoUtils.h"
@@ -55,11 +54,11 @@ class FrameHeader {
     mVariableBlockSize = br.ReadBit();
 
     // Block size and sample rate codes.
-    int bs_code = br.ReadBits(4);
-    int sr_code = br.ReadBits(4);
+    int bs_code = AssertedCast<int>(br.ReadBits(4));
+    int sr_code = AssertedCast<int>(br.ReadBits(4));
 
     // Channels and decorrelation.
-    int ch_mode = br.ReadBits(4);
+    uint32_t ch_mode = br.ReadBits(4);
     if (ch_mode < FLAC_MAX_CHANNELS) {
       mInfo.mChannels = ch_mode + 1;
     } else if (ch_mode < FLAC_MAX_CHANNELS + FLAC_CHMODE_MID_SIDE) {
@@ -72,7 +71,7 @@ class FrameHeader {
     }
 
     // Bits per sample.
-    int bps_code = br.ReadBits(3);
+    int bps_code = AssertedCast<int>(br.ReadBits(3));
     if (bps_code == 3 || bps_code == 7) {
       // Invalid sample size code.
       return false;
@@ -86,8 +85,8 @@ class FrameHeader {
     }
 
     // Sample or frame count.
-    int64_t frame_or_sample_num = br.ReadUTF8();
-    if (frame_or_sample_num < 0) {
+    uint64_t frame_or_sample_num = br.ReadUTF8();
+    if (frame_or_sample_num == UINT64_MAX) {
       // Sample/frame number invalid.
       return false;
     }
@@ -96,7 +95,8 @@ class FrameHeader {
     if (bs_code == 0) {
       // reserved blocksize code
       return false;
-    } else if (bs_code == 6) {
+    }
+    if (bs_code == 6) {
       mBlocksize = br.ReadBits(8) + 1;
     } else if (bs_code == 7) {
       mBlocksize = br.ReadBits(16) + 1;
@@ -108,8 +108,10 @@ class FrameHeader {
     // 1- coded sample number if blocksize is variable or
     // 2- coded frame number if blocksize is known.
     // A frame is made of Blocksize sample.
-    mIndex = mVariableBlockSize ? frame_or_sample_num
-                                : frame_or_sample_num * mBlocksize;
+    mIndex = mVariableBlockSize
+                 ? AssertedCast<int64_t>(frame_or_sample_num)
+                 : AssertedCast<int64_t>(frame_or_sample_num * mBlocksize);
+    mFrameOrSampleNum = static_cast<uint64_t>(frame_or_sample_num);
 
     // Sample rate.
     if (sr_code < 12) {
@@ -130,12 +132,21 @@ class FrameHeader {
     for (uint32_t i = 0; i < br.BitCount() / 8; i++) {
       crc = CRC8Table[crc ^ aPacket[i]];
     }
-    mValid = crc == br.ReadBits(8);
+    mValid =
+#ifdef FUZZING
+        true;
+#else
+        crc == br.ReadBits(8);
+#endif
     mSize = br.BitCount() / 8;
 
     if (mValid) {
       // Set the mimetype to make it a valid AudioInfo.
       mInfo.mMimeType = "audio/flac";
+      // Set the codec specific data to flac, but leave it empty since we don't
+      // have METADATA_BLOCK_STREAMINFO in the frame.
+      mInfo.mCodecSpecificConfig =
+          AudioCodecSpecificVariant{FlacCodecSpecificData{}};
     }
 
     return mValid;
@@ -150,25 +161,29 @@ class FrameHeader {
     FLAC_CHMODE_MID_SIDE,
   };
   AudioInfo mInfo;
+  // mFrameOrSampleNum is either:
+  // 1- coded sample number if blocksize is variable or
+  // 2- coded frame number if blocksize is fixed.
+  // A frame is made of Blocksize sample.
+  uint64_t mFrameOrSampleNum = 0;
   // Index in samples from start;
   int64_t mIndex = 0;
   bool mVariableBlockSize = false;
   uint32_t mBlocksize = 0;
-  ;
   uint32_t mSize = 0;
   bool mValid = false;
 
-  static const int FlacSampleRateTable[16];
-  static const int32_t FlacBlocksizeTable[16];
+  static const uint32_t FlacSampleRateTable[16];
+  static const uint32_t FlacBlocksizeTable[16];
   static const uint8_t FlacSampleSizeTable[8];
   static const uint8_t CRC8Table[256];
 };
 
-const int FrameHeader::FlacSampleRateTable[16] = {
+const uint32_t FrameHeader::FlacSampleRateTable[16] = {
     0,     88200, 176400, 192000, 8000, 16000, 22050, 24000,
     32000, 44100, 48000,  96000,  0,    0,     0,     0};
 
-const int32_t FrameHeader::FlacBlocksizeTable[16] = {
+const uint32_t FrameHeader::FlacBlocksizeTable[16] = {
     0,        192,      576 << 0, 576 << 1, 576 << 2, 576 << 3,
     0,        0,        256 << 0, 256 << 1, 256 << 2, 256 << 3,
     256 << 4, 256 << 5, 256 << 6, 256 << 7};
@@ -250,8 +265,8 @@ class Frame {
     Reset();
 
     nsTArray<char> buffer;
-    int64_t originalOffset = aResource.Tell();
-    int64_t offset = originalOffset;
+    uint64_t originalOffset = static_cast<uint64_t>(aResource.Tell());
+    uint64_t offset = originalOffset;
     uint32_t innerOffset = 0;
 
     do {
@@ -268,7 +283,7 @@ class Frame {
           FindNext(reinterpret_cast<uint8_t*>(buffer.Elements()), bufSize);
 
       if (foundOffset >= 0) {
-        SetOffset(aResource, foundOffset + offset);
+        SetOffset(aResource, static_cast<uint64_t>(foundOffset) + offset);
         return true;
       }
 
@@ -291,11 +306,11 @@ class Frame {
     return false;
   }
 
-  int64_t Offset() const { return mOffset; }
+  uint64_t Offset() const { return mOffset; }
 
   const AudioInfo& Info() const { return Header().Info(); }
 
-  void SetEndOffset(int64_t aOffset) { mSize = aOffset - mOffset; }
+  void SetEndOffset(uint64_t aOffset) { mSize = aOffset - mOffset; }
 
   void SetEndTime(int64_t aIndex) {
     if (aIndex > Header().mIndex) {
@@ -303,6 +318,17 @@ class Frame {
     }
   }
 
+  void ResetStartTimeIfNeeded(const Frame& aReferenceFrame) {
+    if (Header().mVariableBlockSize ||
+        aReferenceFrame.Header().mVariableBlockSize ||
+        aReferenceFrame.Header().mBlocksize <= Header().mBlocksize) {
+      // Not a fixed size frame, or nothing to adjust.
+      return;
+    }
+    mHeader.mIndex = AssertedCast<int64_t>(Header().mFrameOrSampleNum *
+                                           aReferenceFrame.Header().mBlocksize);
+  }
+
   uint32_t Size() const { return mSize; }
 
   TimeUnit Time() const {
@@ -310,7 +336,7 @@ class Frame {
       return TimeUnit::Invalid();
     }
     MOZ_ASSERT(Header().Info().mRate, "Invalid Frame. Need Header");
-    return FramesToTimeUnit(Header().mIndex, Header().Info().mRate);
+    return media::TimeUnit(Header().mIndex, Header().Info().mRate);
   }
 
   TimeUnit Duration() const {
@@ -318,7 +344,7 @@ class Frame {
       return TimeUnit();
     }
     MOZ_ASSERT(Header().Info().mRate, "Invalid Frame. Need Header");
-    return FramesToTimeUnit(mDuration, Header().Info().mRate);
+    return media::TimeUnit(mDuration, Header().Info().mRate);
   }
 
   // Returns the parsed frame header.
@@ -338,13 +364,13 @@ class Frame {
   void Reset() { *this = Frame(); }
 
  private:
-  void SetOffset(MediaResourceIndex& aResource, int64_t aOffset) {
+  void SetOffset(MediaResourceIndex& aResource, uint64_t aOffset) {
     mOffset = aOffset;
-    aResource.Seek(SEEK_SET, mOffset);
+    aResource.Seek(SEEK_SET, AssertedCast<int64_t>(mOffset));
   }
 
   // The offset to the start of the header.
-  int64_t mOffset = 0;
+  uint64_t mOffset = 0;
   uint32_t mSize = 0;
   uint32_t mDuration = 0;
   bool mEOS = false;
@@ -380,7 +406,14 @@ class FrameParser {
 
     if (mFrame.IsValid()) {
       if (mNextFrame.EOS()) {
-        mFrame.SetEndOffset(aResource.Tell());
+        mFrame.SetEndOffset(static_cast<uint64_t>(aResource.Tell()));
+        // If the blocksize is fixed, the frame's starting sample number will be
+        // the frame number times the blocksize. However, the last block may
+        // have been incorrectly set as shorter than the stream blocksize.
+        // We recalculate the start time of this last sample using the first
+        // frame blocksize.
+        // TODO: should we use an overall counter of frames instead?
+        mFrame.ResetStartTimeIfNeeded(mFirstFrame);
       } else if (mNextFrame.IsValid()) {
         mFrame.SetEndOffset(mNextFrame.Offset());
         mFrame.SetEndTime(mNextFrame.Header().Index());
@@ -412,7 +445,7 @@ class FrameParser {
   AudioInfo Info() const { return mParser.mInfo; }
 
   // Return a hash table with tag metadata.
-  MetadataTags* GetTags() const { return mParser.GetTags(); }
+  UniquePtr<MetadataTags> GetTags() const { return mParser.GetTags(); }
 
  private:
   bool GetNextFrame(MediaResourceIndex& aResource) {
@@ -422,7 +455,8 @@ class FrameParser {
       aResource.Seek(SEEK_CUR, mNextFrame.Header().Size());
       if (mFrame.IsValid() &&
           mNextFrame.Offset() - mFrame.Offset() < FLAC_MAX_FRAME_SIZE &&
-          !CheckCRC16AtOffset(mFrame.Offset(), mNextFrame.Offset(),
+          !CheckCRC16AtOffset(AssertedCast<int64_t>(mFrame.Offset()),
+                              AssertedCast<int64_t>(mNextFrame.Offset()),
                               aResource)) {
         // The frame doesn't match its CRC or would be too far, skip it..
         continue;
@@ -457,7 +491,7 @@ class FrameParser {
     if (size <= 0) {
       return false;
     }
-    UniquePtr<char[]> buffer(new char[size]);
+    UniquePtr<char[]> buffer(new char[static_cast<size_t>(size)]);
     uint32_t read = 0;
     if (NS_FAILED(aResource.ReadAt(aStart, buffer.get(), size, &read)) ||
         read != size) {
@@ -471,7 +505,11 @@ class FrameParser {
     while (buf < end) {
       crc = CRC16Table[((uint8_t)crc) ^ *buf++] ^ (crc >> 8);
     }
+#ifdef FUZZING
+    return true;
+#else
     return !crc;
+#endif
   }
 
   const uint16_t CRC16Table[256] = {
@@ -566,7 +604,7 @@ FlacTrackDemuxer::FlacTrackDemuxer(MediaResource* aSource)
   Reset();
 }
 
-FlacTrackDemuxer::~FlacTrackDemuxer() {}
+FlacTrackDemuxer::~FlacTrackDemuxer() = default;
 
 bool FlacTrackDemuxer::Init() {
   static const int BUFFER_SIZE = 4096;
@@ -608,7 +646,8 @@ bool FlacTrackDemuxer::Init() {
   // Find the first frame to fully initialise our parser.
   if (mParser->FindNextFrame(mSource)) {
     // Ensure that the next frame returned will be the first.
-    mSource.Seek(SEEK_SET, mParser->FirstFrame().Offset());
+    mSource.Seek(SEEK_SET,
+                 AssertedCast<int64_t>(mParser->FirstFrame().Offset()));
     mParser->EndFrameSession();
   } else if (!mParser->Info().IsValid() || !mParser->FirstFrame().IsValid()) {
     // We must find at least a frame to determine the metadata.
@@ -629,17 +668,27 @@ UniquePtr<TrackInfo> FlacTrackDemuxer::GetInfo() const {
   if (mParser->Info().IsValid()) {
     // We have a proper metadata header.
     UniquePtr<TrackInfo> info = mParser->Info().Clone();
-    nsAutoPtr<MetadataTags> tags(mParser->GetTags());
+    UniquePtr<MetadataTags> tags(mParser->GetTags());
     if (tags) {
-      for (auto iter = tags->Iter(); !iter.Done(); iter.Next()) {
-        info->mTags.AppendElement(MetadataTag(iter.Key(), iter.Data()));
+      for (const auto& entry : *tags) {
+        info->mTags.AppendElement(MetadataTag(entry.GetKey(), entry.GetData()));
       }
     }
+    MOZ_ASSERT(info->IsAudio() &&
+                   info->GetAsAudioInfo()
+                       ->mCodecSpecificConfig.is<FlacCodecSpecificData>(),
+               "Should get flac specific data from parser");
     return info;
-  } else if (mParser->FirstFrame().Info().IsValid()) {
+  }
+
+  if (mParser->FirstFrame().Info().IsValid()) {
     // Use the first frame header.
     UniquePtr<TrackInfo> info = mParser->FirstFrame().Info().Clone();
     info->mDuration = Duration();
+    MOZ_ASSERT(info->IsAudio() &&
+                   info->GetAsAudioInfo()
+                       ->mCodecSpecificConfig.is<FlacCodecSpecificData>(),
+               "Should get flac specific data from parser");
     return info;
   }
   return nullptr;
@@ -678,7 +727,8 @@ TimeUnit FlacTrackDemuxer::FastSeek(const TimeUnit& aTime) {
 
   if (aTime <= mParser->FirstFrame().Time()) {
     // We're attempting to seek prior the first frame, return the first frame.
-    mSource.Seek(SEEK_SET, mParser->FirstFrame().Offset());
+    mSource.Seek(SEEK_SET,
+                 AssertedCast<int64_t>(mParser->FirstFrame().Offset()));
     return mParser->FirstFrame().Time();
   }
 
@@ -687,15 +737,16 @@ TimeUnit FlacTrackDemuxer::FastSeek(const TimeUnit& aTime) {
   // Typically, with flac such approximation is typically useless.
 
   // Estimate where the position might be.
-  int64_t pivot =
-      aTime.ToSeconds() * AverageFrameLength() + mParser->FirstFrame().Offset();
+  int64_t pivot = AssertedCast<int64_t>(
+      aTime.ToSeconds() * AverageFrameLength() +
+      AssertedCast<double>(mParser->FirstFrame().Offset()));
 
   // Time in seconds where we can stop seeking and will continue using
   // ScanUntil.
   static const int GAP_THRESHOLD = 5;
-  int64_t first = mParser->FirstFrame().Offset();
+  int64_t first = AssertedCast<int64_t>(mParser->FirstFrame().Offset());
   int64_t last = mSource.GetLength();
-  Maybe<int64_t> lastFoundOffset;
+  Maybe<uint64_t> lastFoundOffset;
   uint32_t iterations = 0;
   TimeUnit timeSeekedTo;
 
@@ -703,13 +754,22 @@ TimeUnit FlacTrackDemuxer::FastSeek(const TimeUnit& aTime) {
     iterations++;
     mSource.Seek(SEEK_SET, pivot);
     flac::Frame frame;
-    if (!frame.FindNext(mSource)) {
+    bool found = frame.FindNext(mSource);
+    if (!found) {
       NS_WARNING("We should have found a point");
       break;
     }
+    if (!frame.IsValid()) {
+      NS_WARNING("Invalid frame after seeking and sync on a frame");
+      break;
+    }
+    // When the rate is 0, the rate from STREAMINFO is to be used.
+    if (frame.Header().Info().mRate == 0) {
+      frame.SetRate(mParser->FirstFrame().Info().mRate);
+    }
     timeSeekedTo = frame.Time();
 
-    LOGV("FastSeek: interation:%u found:%f @ %" PRId64, iterations,
+    LOGV("FastSeek: interation:%u found:%f @ %" PRIu64, iterations,
          timeSeekedTo.ToSeconds(), frame.Offset());
 
     if (lastFoundOffset && lastFoundOffset.ref() == frame.Offset()) {
@@ -737,7 +797,7 @@ TimeUnit FlacTrackDemuxer::FastSeek(const TimeUnit& aTime) {
   } while (true);
 
   if (lastFoundOffset) {
-    mSource.Seek(SEEK_SET, lastFoundOffset.ref());
+    mSource.Seek(SEEK_SET, AssertedCast<int64_t>(lastFoundOffset.ref()));
   }
 
   return timeSeekedTo;
@@ -756,7 +816,7 @@ TimeUnit FlacTrackDemuxer::ScanUntil(const TimeUnit& aTime) {
   int64_t previousOffset = 0;
   TimeUnit previousTime;
   while (FindNextFrame().IsValid() && mParser->CurrentFrame().Time() < aTime) {
-    previousOffset = mParser->CurrentFrame().Offset();
+    previousOffset = AssertedCast<int64_t>(mParser->CurrentFrame().Offset());
     previousTime = mParser->CurrentFrame().Time();
   }
 
@@ -789,16 +849,19 @@ RefPtr<FlacTrackDemuxer::SamplesPromise> FlacTrackDemuxer::GetSamples(
   while (aNumSamples--) {
     RefPtr<MediaRawData> frame(GetNextFrame(FindNextFrame()));
     if (!frame) break;
-
-    frames->mSamples.AppendElement(frame);
+    if (!frame->HasValidTime()) {
+      return SamplesPromise::CreateAndReject(NS_ERROR_DOM_MEDIA_DEMUXER_ERR,
+                                             __func__);
+    }
+    frames->AppendSample(frame);
   }
 
   LOGV("GetSamples() End mSamples.Length=%zu aNumSamples=%d offset=%" PRId64
        " mParsedFramesDuration=%f mTotalFrameLen=%" PRIu64,
-       frames->mSamples.Length(), aNumSamples, GetResourceOffset(),
+       frames->GetSamples().Length(), aNumSamples, GetResourceOffset(),
        mParsedFramesDuration.ToSeconds(), mTotalFrameLen);
 
-  if (frames->mSamples.IsEmpty()) {
+  if (frames->GetSamples().IsEmpty()) {
     return SamplesPromise::CreateAndReject(NS_ERROR_DOM_MEDIA_END_OF_STREAM,
                                            __func__);
   }
@@ -810,7 +873,8 @@ void FlacTrackDemuxer::Reset() {
   LOG("Reset()");
   MOZ_ASSERT(mParser);
   if (mParser->FirstFrame().IsValid()) {
-    mSource.Seek(SEEK_SET, mParser->FirstFrame().Offset());
+    mSource.Seek(SEEK_SET,
+                 AssertedCast<int64_t>(mParser->FirstFrame().Offset()));
   } else {
     mSource.Seek(SEEK_SET, 0);
   }
@@ -840,7 +904,7 @@ TimeIntervals FlacTrackDemuxer::GetBuffered() {
 }
 
 const flac::Frame& FlacTrackDemuxer::FindNextFrame() {
-  LOGV("FindNext() Begin offset=%" PRId64
+  LOGV("FindNextFrame() Begin offset=%" PRId64
        " mParsedFramesDuration=%f"
        " mTotalFrameLen=%" PRIu64,
        GetResourceOffset(), mParsedFramesDuration.ToSeconds(), mTotalFrameLen);
@@ -856,7 +920,7 @@ const flac::Frame& FlacTrackDemuxer::FindNextFrame() {
                                                mParser->FirstFrame().Offset() +
                                                mParser->CurrentFrame().Size());
 
-    LOGV("FindNext() End time=%f offset=%" PRId64
+    LOGV("FindNextFrame() End time=%f offset=%" PRId64
          " mParsedFramesDuration=%f"
          " mTotalFrameLen=%" PRIu64,
          mParser->CurrentFrame().Time().ToSeconds(), GetResourceOffset(),
@@ -876,19 +940,20 @@ already_AddRefed<MediaRawData> FlacTrackDemuxer::GetNextFrame(
   LOG("GetNextFrame() Begin(time=%f offset=%" PRId64 " size=%u)",
       aFrame.Time().ToSeconds(), aFrame.Offset(), aFrame.Size());
 
-  const int64_t offset = aFrame.Offset();
+  const uint64_t offset = aFrame.Offset();
   const uint32_t size = aFrame.Size();
 
   RefPtr<MediaRawData> frame = new MediaRawData();
-  frame->mOffset = offset;
+  frame->mOffset = AssertedCast<int64_t>(offset);
 
-  nsAutoPtr<MediaRawDataWriter> frameWriter(frame->CreateWriter());
+  UniquePtr<MediaRawDataWriter> frameWriter(frame->CreateWriter());
   if (!frameWriter->SetSize(size)) {
     LOG("GetNext() Exit failed to allocated media buffer");
     return nullptr;
   }
 
-  const uint32_t read = Read(frameWriter->Data(), offset, size);
+  const uint32_t read = Read(frameWriter->Data(), AssertedCast<int64_t>(offset),
+                             AssertedCast<int32_t>(size));
   if (read != size) {
     LOG("GetNextFrame() Exit read=%u frame->Size=%zu", read, frame->Size());
     return nullptr;
@@ -897,7 +962,7 @@ already_AddRefed<MediaRawData> FlacTrackDemuxer::GetNextFrame(
   frame->mTime = aFrame.Time();
   frame->mDuration = aFrame.Duration();
   frame->mTimecode = frame->mTime;
-  frame->mOffset = aFrame.Offset();
+  frame->mOffset = AssertedCast<int64_t>(aFrame.Offset());
   frame->mKeyframe = true;
 
   MOZ_ASSERT(!frame->mTime.IsNegative());
@@ -906,18 +971,19 @@ already_AddRefed<MediaRawData> FlacTrackDemuxer::GetNextFrame(
   return frame.forget();
 }
 
-int32_t FlacTrackDemuxer::Read(uint8_t* aBuffer, int64_t aOffset,
-                               int32_t aSize) {
+uint32_t FlacTrackDemuxer::Read(uint8_t* aBuffer, int64_t aOffset,
+                                int32_t aSize) {
   uint32_t read = 0;
   const nsresult rv = mSource.ReadAt(aOffset, reinterpret_cast<char*>(aBuffer),
                                      static_cast<uint32_t>(aSize), &read);
   NS_ENSURE_SUCCESS(rv, 0);
-  return static_cast<int32_t>(read);
+  return read;
 }
 
 double FlacTrackDemuxer::AverageFrameLength() const {
   if (mParsedFramesDuration.ToMicroseconds()) {
-    return mTotalFrameLen / mParsedFramesDuration.ToSeconds();
+    return AssertedCast<double>(mTotalFrameLen) /
+           mParsedFramesDuration.ToSeconds();
   }
 
   return 0.0;
@@ -961,13 +1027,15 @@ TimeUnit FlacTrackDemuxer::TimeAtEnd() {
   // Update our current progress stats.
   mParsedFramesDuration =
       previousTime + previousDuration - mParser->FirstFrame().Time();
-  mTotalFrameLen = streamLen - mParser->FirstFrame().Offset();
+
+  mTotalFrameLen =
+      static_cast<uint64_t>(streamLen) - mParser->FirstFrame().Offset();
 
   return mParsedFramesDuration;
 }
 
-/* static */ bool FlacDemuxer::FlacSniffer(const uint8_t* aData,
-                                           const uint32_t aLength) {
+/* static */
+bool FlacDemuxer::FlacSniffer(const uint8_t* aData, const uint32_t aLength) {
   if (aLength < FLAC_MIN_FRAME_SIZE) {
     return false;
   }
