@@ -6,22 +6,25 @@
 
 #ifdef XP_WIN
 // Include Windows headers required for enabling high precision timers.
-#  include "windows.h"
-#  include "mmsystem.h"
+#  include <windows.h>
+#  include <mmsystem.h>
 #endif
 
 #include "VideoSink.h"
 
+#include "AudioDeviceInfo.h"
 #include "MediaQueue.h"
 #include "VideoUtils.h"
 
 #include "mozilla/IntegerPrintfMacros.h"
+#include "mozilla/ProfilerLabels.h"
+#include "mozilla/ProfilerMarkerTypes.h"
 #include "mozilla/StaticPrefs_browser.h"
 #include "mozilla/StaticPrefs_media.h"
 
 namespace mozilla {
-
 extern LazyLogModule gMediaDecoderLog;
+}
 
 #undef FMT
 
@@ -31,14 +34,7 @@ extern LazyLogModule gMediaDecoderLog;
 #define VSINK_LOG_V(x, ...) \
   MOZ_LOG(gMediaDecoderLog, LogLevel::Verbose, (FMT(x, ##__VA_ARGS__)))
 
-#ifdef MOZ_GECKO_PROFILER
-#  include "ProfilerMarkerPayload.h"
-#  define VSINK_ADD_PROFILER_MARKER(tag, startTime, endTime) \
-    PROFILER_ADD_MARKER_WITH_PAYLOAD(                        \
-        tag, MEDIA_PLAYBACK, MediaSampleMarkerPayload, (startTime, endTime))
-#else
-#  define VSINK_ADD_PROFILER_MARKER(tag, startTime, endTime)
-#endif
+namespace mozilla {
 
 using namespace mozilla::layers;
 
@@ -53,7 +49,7 @@ static void SetImageToGreenPixel(PlanarYCbCrImage* aImage) {
   data.mCbChannel = greenPixel + 1;
   data.mCrChannel = greenPixel + 2;
   data.mYStride = data.mCbCrStride = 1;
-  data.mPicSize = data.mYSize = data.mCbCrSize = gfx::IntSize(1, 1);
+  data.mPictureRect = gfx::IntRect(0, 0, 1, 1);
   data.mYUVColorSpace = gfx::YUVColorSpace::BT601;
   aImage->CopyData(data);
 }
@@ -108,7 +104,7 @@ RefPtr<VideoSink::EndedPromise> VideoSink::OnEnded(TrackType aType) {
   return nullptr;
 }
 
-TimeUnit VideoSink::GetEndTime(TrackType aType) const {
+media::TimeUnit VideoSink::GetEndTime(TrackType aType) const {
   AssertOwnerThread();
   MOZ_ASSERT(mAudioSink->IsStarted(), "Must be called after playback starts.");
 
@@ -117,10 +113,10 @@ TimeUnit VideoSink::GetEndTime(TrackType aType) const {
   } else if (aType == TrackInfo::kAudioTrack) {
     return mAudioSink->GetEndTime(aType);
   }
-  return TimeUnit::Zero();
+  return media::TimeUnit::Zero();
 }
 
-TimeUnit VideoSink::GetPosition(TimeStamp* aTimeStamp) const {
+media::TimeUnit VideoSink::GetPosition(TimeStamp* aTimeStamp) {
   AssertOwnerThread();
   return mAudioSink->GetPosition(aTimeStamp);
 }
@@ -133,6 +129,14 @@ bool VideoSink::HasUnplayedFrames(TrackType aType) const {
   return mAudioSink->HasUnplayedFrames(aType);
 }
 
+media::TimeUnit VideoSink::UnplayedDuration(TrackType aType) const {
+  AssertOwnerThread();
+  MOZ_ASSERT(aType == TrackInfo::kAudioTrack,
+             "Not implemented for non audio tracks.");
+
+  return mAudioSink->UnplayedDuration(aType);
+}
+
 void VideoSink::SetPlaybackRate(double aPlaybackRate) {
   AssertOwnerThread();
 
@@ -145,12 +149,23 @@ void VideoSink::SetVolume(double aVolume) {
   mAudioSink->SetVolume(aVolume);
 }
 
+void VideoSink::SetStreamName(const nsAString& aStreamName) {
+  AssertOwnerThread();
+
+  mAudioSink->SetStreamName(aStreamName);
+}
+
 void VideoSink::SetPreservesPitch(bool aPreservesPitch) {
   AssertOwnerThread();
 
   mAudioSink->SetPreservesPitch(aPreservesPitch);
 }
 
+RefPtr<GenericPromise> VideoSink::SetAudioDevice(
+    RefPtr<AudioDeviceInfo> aDevice) {
+  return mAudioSink->SetAudioDevice(std::move(aDevice));
+}
+
 double VideoSink::PlaybackRate() const {
   AssertOwnerThread();
 
@@ -210,7 +225,8 @@ void VideoSink::SetPlaying(bool aPlaying) {
   EnsureHighResTimersOnOnlyIfPlaying();
 }
 
-nsresult VideoSink::Start(const TimeUnit& aStartTime, const MediaInfo& aInfo) {
+nsresult VideoSink::Start(const media::TimeUnit& aStartTime,
+                          const MediaInfo& aInfo) {
   AssertOwnerThread();
   VSINK_LOG("[%s]", __func__);
 
@@ -270,7 +286,7 @@ void VideoSink::Stop() {
     mEndPromiseHolder.ResolveIfExists(true, __func__);
     mEndPromise = nullptr;
   }
-  mVideoFrameEndTime = TimeUnit::Zero();
+  mVideoFrameEndTime = media::TimeUnit::Zero();
 
   EnsureHighResTimersOnOnlyIfPlaying();
 }
@@ -367,7 +383,7 @@ void VideoSink::TryUpdateRenderedVideoFrames() {
   }
 
   TimeStamp nowTime;
-  const TimeUnit clockTime = mAudioSink->GetPosition(&nowTime);
+  const media::TimeUnit clockTime = mAudioSink->GetPosition(&nowTime);
   if (clockTime >= v->mTime) {
     // Time to render this frame.
     UpdateRenderedVideoFrames();
@@ -458,12 +474,15 @@ void VideoSink::RenderVideoFrames(int32_t aMaxFrames, int64_t aClockTime,
     img->mFrameID = frame->mFrameID;
     img->mProducerID = mProducerID;
 
-    VSINK_LOG_V("playing video frame %" PRId64 " (id=%x) (vq-queued=%zu)",
+    VSINK_LOG_V("playing video frame %" PRId64
+                " (id=%x, vq-queued=%zu, clock=%" PRId64 ")",
                 frame->mTime.ToMicroseconds(), frame->mFrameID,
-                VideoQueue().GetSize());
+                VideoQueue().GetSize(), aClockTime);
     if (!wasSent) {
-      VSINK_ADD_PROFILER_MARKER("PlayVideo", frame->mTime.ToMicroseconds(),
-                                frame->GetEndTime().ToMicroseconds());
+      PROFILER_MARKER("PlayVideo", MEDIA_PLAYBACK, {}, MediaSampleMarker,
+                      frame->mTime.ToMicroseconds(),
+                      frame->GetEndTime().ToMicroseconds(),
+                      VideoQueue().GetSize());
     }
   }
 
@@ -490,7 +509,7 @@ void VideoSink::UpdateRenderedVideoFrames() {
   uint32_t droppedInSink = 0;
 
   // Skip frames up to the playback position.
-  TimeUnit lastFrameEndTime;
+  media::TimeUnit lastFrameEndTime;
   while (VideoQueue().GetSize() > mMinVideoQueueSize &&
          clockTime >= VideoQueue().PeekFront()->GetEndTime()) {
     RefPtr<VideoData> frame = VideoQueue().PopFront();
@@ -502,8 +521,35 @@ void VideoSink::UpdateRenderedVideoFrames() {
       VSINK_LOG_V("discarding video frame mTime=%" PRId64
                   " clock_time=%" PRId64,
                   frame->mTime.ToMicroseconds(), clockTime.ToMicroseconds());
-      VSINK_ADD_PROFILER_MARKER("DiscardVideo", frame->mTime.ToMicroseconds(),
-                                frame->GetEndTime().ToMicroseconds());
+
+      struct VideoSinkDroppedFrameMarker {
+        static constexpr Span<const char> MarkerTypeName() {
+          return MakeStringSpan("VideoSinkDroppedFrame");
+        }
+        static void StreamJSONMarkerData(
+            baseprofiler::SpliceableJSONWriter& aWriter,
+            int64_t aSampleStartTimeUs, int64_t aSampleEndTimeUs,
+            int64_t aClockTimeUs) {
+          aWriter.IntProperty("sampleStartTimeUs", aSampleStartTimeUs);
+          aWriter.IntProperty("sampleEndTimeUs", aSampleEndTimeUs);
+          aWriter.IntProperty("clockTimeUs", aClockTimeUs);
+        }
+        static MarkerSchema MarkerTypeDisplay() {
+          using MS = MarkerSchema;
+          MS schema{MS::Location::MarkerChart, MS::Location::MarkerTable};
+          schema.AddKeyLabelFormat("sampleStartTimeUs", "Sample start time",
+                                   MS::Format::Microseconds);
+          schema.AddKeyLabelFormat("sampleEndTimeUs", "Sample end time",
+                                   MS::Format::Microseconds);
+          schema.AddKeyLabelFormat("clockTimeUs", "Audio clock time",
+                                   MS::Format::Microseconds);
+          return schema;
+        }
+      };
+      profiler_add_marker(
+          "VideoSinkDroppedFrame", geckoprofiler::category::MEDIA_PLAYBACK, {},
+          VideoSinkDroppedFrameMarker{}, frame->mTime.ToMicroseconds(),
+          frame->GetEndTime().ToMicroseconds(), clockTime.ToMicroseconds());
     }
   }
 
@@ -654,4 +700,5 @@ bool VideoSink::InitializeBlankImage() {
   SetImageToGreenPixel(mBlankImage->AsPlanarYCbCrImage());
   return true;
 }
+
 }  // namespace mozilla
