@@ -7,9 +7,7 @@
 #include "MediaSourceDemuxer.h"
 
 #include "MediaSourceUtils.h"
-#include "OpusDecoder.h"
 #include "SourceBufferList.h"
-#include "VorbisDecoder.h"
 #include "VideoUtils.h"
 #include "nsPrintfCString.h"
 
@@ -17,6 +15,12 @@
 #include <limits>
 #include <stdint.h>
 
+extern mozilla::LogModule* GetMediaSourceLog();
+
+#define MSE_DEBUG(arg, ...)                                              \
+  DDMOZ_LOG(GetMediaSourceLog(), mozilla::LogLevel::Debug, "::%s: " arg, \
+            __func__, ##__VA_ARGS__)
+
 namespace mozilla {
 
 typedef TrackInfo::TrackType TrackType;
@@ -27,7 +31,7 @@ MediaSourceDemuxer::MediaSourceDemuxer(AbstractThread* aAbstractMainThread)
     : mTaskQueue(
           TaskQueue::Create(GetMediaThreadPool(MediaThreadType::SUPERVISOR),
                             "MediaSourceDemuxer::mTaskQueue")),
-      mMonitor("MediaSourceDemuxer") {
+      mMutex("MediaSourceDemuxer") {
   MOZ_ASSERT(NS_IsMainThread());
 }
 
@@ -90,7 +94,7 @@ bool MediaSourceDemuxer::ScanSourceBuffersForContent() {
     return false;
   }
 
-  MonitorAutoLock mon(mMonitor);
+  MutexAutoLock mon(mMutex);
 
   bool haveEmptySourceBuffer = false;
   for (const auto& sourceBuffer : mSourceBuffers) {
@@ -118,7 +122,7 @@ bool MediaSourceDemuxer::ScanSourceBuffersForContent() {
 }
 
 uint32_t MediaSourceDemuxer::GetNumberTracks(TrackType aType) const {
-  MonitorAutoLock mon(mMonitor);
+  MutexAutoLock mon(mMutex);
 
   switch (aType) {
     case TrackType::kAudioTrack:
@@ -132,7 +136,7 @@ uint32_t MediaSourceDemuxer::GetNumberTracks(TrackType aType) const {
 
 already_AddRefed<MediaTrackDemuxer> MediaSourceDemuxer::GetTrackDemuxer(
     TrackType aType, uint32_t aTrackNumber) {
-  MonitorAutoLock mon(mMonitor);
+  MutexAutoLock mon(mMutex);
   RefPtr<TrackBuffersManager> manager = GetManager(aType);
   if (!manager) {
     return nullptr;
@@ -147,14 +151,14 @@ already_AddRefed<MediaTrackDemuxer> MediaSourceDemuxer::GetTrackDemuxer(
 bool MediaSourceDemuxer::IsSeekable() const { return true; }
 
 UniquePtr<EncryptionInfo> MediaSourceDemuxer::GetCrypto() {
-  MonitorAutoLock mon(mMonitor);
+  MutexAutoLock mon(mMutex);
   auto crypto = MakeUnique<EncryptionInfo>();
   *crypto = mInfo.mCrypto;
   return crypto;
 }
 
 void MediaSourceDemuxer::AttachSourceBuffer(
-    RefPtr<TrackBuffersManager>& aSourceBuffer) {
+    const RefPtr<TrackBuffersManager>& aSourceBuffer) {
   nsCOMPtr<nsIRunnable> task = NewRunnableMethod<RefPtr<TrackBuffersManager>&&>(
       "MediaSourceDemuxer::DoAttachSourceBuffer", this,
       &MediaSourceDemuxer::DoAttachSourceBuffer, aSourceBuffer);
@@ -171,7 +175,7 @@ void MediaSourceDemuxer::DoAttachSourceBuffer(
 }
 
 void MediaSourceDemuxer::DetachSourceBuffer(
-    RefPtr<TrackBuffersManager>& aSourceBuffer) {
+    const RefPtr<TrackBuffersManager>& aSourceBuffer) {
   nsCOMPtr<nsIRunnable> task =
       NS_NewRunnableFunction("MediaSourceDemuxer::DoDetachSourceBuffer",
                              [self = RefPtr{this}, aSourceBuffer]() {
@@ -192,7 +196,7 @@ void MediaSourceDemuxer::DoDetachSourceBuffer(
 
   AutoTArray<RefPtr<MediaSourceTrackDemuxer>, 2> matchingDemuxers;
   {
-    MonitorAutoLock mon(mMonitor);
+    MutexAutoLock mon(mMutex);
     if (aSourceBuffer == mAudioTrack) {
       mAudioTrack = nullptr;
     }
@@ -244,7 +248,7 @@ MediaSourceDemuxer::~MediaSourceDemuxer() {
 
 RefPtr<GenericPromise> MediaSourceDemuxer::GetDebugInfo(
     dom::MediaSourceDemuxerDebugInfo& aInfo) const {
-  MonitorAutoLock mon(mMonitor);
+  MutexAutoLock mon(mMutex);
   nsTArray<RefPtr<GenericPromise>> promises;
   if (mAudioTrack) {
     promises.AppendElement(mAudioTrack->RequestDebugInfo(aInfo.mAudioTrack));
@@ -267,13 +271,13 @@ MediaSourceTrackDemuxer::MediaSourceTrackDemuxer(MediaSourceDemuxer* aParent,
     : mParent(aParent),
       mTaskQueue(mParent->GetTaskQueue()),
       mType(aType),
-      mMonitor("MediaSourceTrackDemuxer"),
+      mMutex("MediaSourceTrackDemuxer", this),
       mManager(aManager),
       mReset(true),
       mPreRoll(TimeUnit::FromMicroseconds(
-          OpusDataDecoder::IsOpus(mParent->GetTrackInfo(mType)->mMimeType) ||
-                  VorbisDataDecoder::IsVorbis(
-                      mParent->GetTrackInfo(mType)->mMimeType)
+          mParent->GetTrackInfo(mType)->mMimeType.EqualsLiteral("audio/opus") ||
+                  mParent->GetTrackInfo(mType)->mMimeType.EqualsLiteral(
+                      "audio/vorbis")
               ? 80000
           : mParent->GetTrackInfo(mType)->mMimeType.EqualsLiteral(
                 "audio/mp4a-latm")
@@ -289,7 +293,7 @@ MediaSourceTrackDemuxer::MediaSourceTrackDemuxer(MediaSourceDemuxer* aParent,
 }
 
 UniquePtr<TrackInfo> MediaSourceTrackDemuxer::GetInfo() const {
-  MonitorAutoLock mon(mParent->mMonitor);
+  MutexAutoLock mon(mParent->mMutex);
   return mParent->GetTrackInfo(mType)->Clone();
 }
 
@@ -312,6 +316,7 @@ void MediaSourceTrackDemuxer::Reset() {
   RefPtr<MediaSourceTrackDemuxer> self = this;
   nsCOMPtr<nsIRunnable> task =
       NS_NewRunnableFunction("MediaSourceTrackDemuxer::Reset", [self]() {
+        self->mMutex.AssertOnWritingThread();
         self->mNextSample.reset();
         self->mReset = true;
         if (!self->mManager) {
@@ -320,7 +325,7 @@ void MediaSourceTrackDemuxer::Reset() {
         MOZ_ASSERT(self->OnTaskQueue());
         self->mManager->Seek(self->mType, TimeUnit::Zero(), TimeUnit::Zero());
         {
-          MonitorAutoLock mon(self->mMonitor);
+          MutexSingleWriterAutoLockOnThread(lock, self->mMutex);
           self->mNextRandomAccessPoint =
               self->mManager->GetNextRandomAccessPoint(
                   self->mType, MediaSourceDemuxer::EOS_FUZZ);
@@ -332,7 +337,7 @@ void MediaSourceTrackDemuxer::Reset() {
 }
 
 nsresult MediaSourceTrackDemuxer::GetNextRandomAccessPoint(TimeUnit* aTime) {
-  MonitorAutoLock mon(mMonitor);
+  MutexSingleWriterAutoLock mon(mMutex);
   *aTime = mNextRandomAccessPoint;
   return NS_OK;
 }
@@ -346,7 +351,7 @@ MediaSourceTrackDemuxer::SkipToNextRandomAccessPoint(
 }
 
 media::TimeIntervals MediaSourceTrackDemuxer::GetBuffered() {
-  MonitorAutoLock mon(mMonitor);
+  MutexSingleWriterAutoLock mon(mMutex);
   if (!mManager) {
     return media::TimeIntervals();
   }
@@ -367,6 +372,7 @@ void MediaSourceTrackDemuxer::BreakCycles() {
 
 RefPtr<MediaSourceTrackDemuxer::SeekPromise> MediaSourceTrackDemuxer::DoSeek(
     const TimeUnit& aTime) {
+  mMutex.AssertOnWritingThread();
   if (!mManager) {
     return SeekPromise::CreateAndReject(
         MediaResult(NS_ERROR_DOM_MEDIA_CANCELED,
@@ -387,6 +393,12 @@ RefPtr<MediaSourceTrackDemuxer::SeekPromise> MediaSourceTrackDemuxer::DoSeek(
     seekTime = std::max(mManager->HighestStartTime(mType) - mPreRoll,
                         TimeUnit::Zero());
   }
+
+  MSE_DEBUG("DoSeek, original target=%" PRId64 "%s, seekTime=%" PRId64
+            "%s, buffered=%s",
+            aTime.ToMicroseconds(), aTime.ToString().get(),
+            seekTime.ToMicroseconds(), seekTime.ToString().get(),
+            DumpTimeRanges(buffered).get());
   if (!buffered.ContainsWithStrictEnd(seekTime)) {
     if (!buffered.ContainsWithStrictEnd(aTime)) {
       // We don't have the data to seek to.
@@ -400,6 +412,10 @@ RefPtr<MediaSourceTrackDemuxer::SeekPromise> MediaSourceTrackDemuxer::DoSeek(
     // the interval.
     TimeIntervals::IndexType index = buffered.Find(aTime);
     MOZ_ASSERT(index != TimeIntervals::NoIndex);
+    MSE_DEBUG("Can't find seekTime %" PRId64
+              " in the buffer range, use the earliest time %" PRId64,
+              seekTime.ToMicroseconds(),
+              buffered[index].mStart.ToMicroseconds());
     seekTime = buffered[index].mStart;
   }
   seekTime = mManager->Seek(mType, seekTime, MediaSourceDemuxer::EOS_FUZZ);
@@ -407,10 +423,12 @@ RefPtr<MediaSourceTrackDemuxer::SeekPromise> MediaSourceTrackDemuxer::DoSeek(
   RefPtr<MediaRawData> sample =
       mManager->GetSample(mType, TimeUnit::Zero(), result);
   MOZ_ASSERT(NS_SUCCEEDED(result) && sample);
-  mNextSample = Some(sample);
+  if (sample) {
+    mNextSample = Some(sample);
+  }
   mReset = false;
   {
-    MonitorAutoLock mon(mMonitor);
+    MutexSingleWriterAutoLockOnThread(lock, mMutex);
     mNextRandomAccessPoint =
         mManager->GetNextRandomAccessPoint(mType, MediaSourceDemuxer::EOS_FUZZ);
   }
@@ -419,6 +437,7 @@ RefPtr<MediaSourceTrackDemuxer::SeekPromise> MediaSourceTrackDemuxer::DoSeek(
 
 RefPtr<MediaSourceTrackDemuxer::SamplesPromise>
 MediaSourceTrackDemuxer::DoGetSamples(int32_t aNumSamples) {
+  mMutex.AssertOnWritingThread();
   if (!mManager) {
     return SamplesPromise::CreateAndReject(
         MediaResult(NS_ERROR_DOM_MEDIA_CANCELED,
@@ -449,28 +468,29 @@ MediaSourceTrackDemuxer::DoGetSamples(int32_t aNumSamples) {
     mReset = false;
   }
   RefPtr<MediaRawData> sample;
-  MediaResult result = NS_OK;
   if (mNextSample) {
     sample = mNextSample.ref();
     mNextSample.reset();
   } else {
+    MediaResult result = NS_OK;
     sample = mManager->GetSample(mType, MediaSourceDemuxer::EOS_FUZZ, result);
-  }
-  if (!sample) {
-    if (result == NS_ERROR_DOM_MEDIA_END_OF_STREAM ||
-        result == NS_ERROR_DOM_MEDIA_WAITING_FOR_DATA) {
-      return SamplesPromise::CreateAndReject(
-          (result == NS_ERROR_DOM_MEDIA_END_OF_STREAM && mManager->IsEnded())
-              ? NS_ERROR_DOM_MEDIA_END_OF_STREAM
-              : NS_ERROR_DOM_MEDIA_WAITING_FOR_DATA,
-          __func__);
+    if (!sample) {
+      if (result == NS_ERROR_DOM_MEDIA_END_OF_STREAM ||
+          result == NS_ERROR_DOM_MEDIA_WAITING_FOR_DATA) {
+        return SamplesPromise::CreateAndReject(
+            (result == NS_ERROR_DOM_MEDIA_END_OF_STREAM && mManager->IsEnded())
+                ? NS_ERROR_DOM_MEDIA_END_OF_STREAM
+                : NS_ERROR_DOM_MEDIA_WAITING_FOR_DATA,
+            __func__);
+      }
+      return SamplesPromise::CreateAndReject(result, __func__);
     }
-    return SamplesPromise::CreateAndReject(result, __func__);
   }
+  MOZ_DIAGNOSTIC_ASSERT(sample);
   RefPtr<SamplesHolder> samples = new SamplesHolder;
   samples->AppendSample(sample);
   {
-    MonitorAutoLock mon(mMonitor);  // spurious warning will be given
+    MutexSingleWriterAutoLockOnThread(lock, mMutex);
     // Diagnostic asserts for bug 1810396
     MOZ_DIAGNOSTIC_ASSERT(sample, "Invalid sample pointer found!");
     MOZ_DIAGNOSTIC_ASSERT(sample->HasValidTime(), "Invalid sample time found!");
@@ -488,6 +508,7 @@ MediaSourceTrackDemuxer::DoGetSamples(int32_t aNumSamples) {
 RefPtr<MediaSourceTrackDemuxer::SkipAccessPointPromise>
 MediaSourceTrackDemuxer::DoSkipToNextRandomAccessPoint(
     const TimeUnit& aTimeThreadshold) {
+  mMutex.AssertOnWritingThread();
   if (!mManager) {
     return SkipAccessPointPromise::CreateAndReject(
         SkipFailureHolder(MediaResult(NS_ERROR_DOM_MEDIA_CANCELED,
@@ -517,14 +538,16 @@ MediaSourceTrackDemuxer::DoSkipToNextRandomAccessPoint(
 }
 
 bool MediaSourceTrackDemuxer::HasManager(TrackBuffersManager* aManager) const {
-  MOZ_ASSERT(OnTaskQueue());
+  mMutex.AssertOnWritingThread();
   return mManager == aManager;
 }
 
 void MediaSourceTrackDemuxer::DetachManager() {
   MOZ_ASSERT(OnTaskQueue());
-  MonitorAutoLock mon(mMonitor);
+  MutexSingleWriterAutoLock mon(mMutex);
   mManager = nullptr;
 }
 
+#undef MSE_DEBUG
+
 }  // namespace mozilla
