@@ -7,14 +7,14 @@
 #include "OggDecoder.h"
 #include "MediaContainerType.h"
 #include "MediaDecoder.h"
-#include "mozilla/StaticPrefs.h"
+#include "mozilla/StaticPrefs_media.h"
 #include "nsMimeTypes.h"
 
 namespace mozilla {
 
 /* static */
 bool OggDecoder::IsSupportedType(const MediaContainerType& aContainerType) {
-  if (!StaticPrefs::MediaOggEnabled()) {
+  if (!StaticPrefs::media_ogg_enabled()) {
     return false;
   }
 
@@ -24,7 +24,10 @@ bool OggDecoder::IsSupportedType(const MediaContainerType& aContainerType) {
     return false;
   }
 
-  const bool isOggVideo = (aContainerType.Type() != MEDIAMIMETYPE(AUDIO_OGG));
+  const bool isOggVideo = (aContainerType.Type() == MEDIAMIMETYPE(VIDEO_OGG));
+  if (isOggVideo && !StaticPrefs::media_theora_enabled()) {
+    return false;
+  }
 
   const MediaCodecs& codecs = aContainerType.ExtendedType().Codecs();
   if (codecs.IsEmpty()) {
@@ -40,8 +43,9 @@ bool OggDecoder::IsSupportedType(const MediaContainerType& aContainerType) {
     }
     // Note: Only accept Theora in a video container type, not in an audio
     // container type.
-    if (isOggVideo && codec.EqualsLiteral("theora")) {
-      continue;
+    if (aContainerType.Type() != MEDIAMIMETYPE(AUDIO_OGG) &&
+        codec.EqualsLiteral("theora")) {
+      return StaticPrefs::media_theora_enabled();
     }
     // Some unsupported codec.
     return false;
@@ -68,14 +72,12 @@ nsTArray<UniquePtr<TrackInfo>> OggDecoder::GetTracksInfo(
         codec.EqualsLiteral("flac")) {
       tracks.AppendElement(
           CreateTrackInfoWithMIMETypeAndContainerTypeExtraParameters(
-              NS_LITERAL_CSTRING("audio/") + NS_ConvertUTF16toUTF8(codec),
-              aType));
+              "audio/"_ns + NS_ConvertUTF16toUTF8(codec), aType));
     } else {
       MOZ_ASSERT(codec.EqualsLiteral("theora"));
       tracks.AppendElement(
           CreateTrackInfoWithMIMETypeAndContainerTypeExtraParameters(
-              NS_LITERAL_CSTRING("video/") + NS_ConvertUTF16toUTF8(codec),
-              aType));
+              "video/"_ns + NS_ConvertUTF16toUTF8(codec), aType));
     }
   }
   return tracks;
