@@ -32,8 +32,7 @@ class FFmpegDecoderModule : public PlatformDecoderModule {
 
   already_AddRefed<MediaDataDecoder> CreateVideoDecoder(
       const CreateDecoderParams& aParams) override {
-    if (Supports(SupportDecoderParams(aParams), nullptr) ==
-        media::DecodeSupport::Unsupported) {
+    if (Supports(SupportDecoderParams(aParams), nullptr).isEmpty()) {
       return nullptr;
     }
     RefPtr<MediaDataDecoder> decoder = new FFmpegVideoDecoder<V>(
@@ -48,12 +47,10 @@ class FFmpegDecoderModule : public PlatformDecoderModule {
 
   already_AddRefed<MediaDataDecoder> CreateAudioDecoder(
       const CreateDecoderParams& aParams) override {
-    if (Supports(SupportDecoderParams(aParams), nullptr) ==
-        media::DecodeSupport::Unsupported) {
+    if (Supports(SupportDecoderParams(aParams), nullptr).isEmpty()) {
       return nullptr;
     }
-    RefPtr<MediaDataDecoder> decoder =
-        new FFmpegAudioDecoder<V>(mLib, aParams.AudioConfig());
+    RefPtr<MediaDataDecoder> decoder = new FFmpegAudioDecoder<V>(mLib, aParams);
     return decoder.forget();
   }
 
@@ -62,7 +59,7 @@ class FFmpegDecoderModule : public PlatformDecoderModule {
       DecoderDoctorDiagnostics* aDiagnostics) const override {
     UniquePtr<TrackInfo> trackInfo = CreateTrackInfoWithMIMEType(aMimeType);
     if (!trackInfo) {
-      return media::DecodeSupport::Unsupported;
+      return media::DecodeSupportSet{};
     }
     return Supports(SupportDecoderParams(*trackInfo), aDiagnostics);
   }
@@ -72,7 +69,7 @@ class FFmpegDecoderModule : public PlatformDecoderModule {
       DecoderDoctorDiagnostics* aDiagnostics) const override {
     // This should only be supported by MFMediaEngineDecoderModule.
     if (aParams.mMediaEngineId) {
-      return media::DecodeSupport::Unsupported;
+      return media::DecodeSupportSet{};
     }
 
     const auto& trackInfo = aParams.mConfig;
@@ -86,28 +83,48 @@ class FFmpegDecoderModule : public PlatformDecoderModule {
       MOZ_LOG(sPDMLog, LogLevel::Debug,
               ("FFmpeg decoder rejects requested type '%s'",
                mimeType.BeginReading()));
-      return media::DecodeSupport::Unsupported;
+      return media::DecodeSupportSet{};
+    }
+
+    if (VPXDecoder::IsVP9(mimeType) &&
+        aParams.mOptions.contains(CreateDecoderParams::Option::LowLatency)) {
+      // SVC layers are unsupported, and may be used in low latency use cases
+      // (WebRTC).
+      return media::DecodeSupportSet{};
     }
 
     AVCodecID videoCodec = FFmpegVideoDecoder<V>::GetCodecId(mimeType);
-    AVCodecID audioCodec = FFmpegAudioDecoder<V>::GetCodecId(mimeType);
+    AVCodecID audioCodec = FFmpegAudioDecoder<V>::GetCodecId(
+        mimeType,
+        trackInfo.GetAsAudioInfo() ? *trackInfo.GetAsAudioInfo() : AudioInfo());
     if (audioCodec == AV_CODEC_ID_NONE && videoCodec == AV_CODEC_ID_NONE) {
       MOZ_LOG(sPDMLog, LogLevel::Debug,
               ("FFmpeg decoder rejects requested type '%s'",
                mimeType.BeginReading()));
-      return media::DecodeSupport::Unsupported;
+      return media::DecodeSupportSet{};
     }
-    AVCodecID codec = audioCodec != AV_CODEC_ID_NONE ? audioCodec : videoCodec;
-    bool supports = !!FFmpegDataDecoder<V>::FindAVCodec(mLib, codec);
+    AVCodecID codecId =
+        audioCodec != AV_CODEC_ID_NONE ? audioCodec : videoCodec;
+    AVCodec* codec = FFmpegDataDecoder<V>::FindAVCodec(mLib, codecId);
     MOZ_LOG(sPDMLog, LogLevel::Debug,
             ("FFmpeg decoder %s requested type '%s'",
-             supports ? "supports" : "rejects", mimeType.BeginReading()));
-    if (supports) {
-      // TODO: Note that we do not yet distinguish between SW/HW decode support.
-      //       Will be done in bug 1754239.
-      return media::DecodeSupport::SoftwareDecode;
+             !!codec ? "supports" : "rejects", mimeType.BeginReading()));
+    if (!codec) {
+      return media::DecodeSupportSet{};
     }
-    return media::DecodeSupport::Unsupported;
+    // This logic is mirrored in FFmpegDataDecoder<LIBAV_VER>::InitDecoder and
+    // FFmpegVideoDecoder<LIBAV_VER>::InitVAAPIDecoder. We prefer to use our own
+    // OpenH264 decoder through the plugin over ffmpeg by default due to broken
+    // decoding with some versions.
+    if (!strcmp(codec->name, "libopenh264") &&
+        !StaticPrefs::media_ffmpeg_allow_openh264()) {
+      MOZ_LOG(sPDMLog, LogLevel::Debug,
+              ("FFmpeg decoder rejects as openh264 disabled by pref"));
+      return media::DecodeSupportSet{};
+    }
+    // TODO: Note that we do not yet distinguish between SW/HW decode support.
+    //       Will be done in bug 1754239.
+    return media::DecodeSupport::SoftwareDecode;
   }
 
  protected:
