@@ -9,8 +9,7 @@
 
 #include "ServiceWorkerUpdateJob.h"
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 // The register job.  This implements the steps in the spec Register algorithm,
 // but then uses ServiceWorkerUpdateJob to implement the Update and Install
@@ -19,7 +18,6 @@ class ServiceWorkerRegisterJob final : public ServiceWorkerUpdateJob {
  public:
   ServiceWorkerRegisterJob(nsIPrincipal* aPrincipal, const nsACString& aScope,
                            const nsACString& aScriptSpec,
-                           nsILoadGroup* aLoadGroup,
                            ServiceWorkerUpdateViaCache aUpdateViaCache);
 
  private:
@@ -30,7 +28,6 @@ class ServiceWorkerRegisterJob final : public ServiceWorkerUpdateJob {
   virtual ~ServiceWorkerRegisterJob();
 };
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
 
 #endif  // mozilla_dom_serviceworkerregisterjob_h
