@@ -7,30 +7,37 @@
 #include "ServiceWorkerRegistration.h"
 
 #include "mozilla/dom/DOMMozPromiseRequestHolder.h"
+#include "mozilla/dom/NavigationPreloadManager.h"
+#include "mozilla/dom/NavigationPreloadManagerBinding.h"
 #include "mozilla/dom/Notification.h"
 #include "mozilla/dom/Promise.h"
 #include "mozilla/dom/PushManager.h"
+#include "mozilla/ipc/PBackgroundSharedTypes.h"
 #include "mozilla/dom/ServiceWorker.h"
 #include "mozilla/dom/ServiceWorkerRegistrationBinding.h"
 #include "mozilla/dom/ServiceWorkerUtils.h"
 #include "mozilla/dom/WorkerPrivate.h"
+#include "mozilla/ipc/PBackgroundChild.h"
+#include "mozilla/ipc/BackgroundChild.h"
+#include "mozilla/ScopeExit.h"
 #include "nsCycleCollectionParticipant.h"
 #include "nsPIDOMWindow.h"
-#include "RemoteServiceWorkerRegistrationImpl.h"
-#include "ServiceWorkerRegistrationImpl.h"
+#include "ServiceWorkerRegistrationChild.h"
 
-namespace mozilla {
-namespace dom {
+using mozilla::ipc::ResponseRejectReason;
+
+namespace mozilla::dom {
 
 NS_IMPL_CYCLE_COLLECTION_INHERITED(ServiceWorkerRegistration,
                                    DOMEventTargetHelper, mInstallingWorker,
-                                   mWaitingWorker, mActiveWorker, mPushManager);
+                                   mWaitingWorker, mActiveWorker,
+                                   mNavigationPreloadManager, mPushManager);
 
 NS_IMPL_ADDREF_INHERITED(ServiceWorkerRegistration, DOMEventTargetHelper)
 NS_IMPL_RELEASE_INHERITED(ServiceWorkerRegistration, DOMEventTargetHelper)
 
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(ServiceWorkerRegistration)
-  NS_INTERFACE_MAP_ENTRY(ServiceWorkerRegistration)
+  NS_INTERFACE_MAP_ENTRY_CONCRETE(ServiceWorkerRegistration)
 NS_INTERFACE_MAP_END_INHERITING(DOMEventTargetHelper)
 
 namespace {
@@ -39,23 +46,41 @@ const uint64_t kInvalidUpdateFoundId = 0;
 
 ServiceWorkerRegistration::ServiceWorkerRegistration(
     nsIGlobalObject* aGlobal,
-    const ServiceWorkerRegistrationDescriptor& aDescriptor,
-    ServiceWorkerRegistration::Inner* aInner)
+    const ServiceWorkerRegistrationDescriptor& aDescriptor)
     : DOMEventTargetHelper(aGlobal),
       mDescriptor(aDescriptor),
-      mInner(aInner),
+      mShutdown(false),
       mScheduledUpdateFoundId(kInvalidUpdateFoundId),
       mDispatchedUpdateFoundId(kInvalidUpdateFoundId) {
-  MOZ_DIAGNOSTIC_ASSERT(mInner);
+  ::mozilla::ipc::PBackgroundChild* parentActor =
+      ::mozilla::ipc::BackgroundChild::GetOrCreateForCurrentThread();
+  if (NS_WARN_IF(!parentActor)) {
+    Shutdown();
+    return;
+  }
 
-  KeepAliveIfHasListenersFor(NS_LITERAL_STRING("updatefound"));
+  auto actor = ServiceWorkerRegistrationChild::Create();
+  if (NS_WARN_IF(!actor)) {
+    Shutdown();
+    return;
+  }
 
-  mInner->SetServiceWorkerRegistration(this);
+  PServiceWorkerRegistrationChild* sentActor =
+      parentActor->SendPServiceWorkerRegistrationConstructor(
+          actor, aDescriptor.ToIPC());
+  if (NS_WARN_IF(!sentActor)) {
+    Shutdown();
+    return;
+  }
+  MOZ_DIAGNOSTIC_ASSERT(sentActor == actor);
+
+  mActor = std::move(actor);
+  mActor->SetOwner(this);
+
+  KeepAliveIfHasListenersFor(nsGkAtoms::onupdatefound);
 }
 
-ServiceWorkerRegistration::~ServiceWorkerRegistration() {
-  mInner->ClearServiceWorkerRegistration(this);
-}
+ServiceWorkerRegistration::~ServiceWorkerRegistration() { Shutdown(); }
 
 JSObject* ServiceWorkerRegistration::WrapObject(
     JSContext* aCx, JS::Handle<JSObject*> aGivenProto) {
@@ -70,16 +95,8 @@ ServiceWorkerRegistration::CreateForMainThread(
   MOZ_ASSERT(aWindow);
   MOZ_ASSERT(NS_IsMainThread());
 
-  RefPtr<Inner> inner;
-  if (ServiceWorkerParentInterceptEnabled()) {
-    inner = new RemoteServiceWorkerRegistrationImpl(aDescriptor);
-  } else {
-    inner = new ServiceWorkerRegistrationMainThread(aDescriptor);
-  }
-  NS_ENSURE_TRUE(inner, nullptr);
-
   RefPtr<ServiceWorkerRegistration> registration =
-      new ServiceWorkerRegistration(aWindow->AsGlobal(), aDescriptor, inner);
+      new ServiceWorkerRegistration(aWindow->AsGlobal(), aDescriptor);
   // This is not called from within the constructor, as it may call content code
   // which can cause the deletion of the registration, so we need to keep a
   // strong reference while calling it.
@@ -97,16 +114,8 @@ ServiceWorkerRegistration::CreateForWorker(
   MOZ_DIAGNOSTIC_ASSERT(aGlobal);
   aWorkerPrivate->AssertIsOnWorkerThread();
 
-  RefPtr<Inner> inner;
-  if (ServiceWorkerParentInterceptEnabled()) {
-    inner = new RemoteServiceWorkerRegistrationImpl(aDescriptor);
-  } else {
-    inner = new ServiceWorkerRegistrationWorkerThread(aDescriptor);
-  }
-  NS_ENSURE_TRUE(inner, nullptr);
-
   RefPtr<ServiceWorkerRegistration> registration =
-      new ServiceWorkerRegistration(aGlobal, aDescriptor, inner);
+      new ServiceWorkerRegistration(aGlobal, aDescriptor);
   // This is not called from within the constructor, as it may call content code
   // which can cause the deletion of the registration, so we need to keep a
   // strong reference while calling it.
@@ -133,7 +142,7 @@ void ServiceWorkerRegistration::RegistrationCleared() {
   // Our underlying registration was removed from SWM, so we
   // will never get an updatefound event again.  We can let
   // the object GC if content is not holding it alive.
-  IgnoreKeepAliveIfHasListenersFor(NS_LITERAL_STRING("updatefound"));
+  IgnoreKeepAliveIfHasListenersFor(nsGkAtoms::onupdatefound);
 }
 
 already_AddRefed<ServiceWorker> ServiceWorkerRegistration::GetInstalling()
@@ -152,6 +161,16 @@ already_AddRefed<ServiceWorker> ServiceWorkerRegistration::GetActive() const {
   return ref.forget();
 }
 
+already_AddRefed<NavigationPreloadManager>
+ServiceWorkerRegistration::NavigationPreload() {
+  RefPtr<ServiceWorkerRegistration> reg = this;
+  if (!mNavigationPreloadManager) {
+    mNavigationPreloadManager = MakeRefPtr<NavigationPreloadManager>(reg);
+  }
+  RefPtr<NavigationPreloadManager> ref = mNavigationPreloadManager;
+  return ref.forget();
+}
+
 void ServiceWorkerRegistration::UpdateState(
     const ServiceWorkerRegistrationDescriptor& aDescriptor) {
   MOZ_DIAGNOSTIC_ASSERT(MatchesDescriptor(aDescriptor));
@@ -161,8 +180,8 @@ void ServiceWorkerRegistration::UpdateState(
   UpdateStateInternal(aDescriptor.GetInstalling(), aDescriptor.GetWaiting(),
                       aDescriptor.GetActive());
 
-  nsTArray<UniquePtr<VersionCallback>> callbackList;
-  mVersionCallbackList.SwapElements(callbackList);
+  nsTArray<UniquePtr<VersionCallback>> callbackList =
+      std::move(mVersionCallbackList);
   for (auto& cb : callbackList) {
     if (cb->mVersion > mDescriptor.Version()) {
       mVersionCallbackList.AppendElement(std::move(cb));
@@ -190,11 +209,6 @@ ServiceWorkerUpdateViaCache ServiceWorkerRegistration::GetUpdateViaCache(
 }
 
 already_AddRefed<Promise> ServiceWorkerRegistration::Update(ErrorResult& aRv) {
-  if (!mInner) {
-    aRv.Throw(NS_ERROR_DOM_INVALID_STATE_ERR);
-    return nullptr;
-  }
-
   nsIGlobalObject* global = GetParentObject();
   if (!global) {
     aRv.Throw(NS_ERROR_DOM_INVALID_STATE_ERR);
@@ -236,9 +250,28 @@ already_AddRefed<Promise> ServiceWorkerRegistration::Update(ErrorResult& aRv) {
 
   RefPtr<ServiceWorkerRegistration> self = this;
 
-  mInner->Update(
+  if (!mActor) {
+    outer->MaybeReject(NS_ERROR_DOM_INVALID_STATE_ERR);
+    return outer.forget();
+  }
+
+  mActor->SendUpdate(
       newestWorkerDescriptor.ref().ScriptURL(),
-      [outer, self](const ServiceWorkerRegistrationDescriptor& aDesc) {
+      [outer,
+       self](const IPCServiceWorkerRegistrationDescriptorOrCopyableErrorResult&
+                 aResult) {
+        if (aResult.type() ==
+            IPCServiceWorkerRegistrationDescriptorOrCopyableErrorResult::
+                TCopyableErrorResult) {
+          // application layer error
+          const auto& rv = aResult.get_CopyableErrorResult();
+          MOZ_DIAGNOSTIC_ASSERT(rv.Failed());
+          outer->MaybeReject(CopyableErrorResult(rv));
+          return;
+        }
+        // success
+        const auto& ipcDesc =
+            aResult.get_IPCServiceWorkerRegistrationDescriptor();
         nsIGlobalObject* global = self->GetParentObject();
         // It's possible this binding was detached from the global.  In cases
         // where we use IPC with Promise callbacks, we use
@@ -260,14 +293,18 @@ already_AddRefed<Promise> ServiceWorkerRegistration::Update(ErrorResult& aRv) {
           return;
         }
         RefPtr<ServiceWorkerRegistration> ref =
-            global->GetOrCreateServiceWorkerRegistration(aDesc);
+            global->GetOrCreateServiceWorkerRegistration(
+                ServiceWorkerRegistrationDescriptor(ipcDesc));
         if (!ref) {
           outer->MaybeReject(NS_ERROR_DOM_INVALID_STATE_ERR);
           return;
         }
         outer->MaybeResolve(ref);
       },
-      [outer, self](ErrorResult&& aRv) { outer->MaybeReject(std::move(aRv)); });
+      [outer](ResponseRejectReason&& aReason) {
+        // IPC layer error
+        outer->MaybeReject(NS_ERROR_DOM_INVALID_STATE_ERR);
+      });
 
   return outer.forget();
 }
@@ -285,18 +322,28 @@ already_AddRefed<Promise> ServiceWorkerRegistration::Unregister(
     return nullptr;
   }
 
-  if (!mInner) {
+  if (!mActor) {
     outer->MaybeResolve(false);
     return outer.forget();
   }
 
-  mInner->Unregister([outer](bool aSuccess) { outer->MaybeResolve(aSuccess); },
-                     [outer](ErrorResult&& aRv) {
-                       // register() should be resilient and resolve false
-                       // instead of rejecting in most cases.
-                       aRv.SuppressException();
-                       outer->MaybeResolve(false);
-                     });
+  mActor->SendUnregister(
+      [outer](std::tuple<bool, CopyableErrorResult>&& aResult) {
+        if (std::get<1>(aResult).Failed()) {
+          // application layer error
+          // register() should be resilient and resolve false instead of
+          // rejecting in most cases.
+          std::get<1>(aResult).SuppressException();
+          outer->MaybeResolve(false);
+          return;
+        }
+        // success
+        outer->MaybeResolve(std::get<0>(aResult));
+      },
+      [outer](ResponseRejectReason&& aReason) {
+        // IPC layer error
+        outer->MaybeResolve(false);
+      });
 
   return outer.forget();
 }
@@ -323,15 +370,20 @@ already_AddRefed<PushManager> ServiceWorkerRegistration::GetPushManager(
   return ret.forget();
 }
 
+// https://notifications.spec.whatwg.org/#dom-serviceworkerregistration-shownotification
 already_AddRefed<Promise> ServiceWorkerRegistration::ShowNotification(
     JSContext* aCx, const nsAString& aTitle,
     const NotificationOptions& aOptions, ErrorResult& aRv) {
+  // Step 1: Let global be this’s relevant global object.
   nsIGlobalObject* global = GetParentObject();
   if (!global) {
     aRv.Throw(NS_ERROR_DOM_INVALID_STATE_ERR);
     return nullptr;
   }
 
+  // Step 3: If this’s active worker is null, then reject promise with a
+  // TypeError and return promise.
+  //
   // Until we ship ServiceWorker objects on worker threads the active
   // worker will always be nullptr.  So limit this check to main
   // thread for now.
@@ -342,12 +394,14 @@ already_AddRefed<Promise> ServiceWorkerRegistration::ShowNotification(
 
   NS_ConvertUTF8toUTF16 scope(mDescriptor.Scope());
 
+  // Step 2, 5, 6
   RefPtr<Promise> p = Notification::ShowPersistentNotification(
       aCx, global, scope, aTitle, aOptions, mDescriptor, aRv);
   if (NS_WARN_IF(aRv.Failed())) {
     return nullptr;
   }
 
+  // Step 7: Return promise.
   return p.forget();
 }
 
@@ -375,6 +429,76 @@ already_AddRefed<Promise> ServiceWorkerRegistration::GetNotifications(
   return Notification::WorkerGet(worker, aOptions, scope, aRv);
 }
 
+void ServiceWorkerRegistration::SetNavigationPreloadEnabled(
+    bool aEnabled, ServiceWorkerBoolCallback&& aSuccessCB,
+    ServiceWorkerFailureCallback&& aFailureCB) {
+  if (!mActor) {
+    aFailureCB(CopyableErrorResult(NS_ERROR_DOM_INVALID_STATE_ERR));
+    return;
+  }
+
+  mActor->SendSetNavigationPreloadEnabled(
+      aEnabled,
+      [successCB = std::move(aSuccessCB), aFailureCB](bool aResult) {
+        if (!aResult) {
+          aFailureCB(CopyableErrorResult(NS_ERROR_DOM_INVALID_STATE_ERR));
+          return;
+        }
+        successCB(aResult);
+      },
+      [aFailureCB](ResponseRejectReason&& aReason) {
+        aFailureCB(CopyableErrorResult(NS_ERROR_DOM_INVALID_STATE_ERR));
+      });
+}
+
+void ServiceWorkerRegistration::SetNavigationPreloadHeader(
+    const nsCString& aHeader, ServiceWorkerBoolCallback&& aSuccessCB,
+    ServiceWorkerFailureCallback&& aFailureCB) {
+  if (!mActor) {
+    aFailureCB(CopyableErrorResult(NS_ERROR_DOM_INVALID_STATE_ERR));
+    return;
+  }
+
+  mActor->SendSetNavigationPreloadHeader(
+      aHeader,
+      [successCB = std::move(aSuccessCB), aFailureCB](bool aResult) {
+        if (!aResult) {
+          aFailureCB(CopyableErrorResult(NS_ERROR_DOM_INVALID_STATE_ERR));
+          return;
+        }
+        successCB(aResult);
+      },
+      [aFailureCB](ResponseRejectReason&& aReason) {
+        aFailureCB(CopyableErrorResult(NS_ERROR_DOM_INVALID_STATE_ERR));
+      });
+}
+
+void ServiceWorkerRegistration::GetNavigationPreloadState(
+    NavigationPreloadGetStateCallback&& aSuccessCB,
+    ServiceWorkerFailureCallback&& aFailureCB) {
+  if (!mActor) {
+    aFailureCB(CopyableErrorResult(NS_ERROR_DOM_INVALID_STATE_ERR));
+    return;
+  }
+
+  mActor->SendGetNavigationPreloadState(
+      [successCB = std::move(aSuccessCB),
+       aFailureCB](Maybe<IPCNavigationPreloadState>&& aState) {
+        if (NS_WARN_IF(!aState)) {
+          aFailureCB(CopyableErrorResult(NS_ERROR_DOM_INVALID_STATE_ERR));
+          return;
+        }
+
+        NavigationPreloadState state;
+        state.mEnabled = aState.ref().enabled();
+        state.mHeaderValue.Construct(std::move(aState.ref().headerValue()));
+        successCB(std::move(state));
+      },
+      [aFailureCB](ResponseRejectReason&& aReason) {
+        aFailureCB(CopyableErrorResult(NS_ERROR_DOM_INVALID_STATE_ERR));
+      });
+}
+
 const ServiceWorkerRegistrationDescriptor&
 ServiceWorkerRegistration::Descriptor() const {
   return mDescriptor;
@@ -431,8 +555,8 @@ void ServiceWorkerRegistration::MaybeDispatchUpdateFoundRunnable() {
       "ServiceWorkerRegistration::MaybeDispatchUpdateFound", this,
       &ServiceWorkerRegistration::MaybeDispatchUpdateFound);
 
-  Unused << global->EventTargetFor(TaskCategory::Other)
-                ->Dispatch(r.forget(), NS_DISPATCH_NORMAL);
+  Unused << global->SerialEventTarget()->Dispatch(r.forget(),
+                                                  NS_DISPATCH_NORMAL);
 }
 
 void ServiceWorkerRegistration::MaybeDispatchUpdateFound() {
@@ -445,7 +569,7 @@ void ServiceWorkerRegistration::MaybeDispatchUpdateFound() {
   }
 
   mDispatchedUpdateFoundId = scheduledId;
-  DispatchTrustedEvent(NS_LITERAL_STRING("updatefound"));
+  DispatchTrustedEvent(u"updatefound"_ns);
 }
 
 void ServiceWorkerRegistration::UpdateStateInternal(
@@ -543,5 +667,29 @@ void ServiceWorkerRegistration::UpdateStateInternal(
   }
 }
 
-}  // namespace dom
-}  // namespace mozilla
+void ServiceWorkerRegistration::RevokeActor(
+    ServiceWorkerRegistrationChild* aActor) {
+  MOZ_DIAGNOSTIC_ASSERT(mActor);
+  MOZ_DIAGNOSTIC_ASSERT(mActor == aActor);
+  mActor->RevokeOwner(this);
+  mActor = nullptr;
+
+  mShutdown = true;
+
+  RegistrationCleared();
+}
+
+void ServiceWorkerRegistration::Shutdown() {
+  if (mShutdown) {
+    return;
+  }
+  mShutdown = true;
+
+  if (mActor) {
+    mActor->RevokeOwner(this);
+    mActor->MaybeStartTeardown();
+    mActor = nullptr;
+  }
+}
+
+}  // namespace mozilla::dom
