@@ -8,9 +8,13 @@
 
 #include "ServiceWorkerManager.h"
 #include "ServiceWorkerPrivate.h"
+#include "ServiceWorkerRegistrationListener.h"
 
-namespace mozilla {
-namespace dom {
+#include "mozilla/Preferences.h"
+#include "mozilla/SchedulerGroup.h"
+#include "mozilla/StaticPrefs_dom.h"
+
+namespace mozilla::dom {
 
 namespace {
 
@@ -38,56 +42,91 @@ class ContinueActivateRunnable final : public LifeCycleEventCallback {
 
 }  // anonymous namespace
 
-void ServiceWorkerRegistrationInfo::Clear() {
-  if (mEvaluatingWorker) {
-    mEvaluatingWorker = nullptr;
-  }
-
-  RefPtr<ServiceWorkerInfo> installing = mInstallingWorker.forget();
-  RefPtr<ServiceWorkerInfo> waiting = mWaitingWorker.forget();
-  RefPtr<ServiceWorkerInfo> active = mActiveWorker.forget();
-
-  UpdateRegistrationState();
-
-  if (installing) {
-    installing->UpdateState(ServiceWorkerState::Redundant);
-    installing->UpdateRedundantTime();
-    installing->WorkerPrivate()->NoteDeadServiceWorkerInfo();
-    // FIXME(nsm): Abort any inflight requests from installing worker.
-  }
-
-  if (waiting) {
-    waiting->UpdateState(ServiceWorkerState::Redundant);
-    waiting->UpdateRedundantTime();
-    waiting->WorkerPrivate()->NoteDeadServiceWorkerInfo();
-  }
-
-  if (active) {
-    active->UpdateState(ServiceWorkerState::Redundant);
-    active->UpdateRedundantTime();
-    active->WorkerPrivate()->NoteDeadServiceWorkerInfo();
-  }
-
-  NotifyChromeRegistrationListeners();
+void ServiceWorkerRegistrationInfo::ShutdownWorkers() {
+  ForEachWorker([](RefPtr<ServiceWorkerInfo>& aWorker) {
+    aWorker->WorkerPrivate()->NoteDeadServiceWorkerInfo();
+    aWorker = nullptr;
+  });
 }
 
+void ServiceWorkerRegistrationInfo::Clear() {
+  ForEachWorker([](RefPtr<ServiceWorkerInfo>& aWorker) {
+    aWorker->UpdateState(ServiceWorkerState::Redundant);
+    aWorker->UpdateRedundantTime();
+  });
+
+  // FIXME: Abort any inflight requests from installing worker.
+
+  ShutdownWorkers();
+  UpdateRegistrationState();
+  NotifyChromeRegistrationListeners();
+  NotifyCleared();
+}
+
+void ServiceWorkerRegistrationInfo::ClearAsCorrupt() {
+  mCorrupt = true;
+  Clear();
+}
+
+bool ServiceWorkerRegistrationInfo::IsCorrupt() const { return mCorrupt; }
+
 ServiceWorkerRegistrationInfo::ServiceWorkerRegistrationInfo(
     const nsACString& aScope, nsIPrincipal* aPrincipal,
-    ServiceWorkerUpdateViaCache aUpdateViaCache)
+    ServiceWorkerUpdateViaCache aUpdateViaCache,
+    IPCNavigationPreloadState&& aNavigationPreloadState)
     : mPrincipal(aPrincipal),
-      mDescriptor(GetNextId(), aPrincipal, aScope, aUpdateViaCache),
+      mDescriptor(GetNextId(), GetNextVersion(), aPrincipal, aScope,
+                  aUpdateViaCache),
       mControlledClientsCounter(0),
       mDelayMultiplier(0),
       mUpdateState(NoUpdate),
       mCreationTime(PR_Now()),
       mCreationTimeStamp(TimeStamp::Now()),
       mLastUpdateTime(0),
-      mPendingUninstall(false) {}
+      mUnregistered(false),
+      mCorrupt(false),
+      mNavigationPreloadState(std::move(aNavigationPreloadState)) {
+  MOZ_ASSERT(XRE_GetProcessType() == GeckoProcessType_Default);
+}
 
 ServiceWorkerRegistrationInfo::~ServiceWorkerRegistrationInfo() {
   MOZ_DIAGNOSTIC_ASSERT(!IsControllingClients());
 }
 
+void ServiceWorkerRegistrationInfo::AddInstance(
+    ServiceWorkerRegistrationListener* aInstance,
+    const ServiceWorkerRegistrationDescriptor& aDescriptor) {
+  MOZ_DIAGNOSTIC_ASSERT(aInstance);
+  MOZ_ASSERT(!mInstanceList.Contains(aInstance));
+  MOZ_DIAGNOSTIC_ASSERT(aDescriptor.Id() == mDescriptor.Id());
+  MOZ_DIAGNOSTIC_ASSERT(aDescriptor.PrincipalInfo() ==
+                        mDescriptor.PrincipalInfo());
+  MOZ_DIAGNOSTIC_ASSERT(aDescriptor.Scope() == mDescriptor.Scope());
+  MOZ_DIAGNOSTIC_ASSERT(aDescriptor.Version() <= mDescriptor.Version());
+  uint64_t lastVersion = aDescriptor.Version();
+  for (auto& entry : mVersionList) {
+    if (lastVersion > entry->mDescriptor.Version()) {
+      continue;
+    }
+    lastVersion = entry->mDescriptor.Version();
+    aInstance->UpdateState(entry->mDescriptor);
+  }
+  // Note, the mDescriptor may be contained in the version list.  Since the
+  // version list is aged out, though, it may also not be in the version list.
+  // So always check for the mDescriptor update here.
+  if (lastVersion < mDescriptor.Version()) {
+    aInstance->UpdateState(mDescriptor);
+  }
+  mInstanceList.AppendElement(aInstance);
+}
+
+void ServiceWorkerRegistrationInfo::RemoveInstance(
+    ServiceWorkerRegistrationListener* aInstance) {
+  MOZ_DIAGNOSTIC_ASSERT(aInstance);
+  DebugOnly<bool> removed = mInstanceList.RemoveElement(aInstance);
+  MOZ_ASSERT(removed);
+}
+
 const nsCString& ServiceWorkerRegistrationInfo::Scope() const {
   return mDescriptor.Scope();
 }
@@ -96,16 +135,24 @@ nsIPrincipal* ServiceWorkerRegistrationInfo::Principal() const {
   return mPrincipal;
 }
 
-bool ServiceWorkerRegistrationInfo::IsPendingUninstall() const {
-  return mPendingUninstall;
+bool ServiceWorkerRegistrationInfo::IsUnregistered() const {
+  return mUnregistered;
 }
 
-void ServiceWorkerRegistrationInfo::SetPendingUninstall() {
-  mPendingUninstall = true;
-}
+void ServiceWorkerRegistrationInfo::SetUnregistered() {
+#ifdef DEBUG
+  MOZ_ASSERT(!mUnregistered);
 
-void ServiceWorkerRegistrationInfo::ClearPendingUninstall() {
-  mPendingUninstall = false;
+  RefPtr<ServiceWorkerManager> swm = ServiceWorkerManager::GetInstance();
+  MOZ_ASSERT(swm);
+
+  RefPtr<ServiceWorkerRegistrationInfo> registration =
+      swm->GetRegistration(Principal(), Scope());
+  MOZ_ASSERT(registration != this);
+#endif
+
+  mUnregistered = true;
+  NotifyChromeRegistrationListeners();
 }
 
 NS_IMPL_ISUPPORTS(ServiceWorkerRegistrationInfo,
@@ -118,6 +165,14 @@ ServiceWorkerRegistrationInfo::GetPrincipal(nsIPrincipal** aPrincipal) {
   return NS_OK;
 }
 
+NS_IMETHODIMP ServiceWorkerRegistrationInfo::GetUnregistered(
+    bool* aUnregistered) {
+  MOZ_ASSERT(NS_IsMainThread());
+  MOZ_ASSERT(aUnregistered);
+  *aUnregistered = mUnregistered;
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 ServiceWorkerRegistrationInfo::GetScope(nsAString& aScope) {
   MOZ_ASSERT(NS_IsMainThread());
@@ -128,7 +183,7 @@ ServiceWorkerRegistrationInfo::GetScope(nsAString& aScope) {
 NS_IMETHODIMP
 ServiceWorkerRegistrationInfo::GetScriptSpec(nsAString& aScriptSpec) {
   MOZ_ASSERT(NS_IsMainThread());
-  RefPtr<ServiceWorkerInfo> newest = Newest();
+  RefPtr<ServiceWorkerInfo> newest = NewestIncludingEvaluating();
   if (newest) {
     CopyUTF8toUTF16(newest->ScriptSpec(), aScriptSpec);
   }
@@ -149,11 +204,20 @@ ServiceWorkerRegistrationInfo::GetLastUpdateTime(PRTime* _retval) {
   return NS_OK;
 }
 
+NS_IMETHODIMP
+ServiceWorkerRegistrationInfo::GetEvaluatingWorker(
+    nsIServiceWorkerInfo** aResult) {
+  MOZ_ASSERT(NS_IsMainThread());
+  RefPtr<ServiceWorkerInfo> info = mEvaluatingWorker;
+  info.forget(aResult);
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 ServiceWorkerRegistrationInfo::GetInstallingWorker(
     nsIServiceWorkerInfo** aResult) {
   MOZ_ASSERT(NS_IsMainThread());
-  nsCOMPtr<nsIServiceWorkerInfo> info = do_QueryInterface(mInstallingWorker);
+  RefPtr<ServiceWorkerInfo> info = mInstallingWorker;
   info.forget(aResult);
   return NS_OK;
 }
@@ -162,7 +226,7 @@ NS_IMETHODIMP
 ServiceWorkerRegistrationInfo::GetWaitingWorker(
     nsIServiceWorkerInfo** aResult) {
   MOZ_ASSERT(NS_IsMainThread());
-  nsCOMPtr<nsIServiceWorkerInfo> info = do_QueryInterface(mWaitingWorker);
+  RefPtr<ServiceWorkerInfo> info = mWaitingWorker;
   info.forget(aResult);
   return NS_OK;
 }
@@ -170,11 +234,28 @@ ServiceWorkerRegistrationInfo::GetWaitingWorker(
 NS_IMETHODIMP
 ServiceWorkerRegistrationInfo::GetActiveWorker(nsIServiceWorkerInfo** aResult) {
   MOZ_ASSERT(NS_IsMainThread());
-  nsCOMPtr<nsIServiceWorkerInfo> info = do_QueryInterface(mActiveWorker);
+  RefPtr<ServiceWorkerInfo> info = mActiveWorker;
   info.forget(aResult);
   return NS_OK;
 }
 
+NS_IMETHODIMP
+ServiceWorkerRegistrationInfo::GetQuotaUsageCheckCount(
+    int32_t* aQuotaUsageCheckCount) {
+  MOZ_ASSERT(NS_IsMainThread());
+  MOZ_ASSERT(aQuotaUsageCheckCount);
+
+  // This value is actually stored on SWM's internal-only
+  // RegistrationDataPerPrincipal structure, but we expose it here for
+  // simplicity for our consumers, so we have to ask SWM to look it up for us.
+  RefPtr<ServiceWorkerManager> swm = ServiceWorkerManager::GetInstance();
+  MOZ_ASSERT(swm);
+
+  *aQuotaUsageCheckCount = swm->GetPrincipalQuotaUsageCheckCount(mPrincipal);
+
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 ServiceWorkerRegistrationInfo::GetWorkerByID(uint64_t aID,
                                              nsIServiceWorkerInfo** aResult) {
@@ -215,6 +296,13 @@ ServiceWorkerRegistrationInfo::RemoveListener(
   return NS_OK;
 }
 
+NS_IMETHODIMP
+ServiceWorkerRegistrationInfo::ForceShutdown() {
+  ClearInstalling();
+  ShutdownWorkers();
+  return NS_OK;
+}
+
 already_AddRefed<ServiceWorkerInfo>
 ServiceWorkerRegistrationInfo::GetServiceWorkerInfoById(uint64_t aId) {
   MOZ_ASSERT(NS_IsMainThread());
@@ -233,16 +321,20 @@ ServiceWorkerRegistrationInfo::GetServiceWorkerInfoById(uint64_t aId) {
   return serviceWorker.forget();
 }
 
-void ServiceWorkerRegistrationInfo::TryToActivateAsync() {
+void ServiceWorkerRegistrationInfo::TryToActivateAsync(
+    TryToActivateCallback&& aCallback) {
   MOZ_ALWAYS_SUCCEEDS(NS_DispatchToMainThread(
-      NewRunnableMethod("ServiceWorkerRegistrationInfo::TryToActivate", this,
-                        &ServiceWorkerRegistrationInfo::TryToActivate)));
+      NewRunnableMethod<StoreCopyPassByRRef<TryToActivateCallback>>(
+          "ServiceWorkerRegistrationInfo::TryToActivate", this,
+          &ServiceWorkerRegistrationInfo::TryToActivate,
+          std::move(aCallback))));
 }
 
 /*
  * TryToActivate should not be called directly, use TryToActivateAsync instead.
  */
-void ServiceWorkerRegistrationInfo::TryToActivate() {
+void ServiceWorkerRegistrationInfo::TryToActivate(
+    TryToActivateCallback&& aCallback) {
   MOZ_ASSERT(NS_IsMainThread());
   bool controlling = IsControllingClients();
   bool skipWaiting = mWaitingWorker && mWaitingWorker->SkipWaitingFlag();
@@ -250,6 +342,10 @@ void ServiceWorkerRegistrationInfo::TryToActivate() {
   if (idle && (!controlling || skipWaiting)) {
     Activate();
   }
+
+  if (aCallback) {
+    aCallback();
+  }
 }
 
 void ServiceWorkerRegistrationInfo::Activate() {
@@ -271,10 +367,6 @@ void ServiceWorkerRegistrationInfo::Activate() {
   MOZ_DIAGNOSTIC_ASSERT(mActiveWorker);
   swm->UpdateClientControllers(this);
 
-  nsCOMPtr<nsIRunnable> failRunnable = NewRunnableMethod<bool>(
-      "dom::ServiceWorkerRegistrationInfo::FinishActivate", this,
-      &ServiceWorkerRegistrationInfo::FinishActivate, false /* success */);
-
   nsMainThreadPtrHandle<ServiceWorkerRegistrationInfo> handle(
       new nsMainThreadPtrHolder<ServiceWorkerRegistrationInfo>(
           "ServiceWorkerRegistrationInfoProxy", this));
@@ -283,16 +375,18 @@ void ServiceWorkerRegistrationInfo::Activate() {
 
   ServiceWorkerPrivate* workerPrivate = mActiveWorker->WorkerPrivate();
   MOZ_ASSERT(workerPrivate);
-  nsresult rv = workerPrivate->SendLifeCycleEvent(NS_LITERAL_STRING("activate"),
-                                                  callback, failRunnable);
+  nsresult rv = workerPrivate->SendLifeCycleEvent(u"activate"_ns, callback);
   if (NS_WARN_IF(NS_FAILED(rv))) {
-    MOZ_ALWAYS_SUCCEEDS(NS_DispatchToMainThread(failRunnable));
+    nsCOMPtr<nsIRunnable> failRunnable = NewRunnableMethod<bool>(
+        "dom::ServiceWorkerRegistrationInfo::FinishActivate", this,
+        &ServiceWorkerRegistrationInfo::FinishActivate, false /* success */);
+    MOZ_ALWAYS_SUCCEEDS(NS_DispatchToMainThread(failRunnable.forget()));
     return;
   }
 }
 
 void ServiceWorkerRegistrationInfo::FinishActivate(bool aSuccess) {
-  if (mPendingUninstall || !mActiveWorker ||
+  if (mUnregistered || !mActiveWorker ||
       mActiveWorker->State() != ServiceWorkerState::Activating) {
     return;
   }
@@ -301,8 +395,7 @@ void ServiceWorkerRegistrationInfo::FinishActivate(bool aSuccess) {
   mActiveWorker->UpdateState(ServiceWorkerState::Activated);
   mActiveWorker->UpdateActivatedTime();
 
-  mDescriptor.SetWorkers(mInstallingWorker, mWaitingWorker, mActiveWorker);
-
+  UpdateRegistrationState();
   NotifyChromeRegistrationListeners();
 
   RefPtr<ServiceWorkerManager> swm = ServiceWorkerManager::GetInstance();
@@ -347,25 +440,42 @@ bool ServiceWorkerRegistrationInfo::IsLastUpdateCheckTimeOverOneDay() const {
 }
 
 void ServiceWorkerRegistrationInfo::UpdateRegistrationState() {
+  UpdateRegistrationState(mDescriptor.UpdateViaCache());
+}
+
+void ServiceWorkerRegistrationInfo::UpdateRegistrationState(
+    ServiceWorkerUpdateViaCache aUpdateViaCache) {
   MOZ_ASSERT(NS_IsMainThread());
 
+  TimeStamp oldest = TimeStamp::Now() - TimeDuration::FromSeconds(30);
+  if (!mVersionList.IsEmpty() && mVersionList[0]->mTimeStamp < oldest) {
+    nsTArray<UniquePtr<VersionEntry>> list = std::move(mVersionList);
+    for (auto& entry : list) {
+      if (entry->mTimeStamp >= oldest) {
+        mVersionList.AppendElement(std::move(entry));
+      }
+    }
+  }
+  mVersionList.AppendElement(MakeUnique<VersionEntry>(mDescriptor));
+
+  // We are going to modify the descriptor, so increase its version number.
+  mDescriptor.SetVersion(GetNextVersion());
+
+  // Note, this also sets the new version number on the ServiceWorkerInfo
+  // objects before we copy over their updated descriptors.
   mDescriptor.SetWorkers(mInstallingWorker, mWaitingWorker, mActiveWorker);
 
-  RefPtr<ServiceWorkerRegistrationInfo> self(this);
-  nsCOMPtr<nsIRunnable> r = NS_NewRunnableFunction(
-      "ServiceWorkerRegistrationInfo::UpdateRegistrationState",
-      [self = Move(self)] {
-        RefPtr<ServiceWorkerManager> swm = ServiceWorkerManager::GetInstance();
-        if (swm) {
-          swm->UpdateRegistrationListeners(self);
-        }
-      });
-  MOZ_ALWAYS_SUCCEEDS(SystemGroup::Dispatch(TaskCategory::Other, r.forget()));
+  mDescriptor.SetUpdateViaCache(aUpdateViaCache);
+
+  for (RefPtr<ServiceWorkerRegistrationListener> pinnedTarget :
+       mInstanceList.ForwardRange()) {
+    pinnedTarget->UpdateState(mDescriptor);
+  }
 }
 
 void ServiceWorkerRegistrationInfo::NotifyChromeRegistrationListeners() {
   nsTArray<nsCOMPtr<nsIServiceWorkerRegistrationInfoListener>> listeners(
-      mListeners);
+      mListeners.Clone());
   for (size_t index = 0; index < listeners.Length(); ++index) {
     listeners[index]->OnChange();
   }
@@ -396,6 +506,22 @@ void ServiceWorkerRegistrationInfo::MaybeScheduleUpdate() {
     return;
   }
 
+  // When reach the navigation fault threshold, calling unregister instead of
+  // scheduling update.
+  if (mActiveWorker && !mUnregistered) {
+    uint32_t navigationFaultCount;
+    mActiveWorker->GetNavigationFaultCount(&navigationFaultCount);
+    const auto navigationFaultThreshold = StaticPrefs::
+        dom_serviceWorkers_mitigations_navigation_fault_threshold();
+    // Disable unregister mitigation when navigation fault threshold is 0.
+    if (navigationFaultThreshold <= navigationFaultCount &&
+        navigationFaultThreshold != 0) {
+      CheckQuotaUsage();
+      swm->Unregister(mPrincipal, nullptr, NS_ConvertUTF8toUTF16(Scope()));
+      return;
+    }
+  }
+
   mUpdateState = NeedUpdate;
 
   swm->ScheduleUpdateTimer(mPrincipal, Scope());
@@ -462,6 +588,11 @@ void ServiceWorkerRegistrationInfo::SetEvaluating(
   MOZ_ASSERT(mActiveWorker != aServiceWorker);
 
   mEvaluatingWorker = aServiceWorker;
+
+  // We don't call UpdateRegistrationState() here because the evaluating worker
+  // is currently not exposed to content on the registration, so calling it here
+  // would produce redundant IPC traffic.
+  NotifyChromeRegistrationListeners();
 }
 
 void ServiceWorkerRegistrationInfo::ClearEvaluating() {
@@ -475,6 +606,9 @@ void ServiceWorkerRegistrationInfo::ClearEvaluating() {
   // We don't update the redundant time for the sw here, since we've not expose
   // evalutingWorker yet.
   mEvaluatingWorker = nullptr;
+
+  // As for SetEvaluating, UpdateRegistrationState() does not need to be called.
+  NotifyChromeRegistrationListeners();
 }
 
 void ServiceWorkerRegistrationInfo::ClearInstalling() {
@@ -484,13 +618,11 @@ void ServiceWorkerRegistrationInfo::ClearInstalling() {
     return;
   }
 
-  RefPtr<ServiceWorkerInfo> installing = mInstallingWorker.forget();
-
-  UpdateRegistrationState();
-
+  RefPtr<ServiceWorkerInfo> installing = std::move(mInstallingWorker);
   installing->UpdateState(ServiceWorkerState::Redundant);
   installing->UpdateRedundantTime();
 
+  UpdateRegistrationState();
   NotifyChromeRegistrationListeners();
 }
 
@@ -499,12 +631,10 @@ void ServiceWorkerRegistrationInfo::TransitionEvaluatingToInstalling() {
   MOZ_ASSERT(mEvaluatingWorker);
   MOZ_ASSERT(!mInstallingWorker);
 
-  mInstallingWorker = mEvaluatingWorker.forget();
-
-  UpdateRegistrationState();
-
+  mInstallingWorker = std::move(mEvaluatingWorker);
   mInstallingWorker->UpdateState(ServiceWorkerState::Installing);
 
+  UpdateRegistrationState();
   NotifyChromeRegistrationListeners();
 }
 
@@ -518,21 +648,15 @@ void ServiceWorkerRegistrationInfo::TransitionInstallingToWaiting() {
     mWaitingWorker->UpdateRedundantTime();
   }
 
-  mWaitingWorker = mInstallingWorker.forget();
-
-  UpdateRegistrationState();
-
+  mWaitingWorker = std::move(mInstallingWorker);
   mWaitingWorker->UpdateState(ServiceWorkerState::Installed);
   mWaitingWorker->UpdateInstalledTime();
 
+  UpdateRegistrationState();
   NotifyChromeRegistrationListeners();
 
-  RefPtr<ServiceWorkerManager> swm = ServiceWorkerManager::GetInstance();
-  if (!swm) {
-    // browser shutdown began
-    return;
-  }
-  swm->StoreRegistration(mPrincipal, this);
+  // TODO: When bug 1426401 is implemented we will need to call
+  //       StoreRegistration() here to persist the waiting worker.
 }
 
 void ServiceWorkerRegistrationInfo::SetActive(
@@ -564,7 +688,6 @@ void ServiceWorkerRegistrationInfo::SetActive(
   // We don't need to update activated time when we load registration from
   // registrar.
   UpdateRegistrationState();
-
   NotifyChromeRegistrationListeners();
 }
 
@@ -580,10 +703,7 @@ void ServiceWorkerRegistrationInfo::TransitionWaitingToActive() {
 
   // We are transitioning from waiting to active normally, so go to
   // the activating state.
-  mActiveWorker = mWaitingWorker.forget();
-
-  UpdateRegistrationState();
-
+  mActiveWorker = std::move(mWaitingWorker);
   mActiveWorker->UpdateState(ServiceWorkerState::Activating);
 
   nsCOMPtr<nsIRunnable> r = NS_NewRunnableFunction(
@@ -593,8 +713,8 @@ void ServiceWorkerRegistrationInfo::TransitionWaitingToActive() {
           swm->CheckPendingReadyPromises();
         }
       });
-  MOZ_ALWAYS_SUCCEEDS(SystemGroup::Dispatch(TaskCategory::Other, r.forget()));
-
+  MOZ_ALWAYS_SUCCEEDS(SchedulerGroup::Dispatch(r.forget()));
+  UpdateRegistrationState();
   NotifyChromeRegistrationListeners();
 }
 
@@ -609,8 +729,7 @@ ServiceWorkerUpdateViaCache ServiceWorkerRegistrationInfo::GetUpdateViaCache()
 
 void ServiceWorkerRegistrationInfo::SetUpdateViaCache(
     ServiceWorkerUpdateViaCache aUpdateViaCache) {
-  mDescriptor.SetUpdateViaCache(aUpdateViaCache);
-  UpdateRegistrationState();
+  UpdateRegistrationState(aUpdateViaCache);
 }
 
 int64_t ServiceWorkerRegistrationInfo::GetLastUpdateTime() const {
@@ -630,8 +749,20 @@ ServiceWorkerRegistrationInfo::Descriptor() const {
   return mDescriptor;
 }
 
-uint32_t ServiceWorkerRegistrationInfo::GetUpdateDelay() {
+uint64_t ServiceWorkerRegistrationInfo::Id() const { return mDescriptor.Id(); }
+
+uint64_t ServiceWorkerRegistrationInfo::Version() const {
+  return mDescriptor.Version();
+}
+
+uint32_t ServiceWorkerRegistrationInfo::GetUpdateDelay(
+    const bool aWithMultiplier) {
   uint32_t delay = Preferences::GetInt("dom.serviceWorkers.update_delay", 1000);
+
+  if (!aWithMultiplier) {
+    return delay;
+  }
+
   // This can potentially happen if you spam registration->Update(). We don't
   // want to wrap to a lower value.
   if (mDelayMultiplier >= INT_MAX / (delay ? delay : 1)) {
@@ -647,6 +778,88 @@ uint32_t ServiceWorkerRegistrationInfo::GetUpdateDelay() {
   return delay;
 }
 
+void ServiceWorkerRegistrationInfo::FireUpdateFound() {
+  for (RefPtr<ServiceWorkerRegistrationListener> pinnedTarget :
+       mInstanceList.ForwardRange()) {
+    pinnedTarget->FireUpdateFound();
+  }
+}
+
+void ServiceWorkerRegistrationInfo::NotifyCleared() {
+  for (RefPtr<ServiceWorkerRegistrationListener> pinnedTarget :
+       mInstanceList.ForwardRange()) {
+    pinnedTarget->RegistrationCleared();
+  }
+}
+
+void ServiceWorkerRegistrationInfo::ClearWhenIdle() {
+  MOZ_ASSERT(NS_IsMainThread());
+  MOZ_ASSERT(IsUnregistered());
+  MOZ_ASSERT(!IsControllingClients());
+  MOZ_ASSERT(!IsIdle(), "Already idle!");
+
+  RefPtr<ServiceWorkerManager> swm = ServiceWorkerManager::GetInstance();
+  MOZ_ASSERT(swm);
+
+  swm->AddOrphanedRegistration(this);
+
+  /**
+   * Although a Service Worker will transition to idle many times during its
+   * lifetime, the promise is only resolved once `GetIdlePromise` has been
+   * called, populating the `MozPromiseHolder`. Additionally, this is the only
+   * time this method will be called for the given ServiceWorker. This means we
+   * will be notified to the transition we are interested in, and there are no
+   * other callers to get confused.
+   *
+   * Note that because we are using `MozPromise`, our callback will be invoked
+   * as a separate task, so there is a small potential for races in the event
+   * code if things are still holding onto the ServiceWorker binding and using
+   * `postMessage()` or other mechanisms to schedule new events on it, which
+   * would make it non-idle. However, this is a race inherent in the spec which
+   * does not deal with the reality of multiple threads in "Try Clear
+   * Registration".
+   */
+  GetActive()->WorkerPrivate()->GetIdlePromise()->Then(
+      GetCurrentSerialEventTarget(), __func__,
+      [self = RefPtr<ServiceWorkerRegistrationInfo>(this)](
+          const GenericPromise::ResolveOrRejectValue& aResult) {
+        MOZ_ASSERT(aResult.IsResolve());
+        // This registration was already unregistered and not controlling
+        // clients when `ClearWhenIdle` was called, so there should be no way
+        // that more clients were acquired.
+        MOZ_ASSERT(!self->IsControllingClients());
+        MOZ_ASSERT(self->IsIdle());
+        self->Clear();
+
+        RefPtr<ServiceWorkerManager> swm = ServiceWorkerManager::GetInstance();
+        if (swm) {
+          swm->RemoveOrphanedRegistration(self);
+        }
+      });
+}
+
+const nsID& ServiceWorkerRegistrationInfo::AgentClusterId() const {
+  return mAgentClusterId;
+}
+
+void ServiceWorkerRegistrationInfo::SetNavigationPreloadEnabled(
+    const bool& aEnabled) {
+  MOZ_ASSERT(NS_IsMainThread());
+  mNavigationPreloadState.enabled() = aEnabled;
+}
+
+void ServiceWorkerRegistrationInfo::SetNavigationPreloadHeader(
+    const nsCString& aHeader) {
+  MOZ_ASSERT(NS_IsMainThread());
+  mNavigationPreloadState.headerValue() = aHeader;
+}
+
+IPCNavigationPreloadState
+ServiceWorkerRegistrationInfo::GetNavigationPreloadState() const {
+  MOZ_ASSERT(NS_IsMainThread());
+  return mNavigationPreloadState;
+}
+
 // static
 uint64_t ServiceWorkerRegistrationInfo::GetNextId() {
   MOZ_ASSERT(NS_IsMainThread());
@@ -654,5 +867,39 @@ uint64_t ServiceWorkerRegistrationInfo::GetNextId() {
   return ++sNextId;
 }
 
-}  // namespace dom
-}  // namespace mozilla
+// static
+uint64_t ServiceWorkerRegistrationInfo::GetNextVersion() {
+  MOZ_ASSERT(NS_IsMainThread());
+  static uint64_t sNextVersion = 0;
+  return ++sNextVersion;
+}
+
+void ServiceWorkerRegistrationInfo::ForEachWorker(
+    void (*aFunc)(RefPtr<ServiceWorkerInfo>&)) {
+  if (mEvaluatingWorker) {
+    aFunc(mEvaluatingWorker);
+  }
+
+  if (mInstallingWorker) {
+    aFunc(mInstallingWorker);
+  }
+
+  if (mWaitingWorker) {
+    aFunc(mWaitingWorker);
+  }
+
+  if (mActiveWorker) {
+    aFunc(mActiveWorker);
+  }
+}
+
+void ServiceWorkerRegistrationInfo::CheckQuotaUsage() {
+  MOZ_ASSERT(NS_IsMainThread());
+
+  RefPtr<ServiceWorkerManager> swm = ServiceWorkerManager::GetInstance();
+  MOZ_ASSERT(swm);
+
+  swm->CheckPrincipalQuotaUsage(mPrincipal, Scope());
+}
+
+}  // namespace mozilla::dom
