@@ -6,13 +6,20 @@
 
 #include "ServiceWorkerUpdateJob.h"
 
+#include "mozilla/Telemetry.h"
 #include "nsIScriptError.h"
 #include "nsIURL.h"
+#include "nsNetUtil.h"
+#include "nsProxyRelease.h"
+#include "ServiceWorkerManager.h"
+#include "ServiceWorkerPrivate.h"
+#include "ServiceWorkerRegistrationInfo.h"
 #include "ServiceWorkerScriptCache.h"
 #include "mozilla/dom/WorkerCommon.h"
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
+
+using serviceWorkerScriptCache::OnFailure;
 
 namespace {
 
@@ -35,103 +42,73 @@ namespace {
  * used.  ScopeStringPrefixMode allows the caller to specify the desired
  * behavior.
  */
-enum ScopeStringPrefixMode {
-  eUseDirectory,
-  eUsePath
-};
-
-nsresult
-GetRequiredScopeStringPrefix(nsIURI* aScriptURI, nsACString& aPrefix,
-                             ScopeStringPrefixMode aPrefixMode)
-{
-  nsresult rv = aScriptURI->GetPrePath(aPrefix);
-  if (NS_WARN_IF(NS_FAILED(rv))) {
-    return rv;
-  }
+enum ScopeStringPrefixMode { eUseDirectory, eUsePath };
 
+nsresult GetRequiredScopeStringPrefix(nsIURI* aScriptURI, nsACString& aPrefix,
+                                      ScopeStringPrefixMode aPrefixMode) {
+  nsresult rv;
   if (aPrefixMode == eUseDirectory) {
     nsCOMPtr<nsIURL> scriptURL(do_QueryInterface(aScriptURI));
     if (NS_WARN_IF(!scriptURL)) {
       return NS_ERROR_FAILURE;
     }
 
-    nsAutoCString dir;
-    rv = scriptURL->GetDirectory(dir);
+    rv = scriptURL->GetDirectory(aPrefix);
     if (NS_WARN_IF(NS_FAILED(rv))) {
       return rv;
     }
-
-    aPrefix.Append(dir);
   } else if (aPrefixMode == eUsePath) {
-    nsAutoCString path;
-    rv = aScriptURI->GetPathQueryRef(path);
+    rv = aScriptURI->GetPathQueryRef(aPrefix);
     if (NS_WARN_IF(NS_FAILED(rv))) {
       return rv;
     }
-
-    aPrefix.Append(path);
   } else {
     MOZ_ASSERT_UNREACHABLE("Invalid value for aPrefixMode");
   }
   return NS_OK;
 }
 
-} // anonymous namespace
+}  // anonymous namespace
 
-class ServiceWorkerUpdateJob::CompareCallback final : public serviceWorkerScriptCache::CompareCallback
-{
+class ServiceWorkerUpdateJob::CompareCallback final
+    : public serviceWorkerScriptCache::CompareCallback {
   RefPtr<ServiceWorkerUpdateJob> mJob;
 
-  ~CompareCallback()
-  {
-  }
+  ~CompareCallback() = default;
 
-public:
-  explicit CompareCallback(ServiceWorkerUpdateJob* aJob)
-    : mJob(aJob)
-  {
+ public:
+  explicit CompareCallback(ServiceWorkerUpdateJob* aJob) : mJob(aJob) {
     MOZ_ASSERT(mJob);
   }
 
-  virtual void
-  ComparisonResult(nsresult aStatus,
-                   bool aInCacheAndEqual,
-                   const nsAString& aNewCacheName,
-                   const nsACString& aMaxScope,
-                   nsLoadFlags aLoadFlags) override
-  {
-    mJob->ComparisonResult(aStatus,
-                           aInCacheAndEqual,
-                           aNewCacheName,
-                           aMaxScope,
-                           aLoadFlags);
+  virtual void ComparisonResult(nsresult aStatus, bool aInCacheAndEqual,
+                                OnFailure aOnFailure,
+                                const nsAString& aNewCacheName,
+                                const nsACString& aMaxScope,
+                                nsLoadFlags aLoadFlags) override {
+    mJob->ComparisonResult(aStatus, aInCacheAndEqual, aOnFailure, aNewCacheName,
+                           aMaxScope, aLoadFlags);
   }
 
   NS_INLINE_DECL_REFCOUNTING(ServiceWorkerUpdateJob::CompareCallback, override)
 };
 
-class ServiceWorkerUpdateJob::ContinueUpdateRunnable final : public LifeCycleEventCallback
-{
+class ServiceWorkerUpdateJob::ContinueUpdateRunnable final
+    : public LifeCycleEventCallback {
   nsMainThreadPtrHandle<ServiceWorkerUpdateJob> mJob;
   bool mSuccess;
 
-public:
-  explicit ContinueUpdateRunnable(const nsMainThreadPtrHandle<ServiceWorkerUpdateJob>& aJob)
-    : mJob(aJob)
-    , mSuccess(false)
-  {
+ public:
+  explicit ContinueUpdateRunnable(
+      const nsMainThreadPtrHandle<ServiceWorkerUpdateJob>& aJob)
+      : mJob(aJob), mSuccess(false) {
     MOZ_ASSERT(NS_IsMainThread());
   }
 
-  void
-  SetResult(bool aResult) override
-  {
-    mSuccess = aResult;
-  }
+  void SetResult(bool aResult) override { mSuccess = aResult; }
 
   NS_IMETHOD
-  Run() override
-  {
+  Run() override {
     MOZ_ASSERT(NS_IsMainThread());
     mJob->ContinueUpdateAfterScriptEval(mSuccess);
     mJob = nullptr;
@@ -139,28 +116,22 @@ public:
   }
 };
 
-class ServiceWorkerUpdateJob::ContinueInstallRunnable final : public LifeCycleEventCallback
-{
+class ServiceWorkerUpdateJob::ContinueInstallRunnable final
+    : public LifeCycleEventCallback {
   nsMainThreadPtrHandle<ServiceWorkerUpdateJob> mJob;
   bool mSuccess;
 
-public:
-  explicit ContinueInstallRunnable(const nsMainThreadPtrHandle<ServiceWorkerUpdateJob>& aJob)
-    : mJob(aJob)
-    , mSuccess(false)
-  {
+ public:
+  explicit ContinueInstallRunnable(
+      const nsMainThreadPtrHandle<ServiceWorkerUpdateJob>& aJob)
+      : mJob(aJob), mSuccess(false) {
     MOZ_ASSERT(NS_IsMainThread());
   }
 
-  void
-  SetResult(bool aResult) override
-  {
-    mSuccess = aResult;
-  }
+  void SetResult(bool aResult) override { mSuccess = aResult; }
 
   NS_IMETHOD
-  Run() override
-  {
+  Run() override {
     MOZ_ASSERT(NS_IsMainThread());
     mJob->ContinueAfterInstallEvent(mSuccess);
     mJob = nullptr;
@@ -169,63 +140,64 @@ public:
 };
 
 ServiceWorkerUpdateJob::ServiceWorkerUpdateJob(
-    nsIPrincipal* aPrincipal,
-    const nsACString& aScope,
-    const nsACString& aScriptSpec,
-    nsILoadGroup* aLoadGroup,
+    nsIPrincipal* aPrincipal, const nsACString& aScope, nsCString aScriptSpec,
     ServiceWorkerUpdateViaCache aUpdateViaCache)
-  : ServiceWorkerJob(Type::Update, aPrincipal, aScope, aScriptSpec)
-  , mLoadGroup(aLoadGroup)
-  , mUpdateViaCache(aUpdateViaCache)
-{
-}
+    : ServiceWorkerUpdateJob(Type::Update, aPrincipal, aScope,
+                             std::move(aScriptSpec), aUpdateViaCache) {}
 
 already_AddRefed<ServiceWorkerRegistrationInfo>
-ServiceWorkerUpdateJob::GetRegistration() const
-{
+ServiceWorkerUpdateJob::GetRegistration() const {
   MOZ_ASSERT(NS_IsMainThread());
   RefPtr<ServiceWorkerRegistrationInfo> ref = mRegistration;
   return ref.forget();
 }
 
 ServiceWorkerUpdateJob::ServiceWorkerUpdateJob(
-    Type aType,
-    nsIPrincipal* aPrincipal,
-    const nsACString& aScope,
-    const nsACString& aScriptSpec,
-    nsILoadGroup* aLoadGroup,
-    ServiceWorkerUpdateViaCache aUpdateViaCache)
-  : ServiceWorkerJob(aType, aPrincipal, aScope, aScriptSpec)
-  , mLoadGroup(aLoadGroup)
-  , mUpdateViaCache(aUpdateViaCache)
-{
-}
+    Type aType, nsIPrincipal* aPrincipal, const nsACString& aScope,
+    nsCString aScriptSpec, ServiceWorkerUpdateViaCache aUpdateViaCache)
+    : ServiceWorkerJob(aType, aPrincipal, aScope, std::move(aScriptSpec)),
+      mUpdateViaCache(aUpdateViaCache),
+      mOnFailure(serviceWorkerScriptCache::OnFailure::DoNothing) {}
 
-ServiceWorkerUpdateJob::~ServiceWorkerUpdateJob()
-{
-}
+ServiceWorkerUpdateJob::~ServiceWorkerUpdateJob() = default;
 
-void
-ServiceWorkerUpdateJob::FailUpdateJob(ErrorResult& aRv)
-{
+void ServiceWorkerUpdateJob::FailUpdateJob(ErrorResult& aRv) {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(aRv.Failed());
 
   // Cleanup after a failed installation.  This essentially implements
-  // step 12 of the Install algorithm.
+  // step 13 of the Install algorithm.
   //
-  //  https://slightlyoff.github.io/ServiceWorker/spec/service_worker/index.html#installation-algorithm
+  //  https://w3c.github.io/ServiceWorker/#installation-algorithm
   //
   // The spec currently only runs this after an install event fails,
   // but we must handle many more internal errors.  So we check for
   // cleanup on every non-successful exit.
   if (mRegistration) {
-    mRegistration->ClearEvaluating();
-    mRegistration->ClearInstalling();
+    // Some kinds of failures indicate there is something broken in the
+    // currently installed registration.  In these cases we want to fully
+    // unregister.
+    if (mOnFailure == OnFailure::Uninstall) {
+      mRegistration->ClearAsCorrupt();
+    }
+
+    // Otherwise just clear the workers we may have created as part of the
+    // update process.
+    else {
+      mRegistration->ClearEvaluating();
+      mRegistration->ClearInstalling();
+    }
 
     RefPtr<ServiceWorkerManager> swm = ServiceWorkerManager::GetInstance();
     if (swm) {
       swm->MaybeRemoveRegistration(mRegistration);
+
+      // Also clear the registration on disk if we are forcing uninstall
+      // due to a particularly bad failure.
+      if (mOnFailure == OnFailure::Uninstall) {
+        swm->MaybeSendUnregister(mRegistration->Principal(),
+                                 mRegistration->Scope());
+      }
     }
   }
 
@@ -234,16 +206,12 @@ ServiceWorkerUpdateJob::FailUpdateJob(ErrorResult& aRv)
   Finish(aRv);
 }
 
-void
-ServiceWorkerUpdateJob::FailUpdateJob(nsresult aRv)
-{
+void ServiceWorkerUpdateJob::FailUpdateJob(nsresult aRv) {
   ErrorResult rv(aRv);
   FailUpdateJob(rv);
 }
 
-void
-ServiceWorkerUpdateJob::AsyncExecute()
-{
+void ServiceWorkerUpdateJob::AsyncExecute() {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(GetType() == Type::Update);
 
@@ -253,29 +221,32 @@ ServiceWorkerUpdateJob::AsyncExecute()
     return;
   }
 
-  // Begin step 1 of the Update algorithm.
+  // Invoke Update algorithm:
+  // https://w3c.github.io/ServiceWorker/#update-algorithm
   //
-  //  https://slightlyoff.github.io/ServiceWorker/spec/service_worker/index.html#update-algorithm
-
+  // "Let registration be the result of running the Get Registration algorithm
+  // passing job’s scope url as the argument."
   RefPtr<ServiceWorkerRegistrationInfo> registration =
-    swm->GetRegistration(mPrincipal, mScope);
+      swm->GetRegistration(mPrincipal, mScope);
 
-  if (!registration || registration->IsPendingUninstall()) {
+  if (!registration) {
     ErrorResult rv;
-    rv.ThrowTypeError<MSG_SW_UPDATE_BAD_REGISTRATION>(NS_ConvertUTF8toUTF16(mScope),
-                                                      NS_LITERAL_STRING("uninstalled"));
+    rv.ThrowTypeError<MSG_SW_UPDATE_BAD_REGISTRATION>(mScope, "uninstalled");
     FailUpdateJob(rv);
     return;
   }
 
-  // If a Register job with a new script executed ahead of us in the job queue,
-  // then our update for the old script no longer makes sense.  Simply abort
-  // in this case.
+  // "Let newestWorker be the result of running Get Newest Worker algorithm
+  // passing registration as the argument."
   RefPtr<ServiceWorkerInfo> newest = registration->Newest();
-  if (newest && !mScriptSpec.Equals(newest->ScriptSpec())) {
+
+  // "If job’s job type is update, and newestWorker is not null and its script
+  // url does not equal job’s script url, then:
+  //   1. Invoke Reject Job Promise with job and TypeError.
+  //   2. Invoke Finish Job with job and abort these steps."
+  if (newest && !newest->ScriptSpec().Equals(mScriptSpec)) {
     ErrorResult rv;
-    rv.ThrowTypeError<MSG_SW_UPDATE_BAD_REGISTRATION>(NS_ConvertUTF8toUTF16(mScope),
-                                                      NS_LITERAL_STRING("changed"));
+    rv.ThrowTypeError<MSG_SW_UPDATE_BAD_REGISTRATION>(mScope, "changed");
     FailUpdateJob(rv);
     return;
   }
@@ -284,9 +255,8 @@ ServiceWorkerUpdateJob::AsyncExecute()
   Update();
 }
 
-void
-ServiceWorkerUpdateJob::SetRegistration(ServiceWorkerRegistrationInfo* aRegistration)
-{
+void ServiceWorkerUpdateJob::SetRegistration(
+    ServiceWorkerRegistrationInfo* aRegistration) {
   MOZ_ASSERT(NS_IsMainThread());
 
   MOZ_ASSERT(!mRegistration);
@@ -294,9 +264,7 @@ ServiceWorkerUpdateJob::SetRegistration(ServiceWorkerRegistrationInfo* aRegistra
   mRegistration = aRegistration;
 }
 
-void
-ServiceWorkerUpdateJob::Update()
-{
+void ServiceWorkerUpdateJob::Update() {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(!Canceled());
 
@@ -318,31 +286,28 @@ ServiceWorkerUpdateJob::Update()
 
   RefPtr<CompareCallback> callback = new CompareCallback(this);
 
-  nsresult rv =
-    serviceWorkerScriptCache::Compare(mRegistration, mPrincipal, cacheName,
-                                      NS_ConvertUTF8toUTF16(mScriptSpec),
-                                      callback, mLoadGroup);
+  nsresult rv = serviceWorkerScriptCache::Compare(
+      mRegistration, mPrincipal, cacheName, mScriptSpec, callback);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     FailUpdateJob(rv);
     return;
   }
 }
 
-ServiceWorkerUpdateViaCache
-ServiceWorkerUpdateJob::GetUpdateViaCache() const
-{
+ServiceWorkerUpdateViaCache ServiceWorkerUpdateJob::GetUpdateViaCache() const {
   return mUpdateViaCache;
 }
 
-void
-ServiceWorkerUpdateJob::ComparisonResult(nsresult aStatus,
-                                         bool aInCacheAndEqual,
-                                         const nsAString& aNewCacheName,
-                                         const nsACString& aMaxScope,
-                                         nsLoadFlags aLoadFlags)
-{
+void ServiceWorkerUpdateJob::ComparisonResult(nsresult aStatus,
+                                              bool aInCacheAndEqual,
+                                              OnFailure aOnFailure,
+                                              const nsAString& aNewCacheName,
+                                              const nsACString& aMaxScope,
+                                              nsLoadFlags aLoadFlags) {
   MOZ_ASSERT(NS_IsMainThread());
 
+  mOnFailure = aOnFailure;
+
   RefPtr<ServiceWorkerManager> swm = ServiceWorkerManager::GetInstance();
   if (NS_WARN_IF(Canceled() || !swm)) {
     FailUpdateJob(NS_ERROR_DOM_ABORT_ERR);
@@ -374,8 +339,7 @@ ServiceWorkerUpdateJob::ComparisonResult(nsresult aStatus,
 
   nsCOMPtr<nsIURI> maxScopeURI;
   if (!aMaxScope.IsEmpty()) {
-    rv = NS_NewURI(getter_AddRefs(maxScopeURI), aMaxScope,
-                   nullptr, scriptURI);
+    rv = NS_NewURI(getter_AddRefs(maxScopeURI), aMaxScope, nullptr, scriptURI);
     if (NS_WARN_IF(NS_FAILED(rv))) {
       FailUpdateJob(NS_ERROR_DOM_SECURITY_ERR);
       return;
@@ -399,20 +363,31 @@ ServiceWorkerUpdateJob::ComparisonResult(nsresult aStatus,
     }
   }
 
-  if (!StringBeginsWith(mRegistration->Scope(), maxPrefix)) {
+  nsCOMPtr<nsIURI> scopeURI;
+  rv = NS_NewURI(getter_AddRefs(scopeURI), mRegistration->Scope(), nullptr,
+                 scriptURI);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    FailUpdateJob(NS_ERROR_FAILURE);
+    return;
+  }
+
+  nsAutoCString scopeString;
+  rv = scopeURI->GetPathQueryRef(scopeString);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    FailUpdateJob(NS_ERROR_FAILURE);
+    return;
+  }
+
+  if (!StringBeginsWith(scopeString, maxPrefix)) {
     nsAutoString message;
     NS_ConvertUTF8toUTF16 reportScope(mRegistration->Scope());
     NS_ConvertUTF8toUTF16 reportMaxPrefix(maxPrefix);
-    const char16_t* params[] = { reportScope.get(), reportMaxPrefix.get() };
 
-    rv = nsContentUtils::FormatLocalizedString(nsContentUtils::eDOM_PROPERTIES,
-                                               "ServiceWorkerScopePathMismatch",
-                                               params, message);
+    rv = nsContentUtils::FormatLocalizedString(
+        message, nsContentUtils::eDOM_PROPERTIES,
+        "ServiceWorkerScopePathMismatch", reportScope, reportMaxPrefix);
     NS_WARNING_ASSERTION(NS_SUCCEEDED(rv), "Failed to format localized string");
-    swm->ReportToAllClients(mScope,
-                            message,
-                            EmptyString(),
-                            EmptyString(), 0, 0,
+    swm->ReportToAllClients(mScope, message, u""_ns, u""_ns, 0, 0,
                             nsIScriptError::errorFlag);
     FailUpdateJob(NS_ERROR_DOM_SECURITY_ERR);
     return;
@@ -433,18 +408,23 @@ ServiceWorkerUpdateJob::ComparisonResult(nsresult aStatus,
     flags |= nsIRequest::VALIDATE_ALWAYS;
   }
 
-  RefPtr<ServiceWorkerInfo> sw =
-    new ServiceWorkerInfo(mRegistration->Principal(),
-                          mRegistration->Scope(),
-                          mScriptSpec,
-                          aNewCacheName,
-                          flags);
+  RefPtr<ServiceWorkerInfo> sw = new ServiceWorkerInfo(
+      mRegistration->Principal(), mRegistration->Scope(), mRegistration->Id(),
+      mRegistration->Version(), mScriptSpec, aNewCacheName, flags);
+
+  // If the registration is corrupt enough to force an uninstall if the
+  // upgrade fails, then we want to make sure the upgrade takes effect
+  // if it succeeds.  Therefore force the skip-waiting flag on to replace
+  // the broken worker after install.
+  if (aOnFailure == OnFailure::Uninstall) {
+    sw->SetSkipWaitingFlag();
+  }
 
   mRegistration->SetEvaluating(sw);
 
   nsMainThreadPtrHandle<ServiceWorkerUpdateJob> handle(
       new nsMainThreadPtrHolder<ServiceWorkerUpdateJob>(
-        "ServiceWorkerUpdateJob", this));
+          "ServiceWorkerUpdateJob", this));
   RefPtr<LifeCycleEventCallback> callback = new ContinueUpdateRunnable(handle);
 
   ServiceWorkerPrivate* workerPrivate = sw->WorkerPrivate();
@@ -457,9 +437,8 @@ ServiceWorkerUpdateJob::ComparisonResult(nsresult aStatus,
   }
 }
 
-void
-ServiceWorkerUpdateJob::ContinueUpdateAfterScriptEval(bool aScriptEvaluationResult)
-{
+void ServiceWorkerUpdateJob::ContinueUpdateAfterScriptEval(
+    bool aScriptEvaluationResult) {
   MOZ_ASSERT(NS_IsMainThread());
 
   RefPtr<ServiceWorkerManager> swm = ServiceWorkerManager::GetInstance();
@@ -473,23 +452,18 @@ ServiceWorkerUpdateJob::ContinueUpdateAfterScriptEval(bool aScriptEvaluationResu
 
   if (NS_WARN_IF(!aScriptEvaluationResult)) {
     ErrorResult error;
-
-    NS_ConvertUTF8toUTF16 scriptSpec(mScriptSpec);
-    NS_ConvertUTF8toUTF16 scope(mRegistration->Scope());
-    error.ThrowTypeError<MSG_SW_SCRIPT_THREW>(scriptSpec, scope);
+    error.ThrowTypeError<MSG_SW_SCRIPT_THREW>(mScriptSpec,
+                                              mRegistration->Scope());
     FailUpdateJob(error);
     return;
   }
 
-  Install(swm);
+  Install();
 }
 
-void
-ServiceWorkerUpdateJob::Install(ServiceWorkerManager* aSWM)
-{
+void ServiceWorkerUpdateJob::Install() {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_DIAGNOSTIC_ASSERT(!Canceled());
-  MOZ_DIAGNOSTIC_ASSERT(aSWM);
 
   MOZ_ASSERT(!mRegistration->GetInstalling());
 
@@ -502,45 +476,26 @@ ServiceWorkerUpdateJob::Install(ServiceWorkerManager* aSWM)
   // Step 6 of the Install algorithm resolving the job promise.
   InvokeResultCallbacks(NS_OK);
 
-  // The job promise cannot be rejected after this point, but the job can
-  // still fail; e.g. if the install event handler throws, etc.
-
-  // fire the updatefound event
-  nsCOMPtr<nsIRunnable> upr =
-    NewRunnableMethod<RefPtr<ServiceWorkerRegistrationInfo>>(
-      "dom::ServiceWorkerManager::"
-      "FireUpdateFoundOnServiceWorkerRegistrations",
-      aSWM,
-      &ServiceWorkerManager::FireUpdateFoundOnServiceWorkerRegistrations,
-      mRegistration);
-  NS_DispatchToMainThread(upr);
-
-  // Call ContinueAfterInstallEvent(false) on main thread if the SW
-  // script fails to load.
-  nsCOMPtr<nsIRunnable> failRunnable = NewRunnableMethod<bool>(
-    "dom::ServiceWorkerUpdateJob::ContinueAfterInstallEvent",
-    this,
-    &ServiceWorkerUpdateJob::ContinueAfterInstallEvent,
-    false);
+  // Queue a task to fire an event named updatefound at all the
+  // ServiceWorkerRegistration.
+  mRegistration->FireUpdateFound();
 
   nsMainThreadPtrHandle<ServiceWorkerUpdateJob> handle(
-    new nsMainThreadPtrHolder<ServiceWorkerUpdateJob>(
-      "ServiceWorkerUpdateJob", this));
+      new nsMainThreadPtrHolder<ServiceWorkerUpdateJob>(
+          "ServiceWorkerUpdateJob", this));
   RefPtr<LifeCycleEventCallback> callback = new ContinueInstallRunnable(handle);
 
   // Send the install event to the worker thread
   ServiceWorkerPrivate* workerPrivate =
-    mRegistration->GetInstalling()->WorkerPrivate();
-  nsresult rv = workerPrivate->SendLifeCycleEvent(NS_LITERAL_STRING("install"),
-                                                  callback, failRunnable);
+      mRegistration->GetInstalling()->WorkerPrivate();
+  nsresult rv = workerPrivate->SendLifeCycleEvent(u"install"_ns, callback);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     ContinueAfterInstallEvent(false /* aSuccess */);
   }
 }
 
-void
-ServiceWorkerUpdateJob::ContinueAfterInstallEvent(bool aInstallEventSuccess)
-{
+void ServiceWorkerUpdateJob::ContinueAfterInstallEvent(
+    bool aInstallEventSuccess) {
   if (Canceled()) {
     return FailUpdateJob(NS_ERROR_DOM_ABORT_ERR);
   }
@@ -562,20 +517,24 @@ ServiceWorkerUpdateJob::ContinueAfterInstallEvent(bool aInstallEventSuccess)
     return;
   }
 
-  MOZ_DIAGNOSTIC_ASSERT(mRegistration->GetInstalling());
+  // Abort the update Job if the installWorker is null (e.g. when an extension
+  // is shutting down and all its workers have been terminated).
+  if (!mRegistration->GetInstalling()) {
+    return FailUpdateJob(NS_ERROR_DOM_ABORT_ERR);
+  }
+
   mRegistration->TransitionInstallingToWaiting();
 
   Finish(NS_OK);
 
-  // Step 20 calls for explicitly waiting for queued event tasks to fire.  Instead,
-  // we simply queue a runnable to execute Activate.  This ensures the events are
-  // flushed from the queue before proceeding.
+  // Step 20 calls for explicitly waiting for queued event tasks to fire.
+  // Instead, we simply queue a runnable to execute Activate.  This ensures the
+  // events are flushed from the queue before proceeding.
 
-  // Step 22 of the Install algorithm.  Activate is executed after the completion
-  // of this job.  The controlling client and skipWaiting checks are performed
-  // in TryToActivate().
+  // Step 22 of the Install algorithm.  Activate is executed after the
+  // completion of this job.  The controlling client and skipWaiting checks are
+  // performed in TryToActivate().
   mRegistration->TryToActivateAsync();
 }
 
-} // namespace dom
-} // namespace mozilla
+}  // namespace mozilla::dom
