@@ -4,54 +4,55 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "DOMSVGAnimatedNumberList.h"
 #include "mozilla/dom/SVGFEColorMatrixElement.h"
+
+#include "DOMSVGAnimatedNumberList.h"
 #include "mozilla/dom/SVGFEColorMatrixElementBinding.h"
-#include "nsSVGUtils.h"
+#include "mozilla/dom/Document.h"
+#include "mozilla/dom/BindContext.h"
 
 #define NUM_ENTRIES_IN_4x5_MATRIX 20
 
-NS_IMPL_NS_NEW_NAMESPACED_SVG_ELEMENT(FEColorMatrix)
+NS_IMPL_NS_NEW_SVG_ELEMENT(FEColorMatrix)
 
 using namespace mozilla::gfx;
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 JSObject* SVGFEColorMatrixElement::WrapNode(JSContext* aCx,
                                             JS::Handle<JSObject*> aGivenProto) {
-  return SVGFEColorMatrixElementBinding::Wrap(aCx, this, aGivenProto);
+  return SVGFEColorMatrixElement_Binding::Wrap(aCx, this, aGivenProto);
 }
 
-nsSVGEnumMapping SVGFEColorMatrixElement::sTypeMap[] = {
-    {&nsGkAtoms::matrix, SVG_FECOLORMATRIX_TYPE_MATRIX},
-    {&nsGkAtoms::saturate, SVG_FECOLORMATRIX_TYPE_SATURATE},
-    {&nsGkAtoms::hueRotate, SVG_FECOLORMATRIX_TYPE_HUE_ROTATE},
-    {&nsGkAtoms::luminanceToAlpha, SVG_FECOLORMATRIX_TYPE_LUMINANCE_TO_ALPHA},
+SVGEnumMapping SVGFEColorMatrixElement::sTypeMap[] = {
+    {nsGkAtoms::matrix, SVG_FECOLORMATRIX_TYPE_MATRIX},
+    {nsGkAtoms::saturate, SVG_FECOLORMATRIX_TYPE_SATURATE},
+    {nsGkAtoms::hueRotate, SVG_FECOLORMATRIX_TYPE_HUE_ROTATE},
+    {nsGkAtoms::luminanceToAlpha, SVG_FECOLORMATRIX_TYPE_LUMINANCE_TO_ALPHA},
     {nullptr, 0}};
 
-nsSVGElement::EnumInfo SVGFEColorMatrixElement::sEnumInfo[1] = {
-    {&nsGkAtoms::type, sTypeMap, SVG_FECOLORMATRIX_TYPE_MATRIX}};
+SVGElement::EnumInfo SVGFEColorMatrixElement::sEnumInfo[1] = {
+    {nsGkAtoms::type, sTypeMap, SVG_FECOLORMATRIX_TYPE_MATRIX}};
 
-nsSVGElement::StringInfo SVGFEColorMatrixElement::sStringInfo[2] = {
-    {&nsGkAtoms::result, kNameSpaceID_None, true},
-    {&nsGkAtoms::in, kNameSpaceID_None, true}};
+SVGElement::StringInfo SVGFEColorMatrixElement::sStringInfo[2] = {
+    {nsGkAtoms::result, kNameSpaceID_None, true},
+    {nsGkAtoms::in, kNameSpaceID_None, true}};
 
-nsSVGElement::NumberListInfo SVGFEColorMatrixElement::sNumberListInfo[1] = {
-    {&nsGkAtoms::values}};
+SVGElement::NumberListInfo SVGFEColorMatrixElement::sNumberListInfo[1] = {
+    {nsGkAtoms::values}};
 
 //----------------------------------------------------------------------
-// nsIDOMNode methods
+// nsINode methods
 
 NS_IMPL_ELEMENT_CLONE_WITH_INIT(SVGFEColorMatrixElement)
 
 //----------------------------------------------------------------------
 
-already_AddRefed<SVGAnimatedString> SVGFEColorMatrixElement::In1() {
+already_AddRefed<DOMSVGAnimatedString> SVGFEColorMatrixElement::In1() {
   return mStringAttributes[IN1].ToDOMAnimatedString(this);
 }
 
-already_AddRefed<SVGAnimatedEnumeration> SVGFEColorMatrixElement::Type() {
+already_AddRefed<DOMSVGAnimatedEnumeration> SVGFEColorMatrixElement::Type() {
   return mEnumAttributes[TYPE].ToDOMAnimatedEnum(this);
 }
 
@@ -61,24 +62,23 @@ already_AddRefed<DOMSVGAnimatedNumberList> SVGFEColorMatrixElement::Values() {
 }
 
 void SVGFEColorMatrixElement::GetSourceImageNames(
-    nsTArray<nsSVGStringInfo>& aSources) {
-  aSources.AppendElement(nsSVGStringInfo(&mStringAttributes[IN1], this));
+    nsTArray<SVGStringInfo>& aSources) {
+  aSources.AppendElement(SVGStringInfo(&mStringAttributes[IN1], this));
 }
 
 FilterPrimitiveDescription SVGFEColorMatrixElement::GetPrimitiveDescription(
-    nsSVGFilterInstance* aInstance, const IntRect& aFilterSubregion,
+    SVGFilterInstance* aInstance, const IntRect& aFilterSubregion,
     const nsTArray<bool>& aInputsAreTainted,
     nsTArray<RefPtr<SourceSurface>>& aInputImages) {
   uint32_t type = mEnumAttributes[TYPE].GetAnimValue();
   const SVGNumberList& values = mNumberListAttributes[VALUES].GetAnimValue();
 
-  FilterPrimitiveDescription descr(PrimitiveType::ColorMatrix);
+  ColorMatrixAttributes atts;
   if (!mNumberListAttributes[VALUES].IsExplicitlySet() &&
       (type == SVG_FECOLORMATRIX_TYPE_MATRIX ||
        type == SVG_FECOLORMATRIX_TYPE_SATURATE ||
        type == SVG_FECOLORMATRIX_TYPE_HUE_ROTATE)) {
-    descr.Attributes().Set(eColorMatrixType,
-                           (uint32_t)SVG_FECOLORMATRIX_TYPE_MATRIX);
+    atts.mType = (uint32_t)SVG_FECOLORMATRIX_TYPE_MATRIX;
     static const float identityMatrix[] = {
         // clang-format off
         1, 0, 0, 0, 0,
@@ -87,16 +87,15 @@ FilterPrimitiveDescription SVGFEColorMatrixElement::GetPrimitiveDescription(
         0, 0, 0, 1, 0
         // clang-format on
     };
-    descr.Attributes().Set(eColorMatrixValues, identityMatrix, 20);
+    atts.mValues.AppendElements(identityMatrix, 20);
   } else {
-    descr.Attributes().Set(eColorMatrixType, type);
+    atts.mType = type;
     if (values.Length()) {
-      descr.Attributes().Set(eColorMatrixValues, &values[0], values.Length());
-    } else {
-      descr.Attributes().Set(eColorMatrixValues, nullptr, 0);
+      atts.mValues.AppendElements(&values[0], values.Length());
     }
   }
-  return descr;
+
+  return FilterPrimitiveDescription(AsVariant(std::move(atts)));
 }
 
 bool SVGFEColorMatrixElement::AttributeAffectsRendering(
@@ -108,23 +107,31 @@ bool SVGFEColorMatrixElement::AttributeAffectsRendering(
            aAttribute == nsGkAtoms::values));
 }
 
-//----------------------------------------------------------------------
-// nsSVGElement methods
+nsresult SVGFEColorMatrixElement::BindToTree(BindContext& aCtx,
+                                             nsINode& aParent) {
+  if (aCtx.InComposedDoc()) {
+    aCtx.OwnerDoc().SetUseCounter(eUseCounter_custom_feColorMatrix);
+  }
 
-nsSVGElement::EnumAttributesInfo SVGFEColorMatrixElement::GetEnumInfo() {
+  return SVGFEColorMatrixElementBase::BindToTree(aCtx, aParent);
+}
+
+//----------------------------------------------------------------------
+// SVGElement methods
+
+SVGElement::EnumAttributesInfo SVGFEColorMatrixElement::GetEnumInfo() {
   return EnumAttributesInfo(mEnumAttributes, sEnumInfo, ArrayLength(sEnumInfo));
 }
 
-nsSVGElement::StringAttributesInfo SVGFEColorMatrixElement::GetStringInfo() {
+SVGElement::StringAttributesInfo SVGFEColorMatrixElement::GetStringInfo() {
   return StringAttributesInfo(mStringAttributes, sStringInfo,
                               ArrayLength(sStringInfo));
 }
 
-nsSVGElement::NumberListAttributesInfo
+SVGElement::NumberListAttributesInfo
 SVGFEColorMatrixElement::GetNumberListInfo() {
   return NumberListAttributesInfo(mNumberListAttributes, sNumberListInfo,
                                   ArrayLength(sNumberListInfo));
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
