@@ -4,23 +4,14 @@ Execution Tests for the i32 arithmetic unary expression operations
 
 import { makeTestGroup } from '../../../../../common/framework/test_group.js';
 import { GPUTest } from '../../../../gpu_test.js';
-import { i32, TypeI32 } from '../../../../util/conversion.js';
-import { fullI32Range } from '../../../../util/math.js';
-import { makeCaseCache } from '../case_cache.js';
+import { Type } from '../../../../util/conversion.js';
 import { allInputSources, run } from '../expression.js';
 
+import { d } from './i32_arithmetic.cache.js';
 import { unary } from './unary.js';
 
 export const g = makeTestGroup(GPUTest);
 
-export const d = makeCaseCache('unary/i32_arithmetic', {
-  negation: () => {
-    return fullI32Range().map(e => {
-      return { input: i32(e), expected: i32(-e) };
-    });
-  },
-});
-
 g.test('negation')
   .specURL('https://www.w3.org/TR/WGSL/#floating-point-evaluation')
   .desc(
@@ -33,5 +24,5 @@ Expression: -x
   )
   .fn(async t => {
     const cases = await d.get('negation');
-    await run(t, unary('-'), [TypeI32], TypeI32, t.params, cases);
+    await run(t, unary('-'), [Type.i32], Type.i32, t.params, cases);
   });
