@@ -1,7 +1,7 @@
 export const description = `Test that variables in the shader are zero initialized`;
 
 import { makeTestGroup } from '../../../common/framework/test_group.js';
-import { unreachable } from '../../../common/util/util.js';
+import { iterRange, unreachable } from '../../../common/util/util.js';
 import { GPUTest } from '../../gpu_test.js';
 import {
   ScalarType,
@@ -15,7 +15,7 @@ import {
 
 type ShaderTypeInfo =
   | { type: 'container'; containerType: 'array'; elementType: ShaderTypeInfo; length: number }
-  | { type: 'container'; containerType: 'struct'; members: ShaderTypeInfo[] }
+  | { type: 'container'; containerType: 'struct'; members: readonly ShaderTypeInfo[] }
   | {
       type: 'container';
       containerType: keyof typeof kVectorContainerTypeInfo | keyof typeof kMatrixContainerTypeInfo;
@@ -50,17 +50,15 @@ function prettyPrint(t: ShaderTypeInfo): string {
 export const g = makeTestGroup(GPUTest);
 g.test('compute,zero_init')
   .desc(
-    `Test that uninitialized variables in workgroup, private, and function storage classes are initialized to zero.
-
-    TODO: Run a shader before the test to attempt to fill memory with garbage`
+    `Test that uninitialized variables in workgroup, private, and function storage classes are initialized to zero.`
   )
   .params(u =>
     u
       // Only workgroup, function, and private variables can be declared without data bound to them.
       // The implementation's shader translator should ensure these values are initialized.
-      .combine('storageClass', ['workgroup', 'private', 'function'] as const)
-      .expand('workgroupSize', ({ storageClass }) => {
-        switch (storageClass) {
+      .combine('addressSpace', ['workgroup', 'private', 'function'] as const)
+      .expand('workgroupSize', ({ addressSpace }) => {
+        switch (addressSpace) {
           case 'workgroup':
             return [
               [1, 1, 1],
@@ -109,6 +107,10 @@ g.test('compute,zero_init')
               ? [true, false]
               : [false]) {
               for (const scalarType of supportedScalarTypes({ isAtomic, ...p })) {
+                // Fewer subcases: supportedScalarTypes was expanded to include f16
+                // but that may take too much time. It would require more complex code.
+                if (scalarType === 'f16') continue;
+
                 // Fewer subcases: For nested types, skip atomic u32 and non-atomic i32.
                 if (p._containerDepth > 0) {
                   if (scalarType === 'u32' && isAtomic) continue;
@@ -230,6 +232,14 @@ g.test('compute,zero_init')
   )
   .batch(15)
   .fn(async t => {
+    const { workgroupSize } = t.params;
+    const { maxComputeInvocationsPerWorkgroup } = t.device.limits;
+    const numWorkgroupInvocations = workgroupSize.reduce((a, b) => a * b);
+    t.skipIf(
+      numWorkgroupInvocations > maxComputeInvocationsPerWorkgroup,
+      `workgroupSize: ${workgroupSize} > maxComputeInvocationsPerWorkgroup: ${maxComputeInvocationsPerWorkgroup}`
+    );
+
     let moduleScope = `
       struct Output {
         failed : atomic<u32>
@@ -295,10 +305,10 @@ g.test('compute,zero_init')
       }
     })('TestType', t.params._type);
 
-    switch (t.params.storageClass) {
+    switch (t.params.addressSpace) {
       case 'workgroup':
       case 'private':
-        moduleScope += `\nvar<${t.params.storageClass}> testVar: ${typeDecl};`;
+        moduleScope += `\nvar<${t.params.addressSpace}> testVar: ${typeDecl};`;
         break;
       case 'function':
         functionScope += `\nvar testVar: ${typeDecl};`;
@@ -397,7 +407,99 @@ g.test('compute,zero_init')
       }
     `;
 
-    const pipeline = t.device.createComputePipeline({
+    if (t.params.addressSpace === 'workgroup') {
+      // Populate the maximum amount of workgroup memory with known values to
+      // ensure initialization overrides in another shader.
+      const wg_memory_limits = t.device.limits.maxComputeWorkgroupStorageSize;
+      const wg_x_dim = t.device.limits.maxComputeWorkgroupSizeX;
+
+      const wgsl = `
+      @group(0) @binding(0) var<storage, read> inputs : array<u32>;
+      @group(0) @binding(1) var<storage, read_write> outputs : array<u32>;
+      var<workgroup> wg_mem : array<u32, ${wg_memory_limits} / 4>;
+
+      @compute @workgroup_size(${wg_x_dim})
+      fn fill(@builtin(local_invocation_index) lid : u32) {
+        const num_u32_per_invocation = ${wg_memory_limits} / (4 * ${wg_x_dim});
+
+        for (var i = 0u; i < num_u32_per_invocation; i++) {
+          let idx = num_u32_per_invocation * lid + i;
+          wg_mem[idx] = inputs[idx];
+        }
+        workgroupBarrier();
+        // Copy out to avoid wg_mem being elided.
+        for (var i = 0u; i < num_u32_per_invocation; i++) {
+          let idx = num_u32_per_invocation * lid + i;
+          outputs[idx] = wg_mem[idx];
+        }
+      }
+      `;
+
+      const fillLayout = t.device.createBindGroupLayout({
+        entries: [
+          {
+            binding: 0,
+            visibility: GPUShaderStage.COMPUTE,
+            buffer: { type: 'read-only-storage' },
+          },
+          {
+            binding: 1,
+            visibility: GPUShaderStage.COMPUTE,
+            buffer: { type: 'storage' },
+          },
+        ],
+      });
+
+      const fillPipeline = await t.device.createComputePipelineAsync({
+        layout: t.device.createPipelineLayout({ bindGroupLayouts: [fillLayout] }),
+        label: 'Workgroup Fill Pipeline',
+        compute: {
+          module: t.device.createShaderModule({
+            code: wgsl,
+          }),
+          entryPoint: 'fill',
+        },
+      });
+
+      const inputBuffer = t.makeBufferWithContents(
+        new Uint32Array([...iterRange(wg_memory_limits / 4, _i => 0xdeadbeef)]),
+        GPUBufferUsage.STORAGE | GPUBufferUsage.COPY_DST
+      );
+      t.trackForCleanup(inputBuffer);
+      const outputBuffer = t.device.createBuffer({
+        size: wg_memory_limits,
+        usage: GPUBufferUsage.STORAGE | GPUBufferUsage.COPY_SRC,
+      });
+      t.trackForCleanup(outputBuffer);
+
+      const bg = t.device.createBindGroup({
+        layout: fillPipeline.getBindGroupLayout(0),
+        entries: [
+          {
+            binding: 0,
+            resource: {
+              buffer: inputBuffer,
+            },
+          },
+          {
+            binding: 1,
+            resource: {
+              buffer: outputBuffer,
+            },
+          },
+        ],
+      });
+
+      const e = t.device.createCommandEncoder();
+      const p = e.beginComputePass();
+      p.setPipeline(fillPipeline);
+      p.setBindGroup(0, bg);
+      p.dispatchWorkgroups(1);
+      p.end();
+      t.queue.submit([e.finish()]);
+    }
+
+    const pipeline = await t.device.createComputePipelineAsync({
       layout: 'auto',
       compute: {
         module: t.device.createShaderModule({
