@@ -8,10 +8,22 @@
 #include "WorkerPrivate.h"
 #include "WorkerRunnable.h"
 
+#include "mozilla/Logging.h"
 #include "mozilla/dom/ReferrerInfo.h"
 
 namespace mozilla::dom {
 
+static mozilla::LazyLogModule sWorkerEventTargetLog("WorkerEventTarget");
+
+#ifdef LOG
+#  undef LOG
+#endif
+#ifdef LOGV
+#  undef LOGV
+#endif
+#define LOG(args) MOZ_LOG(sWorkerEventTargetLog, LogLevel::Debug, args);
+#define LOGV(args) MOZ_LOG(sWorkerEventTargetLog, LogLevel::Verbose, args);
+
 namespace {
 
 class WrappedControlRunnable final : public WorkerControlRunnable {
@@ -22,7 +34,7 @@ class WrappedControlRunnable final : public WorkerControlRunnable {
  public:
   WrappedControlRunnable(WorkerPrivate* aWorkerPrivate,
                          nsCOMPtr<nsIRunnable>&& aInner)
-      : WorkerControlRunnable(aWorkerPrivate, WorkerThreadUnchangedBusyCount),
+      : WorkerControlRunnable("WrappedControlRunnable"),
         mInner(std::move(aInner)) {}
 
   virtual bool PreDispatch(WorkerPrivate* aWorkerPrivate) override {
@@ -46,16 +58,14 @@ class WrappedControlRunnable final : public WorkerControlRunnable {
     // If the inner runnable is not cancellable, then just do the normal
     // WorkerControlRunnable thing.  This will end up calling Run().
     if (!cr) {
-      WorkerControlRunnable::Cancel();
-      return NS_OK;
+      return Run();
     }
 
     // Otherwise call the inner runnable's Cancel() and treat this like
     // a WorkerRunnable cancel.  We can't call WorkerControlRunnable::Cancel()
     // in this case since that would result in both Run() and the inner
     // Cancel() being called.
-    Unused << cr->Cancel();
-    return WorkerRunnable::Cancel();
+    return cr->Cancel();
   }
 };
 
@@ -68,10 +78,14 @@ WorkerEventTarget::WorkerEventTarget(WorkerPrivate* aWorkerPrivate,
     : mMutex("WorkerEventTarget"),
       mWorkerPrivate(aWorkerPrivate),
       mBehavior(aBehavior) {
+  LOG(("WorkerEventTarget::WorkerEventTarget [%p] aBehavior: %u", this,
+       (uint8_t)aBehavior));
   MOZ_DIAGNOSTIC_ASSERT(mWorkerPrivate);
 }
 
 void WorkerEventTarget::ForgetWorkerPrivate(WorkerPrivate* aWorkerPrivate) {
+  LOG(("WorkerEventTarget::ForgetWorkerPrivate [%p] aWorkerPrivate: %p", this,
+       aWorkerPrivate));
   MutexAutoLock lock(mMutex);
   MOZ_DIAGNOSTIC_ASSERT(!mWorkerPrivate || mWorkerPrivate == aWorkerPrivate);
   mWorkerPrivate = nullptr;
@@ -79,6 +93,8 @@ void WorkerEventTarget::ForgetWorkerPrivate(WorkerPrivate* aWorkerPrivate) {
 
 NS_IMETHODIMP
 WorkerEventTarget::DispatchFromScript(nsIRunnable* aRunnable, uint32_t aFlags) {
+  LOGV(("WorkerEventTarget::DispatchFromScript [%p] aRunnable: %p", this,
+        aRunnable));
   nsCOMPtr<nsIRunnable> runnable(aRunnable);
   return Dispatch(runnable.forget(), aFlags);
 }
@@ -87,6 +103,8 @@ NS_IMETHODIMP
 WorkerEventTarget::Dispatch(already_AddRefed<nsIRunnable> aRunnable,
                             uint32_t aFlags) {
   nsCOMPtr<nsIRunnable> runnable(aRunnable);
+  LOGV(
+      ("WorkerEventTarget::Dispatch [%p] aRunnable: %p", this, runnable.get()));
 
   MutexAutoLock lock(mMutex);
 
@@ -95,18 +113,31 @@ WorkerEventTarget::Dispatch(already_AddRefed<nsIRunnable> aRunnable,
   }
 
   if (mBehavior == Behavior::Hybrid) {
+    LOGV(("WorkerEventTarget::Dispatch [%p] Dispatch as normal runnable(%p)",
+          this, runnable.get()));
+
     RefPtr<WorkerRunnable> r =
         mWorkerPrivate->MaybeWrapAsWorkerRunnable(runnable.forget());
-    if (r->Dispatch()) {
+    if (r->Dispatch(mWorkerPrivate)) {
       return NS_OK;
     }
-
     runnable = std::move(r);
+    LOGV((
+        "WorkerEventTarget::Dispatch [%p] Dispatch as normal runnable(%p) fail",
+        this, runnable.get()));
   }
 
   RefPtr<WorkerControlRunnable> r =
       new WrappedControlRunnable(mWorkerPrivate, std::move(runnable));
-  if (!r->Dispatch()) {
+  LOGV(
+      ("WorkerEventTarget::Dispatch [%p] Wrapped runnable as control "
+       "runnable(%p)",
+       this, r.get()));
+  if (!r->Dispatch(mWorkerPrivate)) {
+    LOGV(
+        ("WorkerEventTarget::Dispatch [%p] Dispatch as control runnable(%p) "
+         "fail",
+         this, r.get()));
     return NS_ERROR_FAILURE;
   }
 
