@@ -10,7 +10,8 @@
 #include "nsAppShell.h"
 #include "nsIAndroidBridge.h"
 
-#include "mozilla/Telemetry.h"
+#include "mozilla/TimeStamp.h"
+#include "mozilla/glean/GleanMetrics.h"
 
 namespace mozilla {
 namespace widget {
@@ -21,7 +22,14 @@ class Telemetry final : public java::TelemetryUtils::Natives<Telemetry> {
  public:
   static void AddHistogram(jni::String::Param aName, int32_t aValue) {
     MOZ_ASSERT(aName);
-    mozilla::Telemetry::Accumulate(aName->ToCString().get(), aValue);
+    nsCString name = aName->ToCString();
+    if (name.EqualsLiteral("GV_STARTUP_RUNTIME_MS")) {
+      glean::geckoview::startup_runtime.AccumulateRawDuration(
+          TimeDuration::FromMilliseconds(aValue));
+    } else if (name.EqualsLiteral("GV_CONTENT_PROCESS_LIFETIME_MS")) {
+      glean::geckoview::content_process_lifetime.AccumulateRawDuration(
+          TimeDuration::FromMilliseconds(aValue));
+    }
   }
 };
 
