@@ -5,19 +5,19 @@
 
 #include <objc/objc-runtime.h>
 
+#include "nsChildView.h"
+#include "nsCocoaUtils.h"
+#include "nsCocoaWindow.h"
 #include "nsMenuBarX.h"
-#include "nsMenuX.h"
 #include "nsMenuItemX.h"
 #include "nsMenuUtilsX.h"
-#include "nsCocoaUtils.h"
-#include "nsChildView.h"
+#include "nsMenuX.h"
 
 #include "nsCOMPtr.h"
 #include "nsString.h"
 #include "nsGkAtoms.h"
 #include "nsObjCExceptions.h"
 #include "nsThreadUtils.h"
-#include "nsTouchBarNativeAPIDefines.h"
 
 #include "nsIContent.h"
 #include "nsIWidget.h"
@@ -38,15 +38,22 @@ NSMenu* sApplicationMenu = nil;
 BOOL sApplicationMenuIsFallback = NO;
 BOOL gSomeMenuBarPainted = NO;
 
+// Controls whether or not native menu items should invoke their commands. See
+// class comments for `GeckoNSMenuItem` and `GeckoNSMenu` below for an
+// explanation of why this switch is necessary.
+static BOOL gMenuItemsExecuteCommands = YES;
+
 // defined in nsCocoaWindow.mm.
 extern BOOL sTouchBarIsInitialized;
 
-// We keep references to the first quit and pref item content nodes we find, which
-// will be from the hidden window. We use these when the document for the current
-// window does not have a quit or pref item. We don't need strong refs here because
-// these items are always strong ref'd by their owning menu bar (instance variable).
+// We keep references to the first quit and pref item content nodes we find,
+// which will be from the hidden window. We use these when the document for the
+// current window does not have a quit or pref item. We don't need strong refs
+// here because these items are always strong ref'd by their owning menu bar
+// (instance variable).
 static nsIContent* sAboutItemContent = nullptr;
 static nsIContent* sPrefItemContent = nullptr;
+static nsIContent* sAccountItemContent = nullptr;
 static nsIContent* sQuitItemContent = nullptr;
 
 //
@@ -76,6 +83,7 @@ nsMenuBarX::nsMenuBarX(mozilla::dom::Element* aElement)
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK;
 
   mMenuGroupOwner = new nsMenuGroupOwnerX(aElement, this);
+  mMenuGroupOwner->RegisterForLocaleChanges();
   mNativeMenu = [[GeckoNSMenu alloc] initWithTitle:@"MainMenuBar"];
 
   mContent = aElement;
@@ -109,6 +117,11 @@ nsMenuBarX::~nsMenuBarX() {
   if (sPrefItemContent == mPrefItemContent) {
     sPrefItemContent = nullptr;
   }
+  if (sAccountItemContent == mAccountItemContent) {
+    sAccountItemContent = nullptr;
+  }
+
+  mMenuGroupOwner->UnregisterForLocaleChanges();
 
   // make sure we unregister ourselves as a content observer
   if (mContent) {
@@ -133,8 +146,9 @@ void nsMenuBarX::ConstructNativeMenus() {
   for (nsIContent* menuContent = mContent->GetFirstChild(); menuContent;
        menuContent = menuContent->GetNextSibling()) {
     if (menuContent->IsXULElement(nsGkAtoms::menu)) {
-      InsertMenuAtIndex(MakeRefPtr<nsMenuX>(this, mMenuGroupOwner, menuContent->AsElement()),
-                        GetMenuCount());
+      InsertMenuAtIndex(
+          MakeRefPtr<nsMenuX>(this, mMenuGroupOwner, menuContent->AsElement()),
+          GetMenuCount());
     }
   }
 }
@@ -149,7 +163,8 @@ void nsMenuBarX::ConstructFallbackNativeMenus() {
 
   nsCOMPtr<nsIStringBundle> stringBundle;
 
-  nsCOMPtr<nsIStringBundleService> bundleSvc = do_GetService(NS_STRINGBUNDLE_CONTRACTID);
+  nsCOMPtr<nsIStringBundleService> bundleSvc =
+      do_GetService(NS_STRINGBUNDLE_CONTRACTID);
   bundleSvc->CreateBundle("chrome://global/locale/fallbackMenubar.properties",
                           getter_AddRefs(stringBundle));
 
@@ -166,8 +181,10 @@ void nsMenuBarX::ConstructFallbackNativeMenus() {
   stringBundle->GetStringFromName(labelProp, labelUTF16);
   stringBundle->GetStringFromName(keyProp, keyUTF16);
 
-  NSString* labelStr = [NSString stringWithUTF8String:NS_ConvertUTF16toUTF8(labelUTF16).get()];
-  NSString* keyStr = [NSString stringWithUTF8String:NS_ConvertUTF16toUTF8(keyUTF16).get()];
+  NSString* labelStr =
+      [NSString stringWithUTF8String:NS_ConvertUTF16toUTF8(labelUTF16).get()];
+  NSString* keyStr =
+      [NSString stringWithUTF8String:NS_ConvertUTF16toUTF8(keyUTF16).get()];
 
   if (!nsMenuBarX::sNativeEventTarget) {
     nsMenuBarX::sNativeEventTarget = [[NativeMenuItemTarget alloc] init];
@@ -175,12 +192,14 @@ void nsMenuBarX::ConstructFallbackNativeMenus() {
 
   sApplicationMenu = [[[[NSApp mainMenu] itemAtIndex:0] submenu] retain];
   if (!mApplicationMenuDelegate) {
-    mApplicationMenuDelegate = [[ApplicationMenuDelegate alloc] initWithApplicationMenu:this];
+    mApplicationMenuDelegate =
+        [[ApplicationMenuDelegate alloc] initWithApplicationMenu:this];
   }
   sApplicationMenu.delegate = mApplicationMenuDelegate;
-  NSMenuItem* quitMenuItem = [[[NSMenuItem alloc] initWithTitle:labelStr
-                                                         action:@selector(menuItemHit:)
-                                                  keyEquivalent:keyStr] autorelease];
+  NSMenuItem* quitMenuItem =
+      [[[GeckoNSMenuItem alloc] initWithTitle:labelStr
+                                       action:@selector(menuItemHit:)
+                                keyEquivalent:keyStr] autorelease];
   quitMenuItem.target = nsMenuBarX::sNativeEventTarget;
   quitMenuItem.tag = eCommand_ID_Quit;
   [sApplicationMenu addItem:quitMenuItem];
@@ -194,7 +213,8 @@ uint32_t nsMenuBarX::GetMenuCount() { return mMenuArray.Length(); }
 bool nsMenuBarX::MenuContainsAppMenu() {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK;
 
-  return (mNativeMenu.numberOfItems > 0 && [mNativeMenu itemAtIndex:0].submenu == sApplicationMenu);
+  return (mNativeMenu.numberOfItems > 0 &&
+          [mNativeMenu itemAtIndex:0].submenu == sApplicationMenu);
 
   NS_OBJC_END_TRY_ABORT_BLOCK;
 }
@@ -213,8 +233,9 @@ void nsMenuBarX::InsertMenuAtIndex(RefPtr<nsMenuX>&& aMenu, uint32_t aIndex) {
 
     // Hook the new Application menu up to the menu bar.
     NSMenu* mainMenu = NSApp.mainMenu;
-    NS_ASSERTION(mainMenu.numberOfItems > 0,
-                 "Main menu does not have any items, something is terribly wrong!");
+    NS_ASSERTION(
+        mainMenu.numberOfItems > 0,
+        "Main menu does not have any items, something is terribly wrong!");
     [mainMenu itemAtIndex:0].submenu = sApplicationMenu;
   }
 
@@ -223,7 +244,8 @@ void nsMenuBarX::InsertMenuAtIndex(RefPtr<nsMenuX>&& aMenu, uint32_t aIndex) {
 
   // hook up submenus
   RefPtr<nsIContent> menuContent = aMenu->Content();
-  if (menuContent->GetChildCount() > 0 && !nsMenuUtilsX::NodeIsHiddenOrCollapsed(menuContent)) {
+  if (menuContent->GetChildCount() > 0 &&
+      !nsMenuUtilsX::NodeIsHiddenOrCollapsed(menuContent)) {
     MenuChildChangedVisibility(MenuChild(aMenu), true);
   }
 
@@ -256,28 +278,34 @@ void nsMenuBarX::RemoveMenuAtIndex(uint32_t aIndex) {
   NS_OBJC_END_TRY_ABORT_BLOCK;
 }
 
-void nsMenuBarX::ObserveAttributeChanged(mozilla::dom::Document* aDocument, nsIContent* aContent,
+void nsMenuBarX::ObserveAttributeChanged(mozilla::dom::Document* aDocument,
+                                         nsIContent* aContent,
                                          nsAtom* aAttribute) {}
 
-void nsMenuBarX::ObserveContentRemoved(mozilla::dom::Document* aDocument, nsIContent* aContainer,
-                                       nsIContent* aChild, nsIContent* aPreviousSibling) {
+void nsMenuBarX::ObserveContentRemoved(mozilla::dom::Document* aDocument,
+                                       nsIContent* aContainer,
+                                       nsIContent* aChild,
+                                       nsIContent* aPreviousSibling) {
   nsINode* parent = NODE_FROM(aContainer, aDocument);
   MOZ_ASSERT(parent);
-  int32_t index = parent->ComputeIndexOf(aPreviousSibling) + 1;
-  RemoveMenuAtIndex(index);
+  const Maybe<uint32_t> index = parent->ComputeIndexOf(aPreviousSibling);
+  MOZ_ASSERT(*index != UINT32_MAX);
+  RemoveMenuAtIndex(index.isSome() ? *index + 1u : 0u);
 }
 
-void nsMenuBarX::ObserveContentInserted(mozilla::dom::Document* aDocument, nsIContent* aContainer,
+void nsMenuBarX::ObserveContentInserted(mozilla::dom::Document* aDocument,
+                                        nsIContent* aContainer,
                                         nsIContent* aChild) {
   InsertMenuAtIndex(MakeRefPtr<nsMenuX>(this, mMenuGroupOwner, aChild),
-                    aContainer->ComputeIndexOf(aChild));
+                    aContainer->ComputeIndexOf(aChild).valueOr(UINT32_MAX));
 }
 
 void nsMenuBarX::ForceUpdateNativeMenuAt(const nsAString& aIndexString) {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK;
 
   NSString* locationString =
-      [NSString stringWithCharacters:reinterpret_cast<const unichar*>(aIndexString.BeginReading())
+      [NSString stringWithCharacters:reinterpret_cast<const unichar*>(
+                                         aIndexString.BeginReading())
                               length:aIndexString.Length()];
   NSArray* indexes = [locationString componentsSeparatedByString:@"|"];
   unsigned int indexCount = indexes.count;
@@ -321,7 +349,9 @@ void nsMenuBarX::ForceUpdateNativeMenuAt(const nsAString& aIndexString) {
       }
       RefPtr<nsIContent> content = targetMenu->match(
           [](const RefPtr<nsMenuX>& aMenu) { return aMenu->Content(); },
-          [](const RefPtr<nsMenuItemX>& aMenuItem) { return aMenuItem->Content(); });
+          [](const RefPtr<nsMenuItemX>& aMenuItem) {
+            return aMenuItem->Content();
+          });
       if (!nsMenuUtilsX::NodeIsHiddenOrCollapsed(content)) {
         visible++;
         if (targetMenu->is<RefPtr<nsMenuX>>() && visible == (targetIndex + 1)) {
@@ -390,6 +420,46 @@ void nsMenuBarX::SetSystemHelpMenu() {
   NS_OBJC_END_TRY_ABORT_BLOCK;
 }
 
+// macOS is adding some (currently 3) hidden menu items every time that
+// `NSApp.mainMenu` is set, but never removes them. This ultimately causes a
+// significant slowdown when switching between windows because the number of
+// items in `NSApp.mainMenu` is growing without bounds.
+//
+// The known hidden, problematic menu items are associated with the following
+// menus:
+//   - Start Dictation...
+//   - Emoji & Symbols
+//
+// Removing these items before setting `NSApp.mainMenu` prevents this slowdown.
+// See bug 1808223.
+static bool RemoveProblematicMenuItems(NSMenu* aMenu) {
+  uint8_t problematicMenuItemCount = 3;
+  NSMutableArray* itemsToRemove =
+      [NSMutableArray arrayWithCapacity:problematicMenuItemCount];
+
+  for (NSInteger i = 0; i < aMenu.numberOfItems; i++) {
+    NSMenuItem* item = [aMenu itemAtIndex:i];
+
+    if (item.hidden &&
+        (item.action == @selector(startDictation:) ||
+         item.action == @selector(orderFrontCharacterPalette:))) {
+      [itemsToRemove addObject:@(i)];
+    }
+
+    if (item.hasSubmenu && RemoveProblematicMenuItems(item.submenu)) {
+      return true;
+    }
+  }
+
+  bool didRemoveItems = false;
+  for (NSNumber* index in [itemsToRemove reverseObjectEnumerator]) {
+    [aMenu removeItemAtIndex:index.integerValue];
+    didRemoveItems = true;
+  }
+
+  return didRemoveItems;
+}
+
 nsresult nsMenuBarX::Paint() {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK;
 
@@ -399,17 +469,33 @@ nsresult nsMenuBarX::Paint() {
 
   // We have to keep the same menu item for the Application menu so we keep
   // passing it along.
-  NSMenu* outgoingMenu = NSApp.mainMenu;
-  NS_ASSERTION(outgoingMenu.numberOfItems > 0,
-               "Main menu does not have any items, something is terribly wrong!");
+  NSMenu* outgoingMenu = [NSApp.mainMenu retain];
+  NS_ASSERTION(
+      outgoingMenu.numberOfItems > 0,
+      "Main menu does not have any items, something is terribly wrong!");
 
   NSMenuItem* appMenuItem = [[outgoingMenu itemAtIndex:0] retain];
   [outgoingMenu removeItemAtIndex:0];
-  [mNativeMenu insertItem:appMenuItem atIndex:0];
+  if (appMenuItem) {
+    [mNativeMenu insertItem:appMenuItem atIndex:0];
+  }
   [appMenuItem release];
+  [outgoingMenu release];
+
+  NS_OBJC_END_TRY_ABORT_BLOCK;
+  NS_OBJC_BEGIN_TRY_ABORT_BLOCK;
+
+  RemoveProblematicMenuItems(mNativeMenu);
+
+  NS_OBJC_END_TRY_ABORT_BLOCK;
+  NS_OBJC_BEGIN_TRY_ABORT_BLOCK;
 
   // Set menu bar and event target.
   NSApp.mainMenu = mNativeMenu;
+
+  NS_OBJC_END_TRY_ABORT_BLOCK;
+  NS_OBJC_BEGIN_TRY_ABORT_BLOCK;
+
   SetSystemHelpMenu();
   nsMenuBarX::sLastGeckoMenuBarPainted = this;
 
@@ -448,13 +534,15 @@ bool nsMenuBarX::PerformKeyEquivalent(NSEvent* aEvent) {
   return [mNativeMenu performSuperKeyEquivalent:aEvent];
 }
 
-void nsMenuBarX::MenuChildChangedVisibility(const MenuChild& aChild, bool aIsVisible) {
-  MOZ_RELEASE_ASSERT(aChild.is<RefPtr<nsMenuX>>(), "nsMenuBarX only has nsMenuX children");
+void nsMenuBarX::MenuChildChangedVisibility(const MenuChild& aChild,
+                                            bool aIsVisible) {
+  MOZ_RELEASE_ASSERT(aChild.is<RefPtr<nsMenuX>>(),
+                     "nsMenuBarX only has nsMenuX children");
   const RefPtr<nsMenuX>& child = aChild.as<RefPtr<nsMenuX>>();
   NSMenuItem* item = child->NativeNSMenuItem();
   if (aIsVisible) {
     NSInteger insertionPoint = CalculateNativeInsertionPoint(child);
-    [mNativeMenu insertItem:child->NativeNSMenuItem() atIndex:insertionPoint];
+    [mNativeMenu insertItem:item atIndex:insertionPoint];
   } else if ([mNativeMenu indexOfItem:item] != -1) {
     [mNativeMenu removeItem:item];
   }
@@ -467,8 +555,9 @@ NSInteger nsMenuBarX::CalculateNativeInsertionPoint(nsMenuX* aChild) {
       return insertionPoint;
     }
     // Only count items that are inside a menu.
-    // XXXmstange Not sure what would cause free-standing items. Maybe for collapsed/hidden menus?
-    // In that case, an nsMenuX::IsVisible() method would be better.
+    // XXXmstange Not sure what would cause free-standing items. Maybe for
+    // collapsed/hidden menus? In that case, an nsMenuX::IsVisible() method
+    // would be better.
     if (currMenu->NativeNSMenuItem().menu) {
       insertionPoint++;
     }
@@ -478,10 +567,12 @@ NSInteger nsMenuBarX::CalculateNativeInsertionPoint(nsMenuX* aChild) {
 
 // Hide the item in the menu by setting the 'hidden' attribute. Returns it so
 // the caller can hang onto it if they so choose.
-RefPtr<Element> nsMenuBarX::HideItem(mozilla::dom::Document* aDocument, const nsAString& aID) {
+RefPtr<Element> nsMenuBarX::HideItem(mozilla::dom::Document* aDocument,
+                                     const nsAString& aID) {
   RefPtr<Element> menuElement = aDocument->GetElementById(aID);
   if (menuElement) {
-    menuElement->SetAttr(kNameSpaceID_None, nsGkAtoms::hidden, u"true"_ns, false);
+    menuElement->SetAttr(kNameSpaceID_None, nsGkAtoms::hidden, u"true"_ns,
+                         false);
   }
   return menuElement;
 }
@@ -512,6 +603,12 @@ void nsMenuBarX::AquifyMenuBar() {
       sPrefItemContent = mPrefItemContent;
     }
 
+    // remove Account Settings item.
+    mAccountItemContent = HideItem(domDoc, u"menu_accountmgr"_ns);
+    if (!sAccountItemContent) {
+      sAccountItemContent = mAccountItemContent;
+    }
+
     // hide items that we use for the Application menu
     HideItem(domDoc, u"menu_mac_services"_ns);
     HideItem(domDoc, u"menu_mac_hide_app"_ns);
@@ -522,7 +619,8 @@ void nsMenuBarX::AquifyMenuBar() {
 }
 
 // for creating menu items destined for the Application menu
-NSMenuItem* nsMenuBarX::CreateNativeAppMenuItem(nsMenuX* aMenu, const nsAString& aNodeID,
+NSMenuItem* nsMenuBarX::CreateNativeAppMenuItem(nsMenuX* aMenu,
+                                                const nsAString& aNodeID,
                                                 SEL aAction, int aTag,
                                                 NativeMenuItemTarget* aTarget) {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK;
@@ -539,8 +637,8 @@ NSMenuItem* nsMenuBarX::CreateNativeAppMenuItem(nsMenuX* aMenu, const nsAString&
 
   // Check collapsed rather than hidden since the app menu items are always
   // hidden in AquifyMenuBar.
-  if (menuItem->AttrValueIs(kNameSpaceID_None, nsGkAtoms::collapsed, nsGkAtoms::_true,
-                            eCaseMatters)) {
+  if (menuItem->AttrValueIs(kNameSpaceID_None, nsGkAtoms::collapsed,
+                            nsGkAtoms::_true, eCaseMatters)) {
     return nil;
   }
 
@@ -561,22 +659,25 @@ NSMenuItem* nsMenuBarX::CreateNativeAppMenuItem(nsMenuX* aMenu, const nsAString&
     if (keyElement) {
       // first grab the key equivalent character
       nsAutoString keyChar(u" "_ns);
-      keyElement->GetAttr(kNameSpaceID_None, nsGkAtoms::key, keyChar);
+      keyElement->GetAttr(nsGkAtoms::key, keyChar);
       if (!keyChar.EqualsLiteral(" ")) {
-        keyEquiv = [[NSString stringWithCharacters:reinterpret_cast<const unichar*>(keyChar.get())
-                                            length:keyChar.Length()] lowercaseString];
+        keyEquiv = [[NSString
+            stringWithCharacters:reinterpret_cast<const unichar*>(keyChar.get())
+                          length:keyChar.Length()] lowercaseString];
       }
       // now grab the key equivalent modifiers
       nsAutoString modifiersStr;
-      keyElement->GetAttr(kNameSpaceID_None, nsGkAtoms::modifiers, modifiersStr);
-      uint8_t geckoModifiers = nsMenuUtilsX::GeckoModifiersForNodeAttribute(modifiersStr);
-      macKeyModifiers = nsMenuUtilsX::MacModifiersForGeckoModifiers(geckoModifiers);
+      keyElement->GetAttr(nsGkAtoms::modifiers, modifiersStr);
+      uint8_t geckoModifiers =
+          nsMenuUtilsX::GeckoModifiersForNodeAttribute(modifiersStr);
+      macKeyModifiers =
+          nsMenuUtilsX::MacModifiersForGeckoModifiers(geckoModifiers);
     }
   }
   // get the label into NSString-form
-  NSString* labelString =
-      [NSString stringWithCharacters:reinterpret_cast<const unichar*>(label.get())
-                              length:label.Length()];
+  NSString* labelString = [NSString
+      stringWithCharacters:reinterpret_cast<const unichar*>(label.get())
+                    length:label.Length()];
 
   if (!labelString) {
     labelString = @"";
@@ -586,9 +687,9 @@ NSMenuItem* nsMenuBarX::CreateNativeAppMenuItem(nsMenuX* aMenu, const nsAString&
   }
 
   // put together the actual NSMenuItem
-  NSMenuItem* newMenuItem = [[NSMenuItem alloc] initWithTitle:labelString
-                                                       action:aAction
-                                                keyEquivalent:keyEquiv];
+  NSMenuItem* newMenuItem = [[GeckoNSMenuItem alloc] initWithTitle:labelString
+                                                            action:aAction
+                                                     keyEquivalent:keyEquiv];
 
   newMenuItem.tag = aTag;
   newMenuItem.target = aTarget;
@@ -618,8 +719,10 @@ void nsMenuBarX::CreateApplicationMenu(nsMenuX* aMenu) {
     = About This App       = <- aboutName
     ========================
     = Preferences...       = <- menu_preferences
+    = Account Settings     = <- menu_accountmgr      Only on Thunderbird
     ========================
-    = Services     >       = <- menu_mac_services    <- (do not define key equivalent)
+    = Services     >       = <- menu_mac_services    <- (do not define key
+    equivalent)
     ========================
     = Hide App             = <- menu_mac_hide_app
     = Hide Others          = <- menu_mac_hide_others
@@ -631,33 +734,37 @@ void nsMenuBarX::CreateApplicationMenu(nsMenuX* aMenu) {
     ========================
 
     If any of them are ommitted from the application's DOM, we just don't add
-    them. We always add a "Quit" item, but if an app developer does not provide a
-    DOM node with the right ID for the Quit item, we add it in English. App
-    developers need only add each node with a label and a key equivalent (if they
-    want one). Other attributes are optional. Like so:
+    them. We always add a "Quit" item, but if an app developer does not provide
+    a DOM node with the right ID for the Quit item, we add it in English. App
+    developers need only add each node with a label and a key equivalent (if
+    they want one). Other attributes are optional. Like so:
 
     <menuitem id="menu_preferences"
            label="&preferencesCmdMac.label;"
              key="open_prefs_key"/>
 
-    We need to use this system for localization purposes, until we have a better way
-    to define the Application menu to be used on Mac OS X.
+    We need to use this system for localization purposes, until we have a better
+    way to define the Application menu to be used on Mac OS X.
   */
 
   if (sApplicationMenu) {
     if (!mApplicationMenuDelegate) {
-      mApplicationMenuDelegate = [[ApplicationMenuDelegate alloc] initWithApplicationMenu:this];
+      mApplicationMenuDelegate =
+          [[ApplicationMenuDelegate alloc] initWithApplicationMenu:this];
     }
     sApplicationMenu.delegate = mApplicationMenuDelegate;
 
-    // This code reads attributes we are going to care about from the DOM elements
+    // This code reads attributes we are going to care about from the DOM
+    // elements
 
     NSMenuItem* itemBeingAdded = nil;
     BOOL addAboutSeparator = FALSE;
+    BOOL addPrefsSeparator = FALSE;
 
     // Add the About menu item
-    itemBeingAdded = CreateNativeAppMenuItem(aMenu, u"aboutName"_ns, @selector(menuItemHit:),
-                                             eCommand_ID_About, nsMenuBarX::sNativeEventTarget);
+    itemBeingAdded = CreateNativeAppMenuItem(
+        aMenu, u"aboutName"_ns, @selector(menuItemHit:), eCommand_ID_About,
+        nsMenuBarX::sNativeEventTarget);
     if (itemBeingAdded) {
       [sApplicationMenu addItem:itemBeingAdded];
       [itemBeingAdded release];
@@ -672,24 +779,40 @@ void nsMenuBarX::CreateApplicationMenu(nsMenuX* aMenu) {
     }
 
     // Add the Preferences menu item
-    itemBeingAdded = CreateNativeAppMenuItem(aMenu, u"menu_preferences"_ns, @selector(menuItemHit:),
-                                             eCommand_ID_Prefs, nsMenuBarX::sNativeEventTarget);
+    itemBeingAdded = CreateNativeAppMenuItem(
+        aMenu, u"menu_preferences"_ns, @selector(menuItemHit:),
+        eCommand_ID_Prefs, nsMenuBarX::sNativeEventTarget);
     if (itemBeingAdded) {
       [sApplicationMenu addItem:itemBeingAdded];
       [itemBeingAdded release];
       itemBeingAdded = nil;
 
-      // Add separator after Preferences menu
+      addPrefsSeparator = TRUE;
+    }
+
+    // Add the Account Settings menu item. This is Thunderbird only
+    itemBeingAdded = CreateNativeAppMenuItem(
+        aMenu, u"menu_accountmgr"_ns, @selector(menuItemHit:),
+        eCommand_ID_Account, nsMenuBarX::sNativeEventTarget);
+    if (itemBeingAdded) {
+      [sApplicationMenu addItem:itemBeingAdded];
+      [itemBeingAdded release];
+      itemBeingAdded = nil;
+    }
+
+    // Add separator after Preferences menu
+    if (addPrefsSeparator) {
       [sApplicationMenu addItem:[NSMenuItem separatorItem]];
     }
 
     // Add Services menu item
-    itemBeingAdded = CreateNativeAppMenuItem(aMenu, u"menu_mac_services"_ns, nil, 0, nil);
+    itemBeingAdded =
+        CreateNativeAppMenuItem(aMenu, u"menu_mac_services"_ns, nil, 0, nil);
     if (itemBeingAdded) {
       [sApplicationMenu addItem:itemBeingAdded];
 
       // set this menu item up as the Mac OS X Services menu
-      NSMenu* servicesMenu = [[GeckoServicesNSMenu alloc] initWithTitle:@""];
+      NSMenu* servicesMenu = [[GeckoNSMenu alloc] initWithTitle:@""];
       itemBeingAdded.submenu = servicesMenu;
       NSApp.servicesMenu = servicesMenu;
 
@@ -703,9 +826,9 @@ void nsMenuBarX::CreateApplicationMenu(nsMenuX* aMenu) {
     BOOL addHideShowSeparator = FALSE;
 
     // Add menu item to hide this application
-    itemBeingAdded =
-        CreateNativeAppMenuItem(aMenu, u"menu_mac_hide_app"_ns, @selector(menuItemHit:),
-                                eCommand_ID_HideApp, nsMenuBarX::sNativeEventTarget);
+    itemBeingAdded = CreateNativeAppMenuItem(
+        aMenu, u"menu_mac_hide_app"_ns, @selector(menuItemHit:),
+        eCommand_ID_HideApp, nsMenuBarX::sNativeEventTarget);
     if (itemBeingAdded) {
       [sApplicationMenu addItem:itemBeingAdded];
       [itemBeingAdded release];
@@ -715,9 +838,9 @@ void nsMenuBarX::CreateApplicationMenu(nsMenuX* aMenu) {
     }
 
     // Add menu item to hide other applications
-    itemBeingAdded =
-        CreateNativeAppMenuItem(aMenu, u"menu_mac_hide_others"_ns, @selector(menuItemHit:),
-                                eCommand_ID_HideOthers, nsMenuBarX::sNativeEventTarget);
+    itemBeingAdded = CreateNativeAppMenuItem(
+        aMenu, u"menu_mac_hide_others"_ns, @selector(menuItemHit:),
+        eCommand_ID_HideOthers, nsMenuBarX::sNativeEventTarget);
     if (itemBeingAdded) {
       [sApplicationMenu addItem:itemBeingAdded];
       [itemBeingAdded release];
@@ -727,9 +850,9 @@ void nsMenuBarX::CreateApplicationMenu(nsMenuX* aMenu) {
     }
 
     // Add menu item to show all applications
-    itemBeingAdded =
-        CreateNativeAppMenuItem(aMenu, u"menu_mac_show_all"_ns, @selector(menuItemHit:),
-                                eCommand_ID_ShowAll, nsMenuBarX::sNativeEventTarget);
+    itemBeingAdded = CreateNativeAppMenuItem(
+        aMenu, u"menu_mac_show_all"_ns, @selector(menuItemHit:),
+        eCommand_ID_ShowAll, nsMenuBarX::sNativeEventTarget);
     if (itemBeingAdded) {
       [sApplicationMenu addItem:itemBeingAdded];
       [itemBeingAdded release];
@@ -746,9 +869,9 @@ void nsMenuBarX::CreateApplicationMenu(nsMenuX* aMenu) {
     BOOL addTouchBarSeparator = NO;
 
     // Add Touch Bar customization menu item.
-    itemBeingAdded =
-        CreateNativeAppMenuItem(aMenu, u"menu_mac_touch_bar"_ns, @selector(menuItemHit:),
-                                eCommand_ID_TouchBar, nsMenuBarX::sNativeEventTarget);
+    itemBeingAdded = CreateNativeAppMenuItem(
+        aMenu, u"menu_mac_touch_bar"_ns, @selector(menuItemHit:),
+        eCommand_ID_TouchBar, nsMenuBarX::sNativeEventTarget);
 
     if (itemBeingAdded) {
       [sApplicationMenu addItem:itemBeingAdded];
@@ -768,9 +891,9 @@ void nsMenuBarX::CreateApplicationMenu(nsMenuX* aMenu) {
     }
 
     // Add quit menu item
-    itemBeingAdded =
-        CreateNativeAppMenuItem(aMenu, u"menu_FileQuitItem"_ns, @selector(menuItemHit:),
-                                eCommand_ID_Quit, nsMenuBarX::sNativeEventTarget);
+    itemBeingAdded = CreateNativeAppMenuItem(
+        aMenu, u"menu_FileQuitItem"_ns, @selector(menuItemHit:),
+        eCommand_ID_Quit, nsMenuBarX::sNativeEventTarget);
     if (itemBeingAdded) {
       [sApplicationMenu addItem:itemBeingAdded];
       [itemBeingAdded release];
@@ -778,9 +901,10 @@ void nsMenuBarX::CreateApplicationMenu(nsMenuX* aMenu) {
     } else {
       // the current application does not have a DOM node for "Quit". Add one
       // anyway, in English.
-      NSMenuItem* defaultQuitItem = [[[NSMenuItem alloc] initWithTitle:@"Quit"
-                                                                action:@selector(menuItemHit:)
-                                                         keyEquivalent:@"q"] autorelease];
+      NSMenuItem* defaultQuitItem =
+          [[[GeckoNSMenuItem alloc] initWithTitle:@"Quit"
+                                           action:@selector(menuItemHit:)
+                                    keyEquivalent:@"q"] autorelease];
       defaultQuitItem.target = nsMenuBarX::sNativeEventTarget;
       defaultQuitItem.tag = eCommand_ID_Quit;
       [sApplicationMenu addItem:defaultQuitItem];
@@ -790,15 +914,38 @@ void nsMenuBarX::CreateApplicationMenu(nsMenuX* aMenu) {
   NS_OBJC_END_TRY_ABORT_BLOCK;
 }
 
+// Objective-C class used for menu items to allow Gecko to override their
+// standard behavior in order to stop key equivalents from firing in certain
+// instances. When gMenuItemsExecuteCommands is NO, we return a dummy target and
+// action instead of the actual target and action.
+@implementation GeckoNSMenuItem
+
+- (id)target {
+  id realTarget = super.target;
+  if (gMenuItemsExecuteCommands) {
+    return realTarget;
+  }
+  return realTarget ? self : nil;
+}
+
+- (SEL)action {
+  SEL realAction = super.action;
+  if (gMenuItemsExecuteCommands) {
+    return realAction;
+  }
+  return realAction ? @selector(_doNothing:) : nullptr;
+}
+
+- (void)_doNothing:(id)aSender {
+}
+
+@end
+
 //
 // Objective-C class used to allow us to have keyboard commands
 // look like they are doing something but actually do nothing.
 // We allow mouse actions to work normally.
 //
-
-// Controls whether or not native menu items should invoke their commands.
-static BOOL gMenuItemsExecuteCommands = YES;
-
 @implementation GeckoNSMenu
 
 // Keyboard commands should not cause menu items to invoke their
@@ -827,13 +974,19 @@ static BOOL gMenuItemsExecuteCommands = YES;
 
   NSResponder* firstResponder = keyWindow.firstResponder;
 
-  gMenuItemsExecuteCommands = NO;
+  if ([keyWindow isKindOfClass:[BaseWindow class]]) {
+    gMenuItemsExecuteCommands = NO;
+  }
+
+  NS_OBJC_BEGIN_TRY_IGNORE_BLOCK
   [super performKeyEquivalent:aEvent];
+  NS_OBJC_END_TRY_IGNORE_BLOCK
+
   gMenuItemsExecuteCommands = YES;  // return to default
 
-  // Return YES if we invoked a command and there is now no key window or we changed
-  // the first responder. In this case we do not want to propagate the event because
-  // we don't want it handled again.
+  // Return YES if we invoked a command and there is now no key window or we
+  // changed the first responder. In this case we do not want to propagate the
+  // event because we don't want it handled again.
   if (!NSApp.keyWindow || NSApp.keyWindow.firstResponder != firstResponder) {
     return YES;
   }
@@ -846,6 +999,46 @@ static BOOL gMenuItemsExecuteCommands = YES;
   return [super performKeyEquivalent:aEvent];
 }
 
+- (void)addItem:(NSMenuItem*)aNewItem {
+  [self _overrideClassOfMenuItem:aNewItem];
+  [super addItem:aNewItem];
+}
+
+- (NSMenuItem*)addItemWithTitle:(NSString*)aString
+                         action:(SEL)aSelector
+                  keyEquivalent:(NSString*)aKeyEquiv {
+  NSMenuItem* newItem = [super addItemWithTitle:aString
+                                         action:aSelector
+                                  keyEquivalent:aKeyEquiv];
+  [self _overrideClassOfMenuItem:newItem];
+  return newItem;
+}
+
+- (void)insertItem:(NSMenuItem*)aNewItem atIndex:(NSInteger)aIndex {
+  [self _overrideClassOfMenuItem:aNewItem];
+  [super insertItem:aNewItem atIndex:aIndex];
+}
+
+- (NSMenuItem*)insertItemWithTitle:(NSString*)aString
+                            action:(SEL)aSelector
+                     keyEquivalent:(NSString*)aKeyEquiv
+                           atIndex:(NSInteger)aIndex {
+  NSMenuItem* newItem = [super insertItemWithTitle:aString
+                                            action:aSelector
+                                     keyEquivalent:aKeyEquiv
+                                           atIndex:aIndex];
+  [self _overrideClassOfMenuItem:newItem];
+  return newItem;
+}
+
+- (void)_overrideClassOfMenuItem:(NSMenuItem*)aMenuItem {
+  if ([aMenuItem class] == [NSMenuItem class]) {
+    // See class comment for `GeckoNSMenuItem` above for an explanation of why
+    // we do this.
+    object_setClass(aMenuItem, [GeckoNSMenuItem class]);
+  }
+}
+
 @end
 
 //
@@ -856,9 +1049,9 @@ static BOOL gMenuItemsExecuteCommands = YES;
 
 // called when some menu item in this menu gets hit
 - (IBAction)menuItemHit:(id)aSender {
-  if (!gMenuItemsExecuteCommands) {
-    return;
-  }
+  // We should never get here when we do not want menu items to execute their
+  // commands.
+  MOZ_RELEASE_ASSERT(gMenuItemsExecuteCommands);
 
   if (![aSender isKindOfClass:[NSMenuItem class]]) {
     return;
@@ -869,7 +1062,8 @@ static BOOL gMenuItemsExecuteCommands = YES;
 
   nsMenuGroupOwnerX* menuGroupOwner = nullptr;
   nsMenuBarX* menuBar = nullptr;
-  MOZMenuItemRepresentedObject* representedObject = nativeMenuItem.representedObject;
+  MOZMenuItemRepresentedObject* representedObject =
+      nativeMenuItem.representedObject;
 
   if (representedObject) {
     menuGroupOwner = representedObject.menuGroupOwner;
@@ -885,13 +1079,15 @@ static BOOL gMenuItemsExecuteCommands = YES;
     [(MenuDelegate*)menu.delegate menu:menu willActivateItem:nativeMenuItem];
   }
 
-  // Get the modifier flags and button for this menu item activation. The menu system does not pass
-  // an NSEvent to our action selector, but we can query the current NSEvent instead. The current
-  // NSEvent can be a key event or a mouseup event, depending on how the menu item is activated.
-  NSEventModifierFlags modifierFlags = NSApp.currentEvent ? NSApp.currentEvent.modifierFlags : 0;
-  mozilla::MouseButton button = NSApp.currentEvent
-                                    ? nsCocoaUtils::ButtonForEvent(NSApp.currentEvent)
-                                    : mozilla::MouseButton::ePrimary;
+  // Get the modifier flags and button for this menu item activation. The menu
+  // system does not pass an NSEvent to our action selector, but we can query
+  // the current NSEvent instead. The current NSEvent can be a key event or a
+  // mouseup event, depending on how the menu item is activated.
+  NSEventModifierFlags modifierFlags =
+      NSApp.currentEvent ? NSApp.currentEvent.modifierFlags : 0;
+  mozilla::MouseButton button =
+      NSApp.currentEvent ? nsCocoaUtils::ButtonForEvent(NSApp.currentEvent)
+                         : mozilla::MouseButton::ePrimary;
 
   // Do special processing if this is for an app-global command.
   if (tag == eCommand_ID_About) {
@@ -910,6 +1106,14 @@ static BOOL gMenuItemsExecuteCommands = YES;
     nsMenuUtilsX::DispatchCommandTo(mostSpecificContent, modifierFlags, button);
     return;
   }
+  if (tag == eCommand_ID_Account) {
+    nsIContent* mostSpecificContent = sAccountItemContent;
+    if (menuBar && menuBar->mAccountItemContent) {
+      mostSpecificContent = menuBar->mAccountItemContent;
+    }
+    nsMenuUtilsX::DispatchCommandTo(mostSpecificContent, modifierFlags, button);
+    return;
+  }
   if (tag == eCommand_ID_HideApp) {
     [NSApp hide:aSender];
     return;
@@ -931,13 +1135,15 @@ static BOOL gMenuItemsExecuteCommands = YES;
     if (menuBar && menuBar->mQuitItemContent) {
       mostSpecificContent = menuBar->mQuitItemContent;
     }
-    // If we have some content for quit we execute it. Otherwise we send a native app terminate
-    // message. If you want to stop a quit from happening, provide quit content and return
-    // the event as unhandled.
+    // If we have some content for quit we execute it. Otherwise we send a
+    // native app terminate message. If you want to stop a quit from happening,
+    // provide quit content and return the event as unhandled.
     if (mostSpecificContent) {
-      nsMenuUtilsX::DispatchCommandTo(mostSpecificContent, modifierFlags, button);
+      nsMenuUtilsX::DispatchCommandTo(mostSpecificContent, modifierFlags,
+                                      button);
     } else {
-      nsCOMPtr<nsIAppStartup> appStartup = mozilla::components::AppStartup::Service();
+      nsCOMPtr<nsIAppStartup> appStartup =
+          mozilla::components::AppStartup::Service();
       if (appStartup) {
         bool userAllowedQuit = true;
         appStartup->Quit(nsIAppStartup::eAttemptQuit, 0, &userAllowedQuit);
@@ -949,83 +1155,16 @@ static BOOL gMenuItemsExecuteCommands = YES;
   // given the commandID, look it up in our hashtable and dispatch to
   // that menu item.
   if (menuGroupOwner) {
-    nsMenuItemX* menuItem = menuGroupOwner->GetMenuItemForCommandID(static_cast<uint32_t>(tag));
-    if (menuItem) {
-      menuItem->DoCommand(modifierFlags, button);
+    if (RefPtr<nsMenuItemX> menuItem = menuGroupOwner->GetMenuItemForCommandID(
+            static_cast<uint32_t>(tag))) {
+      if (nsMenuUtilsX::gIsSynchronouslyActivatingNativeMenuItemDuringTest) {
+        menuItem->DoCommand(modifierFlags, button);
+      } else if (RefPtr<nsMenuX> menu = menuItem->ParentMenu()) {
+        menu->ActivateItemAfterClosing(std::move(menuItem), modifierFlags,
+                                       button);
+      }
     }
   }
 }
 
 @end
-
-// Objective-C class used for menu items on the Services menu to allow Gecko
-// to override their standard behavior in order to stop key equivalents from
-// firing in certain instances. When gMenuItemsExecuteCommands is NO, we return
-// a dummy target and action instead of the actual target and action.
-
-@implementation GeckoServicesNSMenuItem
-
-- (id)target {
-  id realTarget = super.target;
-  if (gMenuItemsExecuteCommands) {
-    return realTarget;
-  }
-  return realTarget ? self : nil;
-}
-
-- (SEL)action {
-  SEL realAction = super.action;
-  if (gMenuItemsExecuteCommands) {
-    return realAction;
-  }
-  return realAction ? @selector(_doNothing:) : nullptr;
-}
-
-- (void)_doNothing:(id)aSender {
-}
-
-@end
-
-// Objective-C class used as the Services menu so that Gecko can override the
-// standard behavior of the Services menu in order to stop key equivalents
-// from firing in certain instances.
-
-@implementation GeckoServicesNSMenu
-
-- (void)addItem:(NSMenuItem*)aNewItem {
-  [self _overrideClassOfMenuItem:aNewItem];
-  [super addItem:aNewItem];
-}
-
-- (NSMenuItem*)addItemWithTitle:(NSString*)aString
-                         action:(SEL)aSelector
-                  keyEquivalent:(NSString*)aKeyEquiv {
-  NSMenuItem* newItem = [super addItemWithTitle:aString action:aSelector keyEquivalent:aKeyEquiv];
-  [self _overrideClassOfMenuItem:newItem];
-  return newItem;
-}
-
-- (void)insertItem:(NSMenuItem*)aNewItem atIndex:(NSInteger)aIndex {
-  [self _overrideClassOfMenuItem:aNewItem];
-  [super insertItem:aNewItem atIndex:aIndex];
-}
-
-- (NSMenuItem*)insertItemWithTitle:(NSString*)aString
-                            action:(SEL)aSelector
-                     keyEquivalent:(NSString*)aKeyEquiv
-                           atIndex:(NSInteger)aIndex {
-  NSMenuItem* newItem = [super insertItemWithTitle:aString
-                                            action:aSelector
-                                     keyEquivalent:aKeyEquiv
-                                           atIndex:aIndex];
-  [self _overrideClassOfMenuItem:newItem];
-  return newItem;
-}
-
-- (void)_overrideClassOfMenuItem:(NSMenuItem*)aMenuItem {
-  if ([aMenuItem class] == [NSMenuItem class]) {
-    object_setClass(aMenuItem, [GeckoServicesNSMenuItem class]);
-  }
-}
-
-@end
